@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@
 // -------------------------------------------------------------------------------- //
 #include "AlListBox.h"
 
+#include "Accelerators.h"
 #include "Commands.h"
 #include "Config.h"
 #include "Images.h"
@@ -27,6 +28,7 @@
 #include "MainFrame.h"
 #include "OnlineLinks.h"
 #include "Utils.h"
+#include "LibPanel.h"
 
 #include <wx/renderer.h>
 
@@ -35,11 +37,15 @@
 // -------------------------------------------------------------------------------- //
 // guAlListBox
 // -------------------------------------------------------------------------------- //
-guAlListBox::guAlListBox( wxWindow * parent, guDbLibrary * db, const wxString &label ) :
+guAlListBox::guAlListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * db, const wxString &label ) :
     guListView( parent, wxLB_MULTIPLE | guLISTVIEW_ALLOWDRAG | guLISTVIEW_HIDE_HEADER )
 {
     m_Db = db;
     m_Items = new guAlbumItems();
+    m_LibPanel = libpanel;
+
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->RegisterObject( this );
 
     guListViewColumn * Column = new guListViewColumn( label, 0 );
     InsertColumn( Column );
@@ -48,17 +54,70 @@ guAlListBox::guAlListBox( wxWindow * parent, guDbLibrary * db, const wxString &l
     Connect( ID_ALBUM_COMMANDS, ID_ALBUM_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnCommandClicked ) );
     Connect( ID_ALBUM_ORDER_NAME, ID_ALBUM_ORDER_ARTIST_YEAR_REVERSE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnOrderSelected ) );
 
+    Connect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guAlListBox::OnConfigUpdated ), NULL, this );
+
+    CreateAcceleratorTable();
+
     ReloadItems();
 }
 
 // -------------------------------------------------------------------------------- //
 guAlListBox::~guAlListBox()
 {
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->UnRegisterObject( this );
+
     if( m_Items )
         delete m_Items;
 
     Disconnect( ID_LASTFM_SEARCH_LINK, ID_LASTFM_SEARCH_LINK + 999, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnSearchLinkClicked ) );
     Disconnect( ID_ALBUM_COMMANDS, ID_ALBUM_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnCommandClicked ) );
+    Disconnect( ID_ALBUM_ORDER_NAME, ID_ALBUM_ORDER_ARTIST_YEAR_REVERSE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guAlListBox::OnOrderSelected ) );
+
+    Disconnect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guAlListBox::OnConfigUpdated ), NULL, this );
+}
+
+// -------------------------------------------------------------------------------- //
+void guAlListBox::OnConfigUpdated( wxCommandEvent &event )
+{
+    int Flags = event.GetInt();
+    if( Flags & guPREFERENCE_PAGE_FLAG_ACCELERATORS )
+    {
+        CreateAcceleratorTable();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guAlListBox::CreateAcceleratorTable( void )
+{
+    wxAcceleratorTable AccelTable;
+    wxArrayInt AliasAccelCmds;
+    wxArrayInt RealAccelCmds;
+
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SAVE );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITLABELS );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITTRACKS );
+    AliasAccelCmds.Add( ID_SONG_PLAY );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SEARCH );
+
+    RealAccelCmds.Add( ID_ALBUM_SAVETOPLAYLIST );
+    RealAccelCmds.Add( ID_ALBUM_EDITLABELS );
+    RealAccelCmds.Add( ID_ALBUM_EDITTRACKS );
+    RealAccelCmds.Add( ID_ALBUM_PLAY );
+    RealAccelCmds.Add( ID_ALBUM_ENQUEUE_AFTER_ALL );
+    RealAccelCmds.Add( ID_ALBUM_ENQUEUE_AFTER_TRACK );
+    RealAccelCmds.Add( ID_ALBUM_ENQUEUE_AFTER_ALBUM );
+    RealAccelCmds.Add( ID_ALBUM_ENQUEUE_AFTER_ARTIST );
+    RealAccelCmds.Add( ID_LIBRARY_SEARCH );
+
+    if( guAccelDoAcceleratorTable( AliasAccelCmds, RealAccelCmds, AccelTable ) )
+    {
+        SetAcceleratorTable( AccelTable );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -110,14 +169,14 @@ void guAlListBox::DrawItem( wxDC &dc, const wxRect &rect, const int row, const i
         dc.DrawText( _( "by " ) + Item->m_ArtistName, rect.x + 45, rect.y + 22 );
     }
 
-    if( Item->m_Thumb && Item->m_Thumb->IsOk() )
+    if( Item->m_Thumb )
     {
         dc.DrawBitmap( * Item->m_Thumb, rect.x + 1, rect.y + 1, false );
     }
-    else if( Item->m_Thumb )
-    {
-        guLogError( wxT( "Thumb image corrupt or not correctly loaded" ) );
-    }
+//    else if( Item->m_Thumb )
+//    {
+//        guLogError( wxT( "Thumb image corrupt or not correctly loaded" ) );
+//    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -140,7 +199,9 @@ wxCoord guAlListBox::OnMeasureItem( size_t n ) const
 // -------------------------------------------------------------------------------- //
 int guAlListBox::GetSelectedSongs( guTrackArray * tracks ) const
 {
-    return m_Db->GetAlbumsSongs( GetSelectedItems(), tracks );
+    int Count = m_Db->GetAlbumsSongs( GetSelectedItems(), tracks );
+    m_LibPanel->NormalizeTracks( tracks );
+    return Count;
 }
 
 // -------------------------------------------------------------------------------- //
@@ -153,7 +214,6 @@ void AddAlbumCommands( wxMenu * Menu, int SelCount )
     if( Menu )
     {
         SubMenu = new wxMenu();
-        wxASSERT( SubMenu );
 
         guConfig * Config = ( guConfig * ) guConfig::Get();
         wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
@@ -182,67 +242,67 @@ void AddAlbumCommands( wxMenu * Menu, int SelCount )
 void guAlListBox::CreateContextMenu( wxMenu * Menu ) const
 {
     wxMenuItem * MenuItem;
-
-    MenuItem = new wxMenuItem( Menu, ID_ALBUM_PLAY, _( "Play" ), _( "Play current selected albums" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
-    Menu->Append( MenuItem );
-
-    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ENQUEUE, _( "Enqueue" ), _( "Add current selected albums to the Playlist" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-    Menu->Append( MenuItem );
-
-    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected albums to the Playlist as Next Tracks" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-    Menu->Append( MenuItem );
+    int ContextMenuFlags = m_LibPanel->GetContextMenuFlags();
 
     int SelCount = GetSelectedCount();
     if( SelCount )
     {
+        MenuItem = new wxMenuItem( Menu, ID_ALBUM_PLAY,
+                                wxString( _( "Play" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_PLAY ),
+                                _( "Play current selected albums" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
+        Menu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( Menu, ID_ALBUM_ENQUEUE_AFTER_ALL,
+                                wxString( _( "Enqueue" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALL ),
+                                _( "Add current selected albums to the Playlist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        Menu->Append( MenuItem );
+
+        wxMenu * EnqueueMenu = new wxMenu();
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_ALBUM_ENQUEUE_AFTER_TRACK,
+                                wxString( _( "Current Track" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_TRACK ),
+                                _( "Add current selected albums to the Playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_ALBUM_ENQUEUE_AFTER_ALBUM,
+                                wxString( _( "Current Album" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALBUM ),
+                                _( "Add current selected albums to the Playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_ALBUM_ENQUEUE_AFTER_ARTIST,
+                                wxString( _( "Current Artist" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ARTIST ),
+                                _( "Add current selected albums to the Playlist as Next Tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        Menu->Append( wxID_ANY, _( "Enqueue after" ), EnqueueMenu, _( "Add the selected albums after" ) );
+
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_ALBUM_EDITLABELS, _( "Edit Labels" ), _( "Edit the labels assigned to the selected albums" ) );
+        MenuItem = new wxMenuItem( Menu, ID_ALBUM_EDITLABELS,
+                                wxString( _( "Edit Labels" ) ) +  guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITLABELS ),
+                                _( "Edit the labels assigned to the selected albums" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tags ) );
         Menu->Append( MenuItem );
 
-        MenuItem = new wxMenuItem( Menu, ID_ALBUM_EDITTRACKS, _( "Edit Album songs" ), _( "Edit the selected albums songs" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit ) );
-        Menu->Append( MenuItem );
-
-        if( SelCount == 1 )
+        if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_EDIT_TRACKS )
         {
-            Menu->AppendSeparator();
-
-            MenuItem = new wxMenuItem( Menu, ID_ALBUM_MANUALCOVER, _( "Download Album cover" ), _( "Download cover for the current selected album" ) );
-            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_download_covers ) );
-            Menu->Append( MenuItem );
-
-            MenuItem = new wxMenuItem( Menu, ID_ALBUM_SELECT_COVER, _( "Select cover location" ), _( "Select the cover image file from disk" ) );
-            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_download_covers ) );
-            Menu->Append( MenuItem );
-
-            MenuItem = new wxMenuItem( Menu, ID_ALBUM_COVER_DELETE, _( "Delete Album cover" ), _( "Delete the cover for the selected album" ) );
-            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_delete ) );
+            MenuItem = new wxMenuItem( Menu, ID_ALBUM_EDITTRACKS,
+                                wxString( _( "Edit Album songs" ) ) + guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITTRACKS ),
+                                _( "Edit the selected albums songs" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
             Menu->Append( MenuItem );
         }
+
+        Menu->AppendSeparator();
     }
 
-    Menu->AppendSeparator();
-
     wxMenu * SubMenu = new wxMenu();
 
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_NAME, _( "Name" ), _( "Albums are sorted by name" ) );
-//    SubMenu->Append( MenuItem );
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_YEAR, _( "Year" ), _( "Albums are sorted by year" ) );
-//    SubMenu->Append( MenuItem );
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_YEAR_REVERSE, _( "Year descending" ), _( "Albums are sorted by year descending" ) );
-//    SubMenu->Append( MenuItem );
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_ARTIST_NAME, _( "Artist, Name" ), _( "Albums are sorted by artist and album name" ) );
-//    SubMenu->Append( MenuItem );
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_ARTIST_YEAR, _( "Artist, Year" ), _( "Albums are sorted by artist and year" ) );
-//    SubMenu->Append( MenuItem );
-//    MenuItem = new wxMenuItem( Menu, ID_ALBUM_ORDER_ARTIST_YEAR_REVERSE, _( "Artist, Year descending" ), _( "Albums are sorted by artist and year descending" ) );
-//    SubMenu->Append( MenuItem );
-
     SubMenu->AppendRadioItem( ID_ALBUM_ORDER_NAME, _( "Name" ), _( "Albums are sorted by name" ) );
     SubMenu->AppendRadioItem( ID_ALBUM_ORDER_YEAR, _( "Year" ), _( "Albums are sorted by year" ) );
     SubMenu->AppendRadioItem( ID_ALBUM_ORDER_YEAR_REVERSE, _( "Year descending" ), _( "Albums are sorted by year descending" ) );
@@ -259,18 +319,58 @@ void guAlListBox::CreateContextMenu( wxMenu * Menu ) const
     {
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_ALBUM_COPYTO, _( "Copy to..." ), _( "Copy the current selected songs to a directory or device" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
+        MenuItem = new wxMenuItem( Menu, ID_ALBUM_SAVETOPLAYLIST,
+                                wxString( _( "Save to PlayList" ) ) +  guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_SAVE ),
+                                _( "Save the selected tracks to PlayList" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_doc_save ) );
         Menu->Append( MenuItem );
 
-        Menu->AppendSeparator();
-        if( SelCount == 1 )
+        if( SelCount == 1 && ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_DOWNLOAD_COVERS ) )
         {
-            AddOnlineLinksMenu( Menu );
+            Menu->AppendSeparator();
+
+            MenuItem = new wxMenuItem( Menu, ID_ALBUM_MANUALCOVER, _( "Download Album cover" ), _( "Download cover for the current selected album" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_download_covers ) );
+            Menu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( Menu, ID_ALBUM_SELECT_COVER, _( "Select cover location" ), _( "Select the cover image file from disk" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_download_covers ) );
+            Menu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( Menu, ID_ALBUM_COVER_DELETE, _( "Delete Album cover" ), _( "Delete the cover for the selected album" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
+            Menu->Append( MenuItem );
         }
 
-        AddAlbumCommands( Menu, SelCount );
+        if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_EMBED_COVERS )
+        {
+            MenuItem = new wxMenuItem( Menu, ID_ALBUM_COVER_EMBED, _( "Embed cover" ), _( "Embed the current cover to the album files" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_doc_save ) );
+            Menu->Append( MenuItem );
+        }
+
+        if( ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COPY_TO ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS ) )
+        {
+            Menu->AppendSeparator();
+
+            if( ( m_LibPanel->GetContextMenuFlags() & guLIBRARY_CONTEXTMENU_COPY_TO ) )
+            {
+                m_LibPanel->CreateCopyToMenu( Menu, ID_ALBUM_COPYTO );
+            }
+
+            if( SelCount == 1 && ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) )
+            {
+                AddOnlineLinksMenu( Menu );
+            }
+
+            if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS )
+                AddAlbumCommands( Menu, SelCount );
+        }
     }
+
+    m_LibPanel->CreateContextMenu( Menu, guLIBRARY_ELEMENT_ALBUMS );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -287,7 +387,6 @@ void guAlListBox::OnSearchLinkClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Links = Config->ReadAStr( wxT( "Link" ), wxEmptyString, wxT( "SearchLinks" ) );
-            wxASSERT( Links.Count() > 0 );
 
             index -= ID_LASTFM_SEARCH_LINK;
             wxString SearchLink = Links[ index ];
@@ -318,10 +417,11 @@ void guAlListBox::OnCommandClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
-            wxASSERT( Commands.Count() > 0 );
 
+            //guLogMessage( wxT( "CommandId: %u" ), index );
             index -= ID_ALBUM_COMMANDS;
             wxString CurCmd = Commands[ index ];
+
             if( CurCmd.Find( wxT( "{bp}" ) ) != wxNOT_FOUND )
             {
                 wxArrayString AlbumPaths = m_Db->GetAlbumsPaths( Selection );
@@ -376,7 +476,6 @@ wxString guAlListBox::GetSearchText( int item ) const
         ( * m_Items )[ item ].m_Name.c_str() );
 }
 
-
 // -------------------------------------------------------------------------------- //
 wxString inline guAlListBox::GetItemName( const int row ) const
 {
@@ -437,13 +536,13 @@ int guAlListBox::GetDragFiles( wxFileDataObject * files )
     guTrackArray Songs;
     int index;
     int count = GetSelectedSongs( &Songs );
+    m_LibPanel->NormalizeTracks( &Songs, true );
     for( index = 0; index < count; index++ )
     {
        wxString FileName = guFileDnDEncode( Songs[ index ].m_FileName );
        //FileName.Replace( wxT( "#" ), wxT( "%23" ) );
 
        //FileName.Replace( wxT( "%" ), wxT( "%25" ) );
-       //guLogMessage( wxT( "Adding song '%s'" ), FileName.c_str() );
        files->AddFile( FileName );
     }
     return count;
