@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -24,12 +24,15 @@
 #include "DbLibrary.h"
 #include "ItemListBox.h"
 
+class guLibPanel;
+
 // -------------------------------------------------------------------------------- //
 class guAlListBox : public  guListView
 {
   private :
     guDbLibrary *       m_Db;
     guAlbumItems *      m_Items;
+    guLibPanel *        m_LibPanel;
 
     void                OnAlbumListActivated( wxListEvent &event );
     void                OnAlbumListSelected( wxListEvent &event );
@@ -45,8 +48,11 @@ class guAlListBox : public  guListView
 
     virtual int         GetDragFiles( wxFileDataObject * files );
 
+    void                OnConfigUpdated( wxCommandEvent &event );
+    void                CreateAcceleratorTable();
+
   public :
-                        guAlListBox( wxWindow * parent, guDbLibrary * db, const wxString &label );
+                        guAlListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * db, const wxString &label );
                         ~guAlListBox();
     bool                SelectAlbumName( const wxString &AlbumName );
 //    void                ReloadItems( const bool reset = true );
