@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 #ifndef ALBUMBROWSER_H
 #define ALBUMBROWSER_H
 
+#include "AutoScrollText.h"
 #include "DbLibrary.h"
 #include "DynamicPlayList.h"
 #include "PlayerPanel.h"
@@ -84,17 +85,19 @@ class guAlbumBrowserItemPanel : public wxPanel
 
     // GUI
     wxBoxSizer *            m_MainSizer;
-    wxStaticBitmap *        m_Bitmap;
-    wxStaticText *          m_ArtistLabel;
-    wxStaticText *          m_AlbumLabel;
-    wxStaticText *          m_TracksLabel;
+    guStaticBitmap *        m_Bitmap;
+    //wxStaticText *          m_ArtistLabel;
+    guAutoScrollText *      m_ArtistLabel;
+    //wxStaticText *          m_AlbumLabel;
+    guAutoScrollText *      m_AlbumLabel;
+    //wxStaticText *          m_TracksLabel;
+    guAutoScrollText *      m_TracksLabel;
 
     void                    OnContextMenu( wxContextMenuEvent &event );
 
     void                    OnSearchLinkClicked( wxCommandEvent &event );
     void                    OnPlayClicked( wxCommandEvent &event );
     void                    OnEnqueueClicked( wxCommandEvent &event );
-    void                    OnEnqueueAsNextClicked( wxCommandEvent &event );
     void                    OnCopyToClipboard( wxCommandEvent &event );
     void                    OnAlbumSelectName( wxCommandEvent &event );
     void                    OnArtistSelectName( wxCommandEvent &event );
@@ -102,6 +105,7 @@ class guAlbumBrowserItemPanel : public wxPanel
     void                    OnAlbumDownloadCoverClicked( wxCommandEvent &event );
     void                    OnAlbumSelectCoverClicked( wxCommandEvent &event );
     void                    OnAlbumDeleteCoverClicked( wxCommandEvent &event );
+    void                    OnAlbumEmbedCoverClicked( wxCommandEvent &event );
     void                    OnAlbumCopyToClicked( wxCommandEvent &event );
     void                    OnAlbumEditLabelsClicked( wxCommandEvent &event );
     void                    OnAlbumEditTracksClicked( wxCommandEvent &event );
@@ -111,6 +115,8 @@ class guAlbumBrowserItemPanel : public wxPanel
     void                    OnBeginDrag( wxMouseEvent &event );
     void                    OnCoverBeginDrag( wxMouseEvent &event );
 
+    void                    OnBitmapMouseOver( wxCommandEvent &event );
+
   public :
     guAlbumBrowserItemPanel( wxWindow * parent, const int index, guAlbumBrowserItem * albumitem = NULL );
     ~guAlbumBrowserItemPanel();
@@ -143,12 +149,17 @@ class guAlbumBrowser : public wxPanel
     unsigned int                    m_PagesCount;       // The number of pages to scroll
     wxBitmap *                      m_BlankCD;
     wxTimer                         m_RefreshTimer;
+    wxArrayString                   m_DynFilterArray;
     guDynPlayList                   m_DynFilter;
+    int                             m_ContextMenuFlags;
 
 
     // GUI
-    wxToggleButton *                m_FilterBtn;
-    wxBitmapButton *                m_EditFilterBtn;
+    wxBoxSizer *                    m_FilterSizer;
+    wxChoice *                      m_FilterChoice;
+    wxBitmapButton *                m_AddFilterButton;
+    wxBitmapButton *                m_DelFilterButton;
+    wxBitmapButton *                m_EditFilterButton;
     wxChoice *                      m_OrderChoice;
 	wxGridSizer *                   m_AlbumsSizer;
 	wxStaticText *                  m_NavLabel;
@@ -158,12 +169,18 @@ class guAlbumBrowser : public wxPanel
     wxTimer                         m_TextChangedTimer;
     wxArrayString                   m_TextSearchFilter;
 
+    bool                            m_InstantSearchEnabled;
+    bool                            m_EnterSelectSearchEnabled;
+
     void                            OnChangedSize( wxSizeEvent &event );
     void                            OnChangingPosition( wxScrollEvent& event );
     void                            OnRefreshTimer( wxTimerEvent &event );
 
     void                            OnFilterSelected( wxCommandEvent &event );
+    void                            OnAddFilterClicked( wxCommandEvent &event );
+    void                            OnDelFilterClicked( wxCommandEvent &event );
     void                            OnEditFilterClicked( wxCommandEvent &event );
+
     void                            OnOrderSelected( wxCommandEvent &event );
     void                            OnUpdateDetails( wxCommandEvent &event );
 
@@ -175,13 +192,31 @@ class guAlbumBrowser : public wxPanel
     void                            OnSearchSelected( wxCommandEvent &event );
     void                            OnTextChangedTimer( wxTimerEvent &event );
 
+    virtual void                    NormalizeTracks( guTrackArray * tracks, const bool isdrag = false ) {};
+
+    virtual void                    OnBitmapMouseOver( const int coverid, const wxPoint &position );
+
+    virtual void                    OnAlbumSelectName( const int albumid );
+    virtual void                    OnArtistSelectName( const int artistid );
+
+    void                            OnGoToSearch( wxCommandEvent &event );
+    void                            OnConfigUpdated( wxCommandEvent &event );
+    void                            CreateAcceleratorTable( void );
+
+    bool                            DoTextSearch( void );
+
   public :
     guAlbumBrowser( wxWindow * parent, guDbLibrary * db, guPlayerPanel * playerpanel );
     ~guAlbumBrowser();
 
+    virtual int                     GetContextMenuFlags( void ) { return m_ContextMenuFlags; }
+    virtual void                    SetContextMenuFlags( const int flags ) { m_ContextMenuFlags = flags; }
+
+    virtual void                    CreateContextMenu( wxMenu * Menu ) {};
+
     void RefreshCount( void )
     {
-        m_AlbumsCount = m_Db->GetAlbumsCount( m_FilterBtn->GetValue() ? &m_DynFilter : NULL, m_TextSearchFilter );
+        m_AlbumsCount = m_Db->GetAlbumsCount( m_FilterChoice->GetSelection() ? &m_DynFilter : NULL, m_TextSearchFilter );
         m_ItemStart = 0;
         RefreshPageCount();
     }
@@ -219,20 +254,18 @@ class guAlbumBrowser : public wxPanel
     void                            ReloadItems( void );
     void                            RefreshAll( void );
 
-    void UpdateNavLabel( const int page )
-    {
-        m_NavLabel->SetLabel( _( "Page" ) + wxString::Format( wxT( " %i / %i" ), page + 1, m_PagesCount ) );
-    }
+    void                            UpdateNavLabel( const int page );
 
-    void                            SelectAlbum( const int albumid, const bool append, const bool asnext = false );
-    int                             GetAlbumTracks( const int albumid, guTrackArray * tracks );
-    void                            OnCommandClicked( const int commandid, const int albumid );
-    void                            OnAlbumDownloadCoverClicked( const int albumid );
-    void                            OnAlbumSelectCoverClicked( const int albumid );
-    void                            OnAlbumDeleteCoverClicked( const int albumid );
-    void                            OnAlbumCopyToClicked( const int albumid );
-    void                            OnAlbumEditLabelsClicked( const guAlbumBrowserItem * albumitem );
-    void                            OnAlbumEditTracksClicked( const int albumid );
+    virtual void                    SelectAlbum( const int albumid, const bool append, const int aftercurrent = 0 );
+    virtual int                     GetAlbumTracks( const int albumid, guTrackArray * tracks );
+    virtual void                    OnCommandClicked( const int commandid, const int albumid );
+    virtual void                    OnAlbumDownloadCoverClicked( const int albumid );
+    virtual void                    OnAlbumSelectCoverClicked( const int albumid );
+    virtual void                    OnAlbumDeleteCoverClicked( const int albumid );
+    virtual void                    OnAlbumEmbedCoverClicked( const int albumid );
+    virtual void                    OnAlbumCopyToClicked( const int albumid, const int commandid );
+    virtual void                    OnAlbumEditLabelsClicked( const guAlbumBrowserItem * albumitem );
+    virtual void                    OnAlbumEditTracksClicked( const int albumid );
 
     void                            LibraryUpdated( void );
 
@@ -242,6 +275,7 @@ class guAlbumBrowser : public wxPanel
 
 
     friend class guUpdateAlbumDetails;
+    friend class guAlbumBrowserItemPanel;
 };
 
 // -------------------------------------------------------------------------------- //
