@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -19,19 +19,27 @@
 //
 // -------------------------------------------------------------------------------- //
 #include "ArListBox.h"
+
+#include "Accelerators.h"
 #include "Commands.h"
 #include "Config.h"
 #include "Images.h"
 #include "OnlineLinks.h"
 #include "MainApp.h"
 #include "Utils.h"
+#include "LibPanel.h"
 
 // -------------------------------------------------------------------------------- //
-guArListBox::guArListBox( wxWindow * parent, guDbLibrary * db, const wxString &label ) :
-             guListBox( parent, db, label, wxLB_MULTIPLE | guLISTVIEW_ALLOWDRAG | guLISTVIEW_HIDE_HEADER )
+guArListBox::guArListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * db, const wxString &label ) :
+             guAccelListBox( parent, db, label )
 {
+    m_LibPanel = libpanel;
+
     Connect( ID_LASTFM_SEARCH_LINK, ID_LASTFM_SEARCH_LINK + 999, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guArListBox::OnSearchLinkClicked ) );
     Connect( ID_ARTIST_COMMANDS, ID_ARTIST_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guArListBox::OnCommandClicked ) );
+
+    CreateAcceleratorTable();
+
     ReloadItems();
 };
 
@@ -42,6 +50,39 @@ guArListBox::~guArListBox()
     Disconnect( ID_ARTIST_COMMANDS, ID_ARTIST_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guArListBox::OnCommandClicked ) );
 }
 
+// -------------------------------------------------------------------------------- //
+void guArListBox::CreateAcceleratorTable( void )
+{
+    wxAcceleratorTable AccelTable;
+    wxArrayInt AliasAccelCmds;
+    wxArrayInt RealAccelCmds;
+
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SAVE );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITLABELS );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITTRACKS );
+    AliasAccelCmds.Add( ID_SONG_PLAY );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SEARCH );
+
+    RealAccelCmds.Add( ID_ARTIST_SAVETOPLAYLIST );
+    RealAccelCmds.Add( ID_ARTIST_EDITLABELS );
+    RealAccelCmds.Add( ID_ARTIST_EDITTRACKS );
+    RealAccelCmds.Add( ID_ARTIST_PLAY );
+    RealAccelCmds.Add( ID_ARTIST_ENQUEUE_AFTER_ALL );
+    RealAccelCmds.Add( ID_ARTIST_ENQUEUE_AFTER_TRACK );
+    RealAccelCmds.Add( ID_ARTIST_ENQUEUE_AFTER_ALBUM );
+    RealAccelCmds.Add( ID_ARTIST_ENQUEUE_AFTER_ARTIST );
+    RealAccelCmds.Add( ID_LIBRARY_SEARCH );
+
+    if( guAccelDoAcceleratorTable( AliasAccelCmds, RealAccelCmds, AccelTable ) )
+    {
+        SetAcceleratorTable( AccelTable );
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 void guArListBox::GetItemsList( void )
 {
@@ -51,7 +92,9 @@ void guArListBox::GetItemsList( void )
 // -------------------------------------------------------------------------------- //
 int guArListBox::GetSelectedSongs( guTrackArray * songs ) const
 {
-    return m_Db->GetArtistsSongs( GetSelectedItems(), songs );
+    int Count = m_Db->GetArtistsSongs( GetSelectedItems(), songs );
+    m_LibPanel->NormalizeTracks( songs );
+    return Count;
 }
 
 // -------------------------------------------------------------------------------- //
@@ -64,7 +107,6 @@ void AddArtistCommands( wxMenu * Menu, int SelCount )
     if( Menu )
     {
         SubMenu = new wxMenu();
-        wxASSERT( SubMenu );
 
         guConfig * Config = ( guConfig * ) guConfig::Get();
         wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
@@ -95,45 +137,96 @@ void guArListBox::CreateContextMenu( wxMenu * Menu ) const
     wxMenuItem * MenuItem;
 
     int SelCount = GetSelectedCount();
+    int ContextMenuFlags = m_LibPanel->GetContextMenuFlags();
 
-    MenuItem = new wxMenuItem( Menu, ID_ARTIST_PLAY, _( "Play" ), _( "Play current selected artists" ) );
+    MenuItem = new wxMenuItem( Menu, ID_ARTIST_PLAY,
+                            wxString( _( "Play" ) ) + guAccelGetCommandKeyCodeString( ID_SONG_PLAY ),
+                            _( "Play current selected artists" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
     Menu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
 
-    MenuItem = new wxMenuItem( Menu, ID_ARTIST_ENQUEUE, _( "Enqueue" ), _( "Add current selected artists to playlist" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
+    MenuItem = new wxMenuItem( Menu, ID_ARTIST_ENQUEUE_AFTER_ALL,
+                            wxString( _( "Enqueue" ) ) + guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALL ),
+                            _( "Add current selected artists to playlist" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
     Menu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
 
-    MenuItem = new wxMenuItem( Menu, ID_ARTIST_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected artists to playlist as Next Tracks" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-    Menu->Append( MenuItem );
+    wxMenu * EnqueueMenu = new wxMenu();
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_ARTIST_ENQUEUE_AFTER_TRACK,
+                            wxString( _( "Current Track" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_TRACK ),
+                            _( "Add current selected tracks to playlist after the current track" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_ARTIST_ENQUEUE_AFTER_ALBUM,
+                            wxString( _( "Current Album" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALBUM ),
+                            _( "Add current selected tracks to playlist after the current album" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_ARTIST_ENQUEUE_AFTER_ARTIST,
+                            wxString( _( "Current Artist" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_TRACK ),
+                            _( "Add current selected tracks to playlist after the current artist" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    Menu->Append( wxID_ANY, _( "Enqueue after" ), EnqueueMenu );
 
     if( SelCount )
     {
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_ARTIST_EDITLABELS, _( "Edit Labels" ), _( "Edit the labels assigned to the selected artists" ) );
+        MenuItem = new wxMenuItem( Menu, ID_ARTIST_EDITLABELS,
+                            wxString( _( "Edit Labels" ) ) + guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITLABELS ),
+                            _( "Edit the labels assigned to the selected artists" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tags ) );
         Menu->Append( MenuItem );
 
-        MenuItem = new wxMenuItem( Menu, ID_ARTIST_EDITTRACKS, _( "Edit Songs" ), _( "Edit the songs from the selected artists" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit ) );
-        Menu->Append( MenuItem );
-
-        Menu->AppendSeparator();
-
-        MenuItem = new wxMenuItem( Menu, ID_ARTIST_COPYTO, _( "Copy to..." ), _( "Copy the current selected songs to a directory or device" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
-        Menu->Append( MenuItem );
-
-        Menu->AppendSeparator();
-        if( SelCount == 1 )
+        if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_EDIT_TRACKS )
         {
-            AddOnlineLinksMenu( Menu );
+            MenuItem = new wxMenuItem( Menu, ID_ARTIST_EDITTRACKS,
+                            wxString( _( "Edit Songs" ) ) + guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITTRACKS ),
+                            _( "Edit the songs from the selected artists" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
+            Menu->Append( MenuItem );
         }
 
-        AddArtistCommands( Menu, SelCount );
+        Menu->AppendSeparator();
+
+        MenuItem = new wxMenuItem( Menu, ID_ARTIST_SAVETOPLAYLIST,
+                            wxString( _( "Save to PlayList" ) ) + guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_SAVE ),
+                            _( "Save the selected tracks to PlayList" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_doc_save ) );
+        Menu->Append( MenuItem );
+
+        if( ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COPY_TO ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS ) )
+        {
+            Menu->AppendSeparator();
+
+            if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COPY_TO )
+            {
+                m_LibPanel->CreateCopyToMenu( Menu, ID_ARTIST_COPYTO );
+            }
+
+            if( SelCount == 1 && ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) )
+            {
+                AddOnlineLinksMenu( Menu );
+            }
+
+            if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS )
+                AddArtistCommands( Menu, SelCount );
+        }
     }
+
+    m_LibPanel->CreateContextMenu( Menu );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -150,7 +243,6 @@ void guArListBox::OnSearchLinkClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Links = Config->ReadAStr( wxT( "Link" ), wxEmptyString, wxT( "SearchLinks" ) );
-            wxASSERT( Links.Count() > 0 );
 
             index -= ID_LASTFM_SEARCH_LINK;
             wxString SearchLink = Links[ index ];
@@ -181,7 +273,6 @@ void guArListBox::OnCommandClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
-            wxASSERT( Commands.Count() > 0 );
 
             //guLogMessage( wxT( "CommandId: %u" ), index );
             index -= ID_ARTIST_COMMANDS;
@@ -266,6 +357,7 @@ int guArListBox::GetDragFiles( wxFileDataObject * files )
     guTrackArray Songs;
     int index;
     int count = GetSelectedSongs( &Songs );
+    m_LibPanel->NormalizeTracks( &Songs, true );
     for( index = 0; index < count; index++ )
     {
        wxString FileName = guFileDnDEncode( Songs[ index ].m_FileName );
