@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -23,12 +23,16 @@
 
 #include <wx/wx.h>
 
-#include "ItemListBox.h"
+#include "AccelListBox.h"
+
+class guLibPanel;
 
 // -------------------------------------------------------------------------------- //
-class guArListBox : public guListBox
+class guArListBox : public guAccelListBox
 {
   protected :
+    guLibPanel *    m_LibPanel;
+
     virtual void    GetItemsList( void );
     virtual void    CreateContextMenu( wxMenu * menu ) const;
     void            OnSearchLinkClicked( wxCommandEvent &event );
@@ -37,8 +41,10 @@ class guArListBox : public guListBox
 
     virtual int     GetDragFiles( wxFileDataObject * files );
 
+    virtual void    CreateAcceleratorTable();
+
   public :
-                    guArListBox( wxWindow * parent, guDbLibrary * db, const wxString &label );
+                    guArListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * db, const wxString &label );
                     ~guArListBox();
     virtual int     GetSelectedSongs( guTrackArray * songs ) const;
     bool            SelectArtistName( const wxString &ArtistName );
