@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -22,65 +22,39 @@
 #include "MD5.h"
 #include "Config.h"
 #include "Utils.h"
-//#include "HTTP.h"
-#include "Commands.h"
+#include "PlayerPanel.h"
 
 #include <wx/curl/http.h>
 #include <wx/tokenzr.h>
 
 // -------------------------------------------------------------------------------- //
-guAudioScrobble::guAudioScrobble( guDbLibrary * db )
+guAudioScrobbleSender::guAudioScrobbleSender( guDbLibrary * db, const wxString &serverurl )
 {
-    m_Db              = db;
-    // Read the configuration saved from previous session
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-    if( Config )
-    {
-        m_UserName    = Config->ReadStr( wxT( "UserName" ), wxEmptyString, wxT( "LastFM" ) );
-        m_Password    = Config->ReadStr( wxT( "Password" ), wxEmptyString, wxT( "LastFM" ) );
-        //m_SessionId   = Config->ReadStr( wxT( "ASSessionKey" ), wxEmptyString, wxT( "LastFM" ) );
-        //m_SubmitUrl   = Config->ReadStr( wxT( "SubmitUrl" ), wxEmptyString, wxT( "LastFM" ) );
-    }
+    m_Db            = db;
+    m_ServerUrl     = serverurl;
+
+    ReadUserConfig();
 
     m_SessionId = wxEmptyString;
     m_NowPlayUrl = wxEmptyString;
     m_SubmitUrl = wxEmptyString;
     m_ErrorCode = guAS_ERROR_NOSESSION;
-    m_SubmitPlayedSongsThread = NULL;
 
-    // Create the SubmitThread and start it
-    m_SubmitPlayedSongsThread = new guASPlayedThread( this, m_Db );
-    if( !m_SubmitPlayedSongsThread )
-    {
-        guLogError( wxT( "Could not create the Submit PLayed Songs thread" ) );
-    }
 }
 
 // -------------------------------------------------------------------------------- //
-guAudioScrobble::~guAudioScrobble()
+guAudioScrobbleSender::~guAudioScrobbleSender()
 {
-    if( m_SubmitPlayedSongsThread )
-    {
-        m_SubmitPlayedSongsThread->Pause();
-        m_SubmitPlayedSongsThread->Delete();
-    }
 }
 
 // -------------------------------------------------------------------------------- //
-void guAudioScrobble::OnConfigUpdated( void )
+void guAudioScrobbleSender::OnConfigUpdated( void )
 {
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-    if( Config )
-    {
-        m_UserName    = Config->ReadStr( wxT( "UserName" ), wxEmptyString, wxT( "LastFM" ) );
-        m_Password    = Config->ReadStr( wxT( "Password" ), wxEmptyString, wxT( "LastFM" ) );
-        //m_SessionId   = Config->ReadStr( wxT( "ASSessionKey" ), wxEmptyString, wxT( "LastFM" ) );
-        //m_SubmitUrl   = Config->ReadStr( wxT( "SubmitUrl" ), wxEmptyString, wxT( "LastFM" ) );
-    }
+    ReadUserConfig();
 }
 
 // -------------------------------------------------------------------------------- //
-int guAudioScrobble::ProcessError( const wxString &ErrorStr )
+int guAudioScrobbleSender::ProcessError( const wxString &ErrorStr )
 {
     if( ErrorStr.Contains( wxT( "BANNED" ) ) )
     {
@@ -102,7 +76,7 @@ int guAudioScrobble::ProcessError( const wxString &ErrorStr )
     else if( ErrorStr.Contains( wxT( "FAILED" ) ) )
     {
         m_ErrorCode = guAS_ERROR_FAILED;
-        guLogError( wxT( "Server Error when connecting to LastFM server :" ) + ErrorStr );
+        guLogError( wxT( "Server Error when connecting to LastFM server : " ) + ErrorStr );
     }
     else
     {
@@ -111,21 +85,17 @@ int guAudioScrobble::ProcessError( const wxString &ErrorStr )
         guLogError( wxT( "Unknown Error autenticating to LastFM server " ) + ErrorStr );
     }
 
-    // Update the MainFrame AudioScrobble Status
-    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
-    event.SetEventObject( ( wxObject * ) this );
-    event.SetInt( m_ErrorCode );
-    wxPostEvent( wxTheApp->GetTopWindow(), event );
     return m_ErrorCode;
 }
 
 // -------------------------------------------------------------------------------- //
-bool guAudioScrobble::GetSessionId( void )
+bool guAudioScrobbleSender::GetSessionId( void )
 {
+    //guLogMessage( wxT( "guAudioScrobbleSender:GetSessionId" ) );
     //http://post.audioscrobbler.com/?hs=true&p=1.2.1&c=<client-id>&v=<client-ver>&u=<user>&t=<timestamp>&a=<auth>
     wxString Content;
     long LocalTime = wxGetUTCTime();
-    wxString AS_Url = guAS_HANDSHAKE_URL wxT( "?hs=true" )\
+    wxString AS_Url = m_ServerUrl + wxT( "?hs=true" )\
                       wxT( "&p=" ) guAS_PROTOCOL_VERSION\
                       wxT( "&c=" ) guAS_CLIENT_ID\
                       wxT( "&v=" ) guAS_CLIENT_VERSION\
@@ -136,9 +106,10 @@ bool guAudioScrobble::GetSessionId( void )
     //guLogMessage( wxT( "AudioScrobble:GetSesionId : " ) + AS_Url );
     char * Buffer = NULL;
     wxCurlHTTP http;
-    http.AddHeader( wxT( "User-Agent: Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.0.5) Gecko/2008121622 Ubuntu/8.10 (intrepid) Firefox/3.0.5" ) );
+    http.AddHeader( wxT( "User-Agent: " ) guDEFAULT_BROWSER_USER_AGENT );
     http.AddHeader( wxT( "Accept: text/html" ) );
     http.AddHeader( wxT( "Accept-Charset: utf-8" ) );
+    http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
     http.Get( Buffer, AS_Url );
     if( Buffer )
     {
@@ -161,57 +132,30 @@ bool guAudioScrobble::GetSessionId( void )
                     //guLogMessage( wxT( "NowPlayUrl : " ) + NowPlayUrl );
                     //guLogMessage( wxT( "SubmitUrl  : " ) + SubmitUrl );
                     m_ErrorCode = guAS_ERROR_NOERROR;
-                    guLogMessage( wxT( "Loged in to lastfm AudioScrobble service." ) );
+                    guLogMessage( wxT( "Loged in to : " ) + m_ServerUrl );
                     //return !SessionId.IsEmpty() && !SubmitUrl.IsEmpty() && !NowPlayUrl.IsEmpty();
 
-                    // Update the MainFrame AudioScrobble Status
-                    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
-                    event.SetEventObject( ( wxObject * ) this );
-                    event.SetInt( m_ErrorCode );
-                    wxPostEvent( wxTheApp->GetTopWindow(), event );
                 }
                 else
                 {
                     ProcessError( Content );
-                    guLogError( wxT( "Could not get a LastFM:AudioScrobble SessionId" ) );
+                    guLogError( wxT( "Could not get the SessionId from: " ) + m_ServerUrl );
                 }
             }
         }
     }
-    if( wxTheApp )
-    {
-        // Update the MainFrame AudioScrobble Status
-        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
-        event.SetEventObject( ( wxObject * ) this );
-        event.SetInt( m_ErrorCode );
-        wxPostEvent( wxTheApp->GetTopWindow(), event );
-    }
     return m_ErrorCode == guAS_ERROR_NOERROR;
 }
 
 // -------------------------------------------------------------------------------- //
-wxString guAudioScrobble::GetAuthToken( int TimeStamp )
+wxString guAudioScrobbleSender::GetAuthToken( int TimeStamp )
 {
     guMD5 md5;
     return md5.MD5( m_Password + wxString::Format( wxT( "%u" ), TimeStamp ) );
 }
 
 // -------------------------------------------------------------------------------- //
-void guAudioScrobble::SendPlayedTrack( const guTrack &track )
-{
-    if( !m_Db->AddCachedPlayedSong( track ) )
-        guLogError( wxT( "Could not add Song to CachedSongs Database" ) );
-
-    if( !m_SubmitPlayedSongsThread )
-    {
-        m_SubmitPlayedSongsThread = new guASPlayedThread( this, m_Db );
-        if( !m_SubmitPlayedSongsThread )
-            guLogError( wxT( "Could no create the AudioScrobble Played thread" ) );
-    }
-}
-
-// -------------------------------------------------------------------------------- //
-bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs )
+bool guAudioScrobbleSender::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs )
 {
     wxString    PostData;
     wxString    Content;
@@ -222,31 +166,46 @@ bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs
     wxString    Track;
     wxCurlHTTP  http;
 
-    /* Info from MediaPortal as seems more complete than the LastFM/api v1.2.1 protocol info
-        s=<sessionID>            The Session ID string as returned by the handshake. Required.
-        a[0]=<artist>            The artist name. Required.
-        t[0]=<track>             The track title. Required.
-        i[0]=<time>              The time the track started playing, in UNIX timestamp format (integer number of seconds since 00:00:00, January 1st 1970 UTC). This must be in the UTC time zone, and is required.
-        o[0]=<source>            The source of the track. Required, must be one of the following codes:
+    /*
+        s=<sessionID>
+            The Session ID string returned by the handshake request. Required.
+        a[0]=<artist>
+            The artist name. Required.
+        t[0]=<track>
+            The track title. Required.
+        i[0]=<time>
+            The time the track started playing, in UNIX timestamp format (integer number of seconds since 00:00:00, January 1st 1970 UTC). This must be in the UTC time zone, and is required.
+        o[0]=<source>
+            The source of the track. Required, must be one of the following codes:
 
-            P = Chosen by the user, no shuffle
-            S = Chosen by the user, shuffle enabled
-            T = Chosen by the user, unknown shuffle status (e.g. iPod)
-            R = Non-personalised broadcast (e.g. Shoutcast, BBC Radio 1)
-            E = Personalised recommendation except Last.fm (e.g. Pandora, Launchcast)
-            L = Last.fm (any mode)
-            U = Source unknown
+            P
+                Chosen by the user (the most common value, unless you have a reason for choosing otherwise, use this).
+            R
+                Non-personalised broadcast (e.g. Shoutcast, BBC Radio 1).
+            E
+                Personalised recommendation except Last.fm (e.g. Pandora, Launchcast).
+            L
+                Last.fm (any mode). In this case, the 5-digit Last.fm recommendation key must be appended to this source ID to prove the validity of the submission (for example, "o[0]=L1b48a").
 
         r[0]=<rating>
+            A single character denoting the rating of the track. Empty if not applicable.
 
-            L = Love
-            B = Ban
-            S = Skip (only if source=L)
+            L
+                Love (on any mode if the user has manually loved the track). This implies a listen.
+            B
+                Ban (only if source=L). This implies a skip, and the client should skip to the next track when a ban happens.
+            S
+                Skip (only if source=L)
 
-        b[0]=<album>             The album title, or empty if not known.
-        l[0]=<secs>              The length of the track in seconds, or empty if not known.
-        n[0]=<tracknumber>       The position of the track on the album, or empty if not known.
-        m[0]=<mb-trackid>        The MusicBrainz Track ID, or empty if not known.
+            Note: Currently a Last.fm web service must also be called to set love (track.love) or ban (track.ban) status. We anticipate that the next version of the scrobble protocol will no longer perform love and ban and this will instead be handled by the web services only.
+        l[0]=<secs>
+            The length of the track in seconds. Required when the source is P, optional otherwise.
+        b[0]=<album>
+            The album title, or an empty string if not known.
+        n[0]=<tracknumber>
+            The position of the track on the album, or an empty string if not known.
+        m[0]=<mb-trackid>
+            The MusicBrainz Track ID, or an empty string if not known.
     */
 
 
@@ -262,12 +221,12 @@ bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs
             Track  = guURLEncode( PlayedSongs[ index ].m_TrackName );
             Album  = guURLEncode( PlayedSongs[ index ].m_AlbumName );
             //
-            PostData += wxString::Format( wxT( "a[%u]=%s&t[%u]=%s&i[%u]=%u&o[%u]=%c&r[%u]=&l[%u]=%u&b[%u]=%s&n[%u]=%s&m[%u]=&" ),
+            PostData += wxString::Format( wxT( "a[%u]=%s&t[%u]=%s&i[%u]=%u&o[%u]=%c&r[%u]=%c&l[%u]=%u&b[%u]=%s&n[%u]=%s&m[%u]=&" ),
                                 index, Artist.c_str(),
                                 index, Track.c_str(),
                                 index, PlayedSongs[ index ].m_PlayedTime,
                                 index, PlayedSongs[ index ].m_Source,
-                                index,
+                                index, PlayedSongs[ index ].m_Rating,
                                 index, PlayedSongs[ index ].m_TrackLen,
                                 index, Album.c_str(),
                                 index, ( PlayedSongs[ index ].m_TrackNum > 0 ) ?
@@ -277,17 +236,19 @@ bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs
         }
         PostData.RemoveLast( 1 ); // we remove the last & added
 
-        //guLogMessage( wxT( "AudioScrobble::Played : " ) + PostData );
+        //guLogMessage( wxT( "AudioScrobble::Played : %s" ), PostData.c_str() );
         http.AddHeader( wxT( "Content-Type: application/x-www-form-urlencoded" ) );
+        http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
         if( http.Post( wxCURL_STRING2BUF( PostData ), PostData.Length(), m_SubmitUrl ) )
         {
             Content = http.GetResponseBody();
             if( !Content.IsEmpty() )
             {
-                //guLogMessage( wxT( "AudioScrobble::Response : " ) + Content );
+                //guLogMessage( wxT( "AudioScrobble::Response :\n%s" ), Content.c_str() );
                 if( Content.Contains( wxT( "OK" ) ) )
                 {
                     m_ErrorCode = guAS_ERROR_NOERROR;
+
                     return true;
                 }
                 else
@@ -299,23 +260,7 @@ bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &PlayedSongs
 }
 
 // -------------------------------------------------------------------------------- //
-void guAudioScrobble::SendNowPlayingTrack( const guTrack &track )
-{
-    guAS_SubmitInfo * SubmitInfo = new guAS_SubmitInfo();
-
-    SubmitInfo->m_ArtistName = track.m_ArtistName;
-    SubmitInfo->m_AlbumName  = track.m_AlbumName;
-    SubmitInfo->m_TrackName  = track.m_SongName;
-    SubmitInfo->m_TrackLen   = track.m_Length;
-    SubmitInfo->m_TrackNum   = track.m_Number;
-
-    guASNowPlayingThread * NowPlayingThread  = new guASNowPlayingThread( this, SubmitInfo );
-    if( !NowPlayingThread )
-        guLogError( wxT( "Could no create the AudioScrobble NowPlaying thread" ) );
-}
-
-// -------------------------------------------------------------------------------- //
-bool guAudioScrobble::SubmitNowPlaying( const guAS_SubmitInfo * cursong )
+bool guAudioScrobbleSender::SubmitNowPlaying( const guAS_SubmitInfo * cursong )
 {
     wxCurlHTTP  http;
     wxString    PostData;
@@ -343,9 +288,10 @@ bool guAudioScrobble::SubmitNowPlaying( const guAS_SubmitInfo * cursong )
                                    wxEmptyString
                         );
 
-    //guLogMessage( wxT( "AudioScrobble::NowPlaying : " ) + PostData );
+    //guLogMessage( wxT( "AudioScrobble::NowPlaying : " ) + m_NowPlayUrl + PostData );
 
     http.AddHeader( wxT( "Content-Type: application/x-www-form-urlencoded" ) );
+    http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
     if( http.Post( wxCURL_STRING2BUF( PostData ), PostData.Length(), m_NowPlayUrl ) )
     {
         Content = http.GetResponseBody();
@@ -362,15 +308,276 @@ bool guAudioScrobble::SubmitNowPlaying( const guAS_SubmitInfo * cursong )
                 ProcessError( Content );
         }
     }
+    else
+    {
+        guLogMessage( wxT( "Error submitting the data to the scrobble server" ) );
+    }
     return false;
 }
 
+
+
+
 // -------------------------------------------------------------------------------- //
-void guAudioScrobble::EndSubmitThread( void )
+guLastFMAudioScrobble::guLastFMAudioScrobble( guDbLibrary * db ) :
+    guAudioScrobbleSender( db, guLASTFM_POST_SERVER )
 {
-    m_SubmitPlayedSongsThread = NULL;
+    ReadUserConfig();
 }
 
+// -------------------------------------------------------------------------------- //
+guLastFMAudioScrobble::~guLastFMAudioScrobble()
+{
+}
+
+// -------------------------------------------------------------------------------- //
+void guLastFMAudioScrobble::ReadUserConfig( void )
+{
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    if( Config )
+    {
+        m_UserName = Config->ReadStr( wxT( "UserName" ), wxEmptyString, wxT( "LastFM" ) );
+        m_Password = Config->ReadStr( wxT( "Password" ), wxEmptyString, wxT( "LastFM" ) );
+    }
+}
+
+
+
+
+// -------------------------------------------------------------------------------- //
+guLibreFMAudioScrobble::guLibreFMAudioScrobble( guDbLibrary * db ) :
+    guAudioScrobbleSender( db, guLIBREFM_POST_SERVER )
+{
+    ReadUserConfig();
+}
+
+// -------------------------------------------------------------------------------- //
+guLibreFMAudioScrobble::~guLibreFMAudioScrobble()
+{
+}
+
+// -------------------------------------------------------------------------------- //
+void guLibreFMAudioScrobble::ReadUserConfig( void )
+{
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    if( Config )
+    {
+        m_UserName = Config->ReadStr( wxT( "UserName" ), wxEmptyString, wxT( "LibreFM" ) );
+        m_Password = Config->ReadStr( wxT( "Password" ), wxEmptyString, wxT( "LibreFM" ) );
+    }
+}
+
+
+
+
+// -------------------------------------------------------------------------------- //
+guAudioScrobble::guAudioScrobble( guDbLibrary * db )
+{
+    m_Db = db;
+    m_LastFMAudioScrobble = NULL;
+    m_LibreFMAudioScrobble = NULL;
+    m_PlayedThread = NULL;
+    m_NowPlayingInfo = NULL;
+
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+
+    if( Config->ReadBool( wxT( "SubmitEnabled" ), false, wxT( "LastFM" ) ) )
+    {
+        m_LastFMAudioScrobble = new guLastFMAudioScrobble( db );
+    }
+
+    if( Config->ReadBool( wxT( "SubmitEnabled" ), false, wxT( "LibreFM" ) ) )
+    {
+        m_LibreFMAudioScrobble = new guLibreFMAudioScrobble( db );
+    }
+
+    // Update the MainFrame AudioScrobble Status
+    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
+    event.SetEventObject( ( wxObject * ) this );
+    event.SetInt( 0 );
+    wxPostEvent( wxTheApp->GetTopWindow(), event );
+
+}
+
+
+// -------------------------------------------------------------------------------- //
+guAudioScrobble::~guAudioScrobble()
+{
+    if( m_PlayedThread )
+    {
+        m_PlayedThread->Pause();
+        m_PlayedThread->Delete();
+    }
+
+    if( m_LastFMAudioScrobble )
+        delete m_LastFMAudioScrobble;
+
+    if( m_LibreFMAudioScrobble )
+        delete m_LibreFMAudioScrobble;
+}
+
+// -------------------------------------------------------------------------------- //
+bool guAudioScrobble::SubmitNowPlaying( const guAS_SubmitInfo * curtrack )
+{
+    //guLogMessage( wxT( "guAudioScrobbler:SubmitNowPlaying: %s" ), curtrack->m_TrackName.c_str() );
+    if( m_LastFMAudioScrobble )
+    {
+        m_LastFMAudioScrobble->SubmitNowPlaying( curtrack );
+    }
+
+    if( m_LibreFMAudioScrobble )
+    {
+        m_LibreFMAudioScrobble->SubmitNowPlaying( curtrack );
+    }
+
+    int HasError = ( m_LastFMAudioScrobble && m_LastFMAudioScrobble->GetErrorCode() ) ||
+                   ( m_LibreFMAudioScrobble && m_LibreFMAudioScrobble->GetErrorCode() );
+
+    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
+    event.SetEventObject( ( wxObject * ) this );
+    event.SetInt( HasError );
+    wxPostEvent( wxTheApp->GetTopWindow(), event );
+
+    return !HasError;
+}
+
+// -------------------------------------------------------------------------------- //
+bool guAudioScrobble::SubmitPlayedSongs( const guAS_SubmitInfoArray &playedtracks )
+{
+    //guLogMessage( wxT( "guAudioScrobbler:SubmitPlayedSongs" ) );
+    if( m_LastFMAudioScrobble )
+    {
+        m_LastFMAudioScrobble->SubmitPlayedSongs( playedtracks );
+    }
+
+    if( m_LibreFMAudioScrobble )
+    {
+        m_LibreFMAudioScrobble->SubmitPlayedSongs( playedtracks );
+    }
+
+    int HasError = ( m_LastFMAudioScrobble && m_LastFMAudioScrobble->GetErrorCode() ) ||
+                   ( m_LibreFMAudioScrobble && m_LibreFMAudioScrobble->GetErrorCode() );
+
+    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_AUDIOSCROBBLE_UPDATED );
+    event.SetEventObject( ( wxObject * ) this );
+    event.SetInt( HasError );
+    wxPostEvent( wxTheApp->GetTopWindow(), event );
+
+    return !HasError;
+}
+
+// -------------------------------------------------------------------------------- //
+void guAudioScrobble::SendPlayedTrack( const guCurrentTrack &track )
+{
+    //guLogMessage( wxT( "guAudioScrobble::SendPlayedTrack" ) );
+    if( !m_Db->AddCachedPlayedSong( track ) )
+        guLogError( wxT( "Could not add Song to CachedSongs Database" ) );
+
+    if( !m_PlayedThread )
+    {
+        m_PlayedThread = new guASPlayedThread( this, m_Db );
+        if( !m_PlayedThread )
+            guLogError( wxT( "Could no create the AudioScrobble Played thread" ) );
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guAudioScrobble::SendNowPlayingTrack( const guCurrentTrack &track )
+{
+    //guLogMessage( wxT( "guAudioScrobble::SendNowPlayingTrack" ) );
+    wxMutexLocker Lock( m_NowPlayingInfoMutex );
+
+    if( m_NowPlayingInfo )
+    {
+        delete m_NowPlayingInfo;
+    }
+    m_NowPlayingInfo = new guAS_SubmitInfo();
+
+    m_NowPlayingInfo->m_ArtistName = track.m_ArtistName;
+    m_NowPlayingInfo->m_AlbumName  = track.m_AlbumName;
+    m_NowPlayingInfo->m_TrackName  = track.m_SongName;
+    m_NowPlayingInfo->m_TrackLen   = track.m_Length;
+    m_NowPlayingInfo->m_TrackNum   = track.m_Number;
+
+    guAS_SubmitInfoArray    SubmitInfo;
+    SubmitInfo = m_Db->GetCachedPlayedSongs( guAS_SUBMITTRACKS );
+    if( !SubmitInfo.Count() )
+    {
+        guASNowPlayingThread * NowPlayingThread  = new guASNowPlayingThread( this, m_NowPlayingInfo );
+        if( !NowPlayingThread )
+            guLogError( wxT( "Could no create the AudioScrobble NowPlaying thread" ) );
+
+        m_NowPlayingInfo = NULL;
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guAudioScrobble::EndPlayedThread( void )
+{
+    wxMutexLocker Lock( m_NowPlayingInfoMutex );
+
+    m_PlayedThread = NULL;
+    if( m_NowPlayingInfo )
+    {
+        guASNowPlayingThread * NowPlayingThread  = new guASNowPlayingThread( this, m_NowPlayingInfo );
+        if( !NowPlayingThread )
+            guLogError( wxT( "Could no create the AudioScrobble NowPlaying thread" ) );
+        m_NowPlayingInfo = NULL;
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guAudioScrobble::OnConfigUpdated( void )
+{
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+
+    if( Config->ReadBool( wxT( "SubmitEnabled" ), false, wxT( "LastFM" ) ) )
+    {
+        if( !m_LastFMAudioScrobble )
+            m_LastFMAudioScrobble = new guLastFMAudioScrobble( m_Db );
+    }
+    else
+    {
+        if( m_LastFMAudioScrobble )
+        {
+            delete m_LastFMAudioScrobble;
+            m_LastFMAudioScrobble = NULL;
+        }
+    }
+
+    if( Config->ReadBool( wxT( "SubmitEnabled" ), false, wxT( "LibreFM" ) ) )
+    {
+        if( !m_LibreFMAudioScrobble )
+            m_LibreFMAudioScrobble = new guLibreFMAudioScrobble( m_Db );
+    }
+    else
+    {
+        if( m_LibreFMAudioScrobble )
+        {
+            delete m_LibreFMAudioScrobble;
+            m_LibreFMAudioScrobble = NULL;
+        }
+    }
+
+    if( m_LastFMAudioScrobble )
+    {
+        m_LastFMAudioScrobble->OnConfigUpdated();
+    }
+
+    if( m_LibreFMAudioScrobble )
+    {
+        m_LibreFMAudioScrobble->OnConfigUpdated();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+bool guAudioScrobble::IsOk( void )
+{
+    return ( m_LastFMAudioScrobble && m_LastFMAudioScrobble->IsOk() ) ||
+           ( m_LibreFMAudioScrobble && m_LibreFMAudioScrobble->IsOk() );
+}
+
+
 // -------------------------------------------------------------------------------- //
 // guASNowPlayingThread
 // -------------------------------------------------------------------------------- //
@@ -408,15 +615,16 @@ guASNowPlayingThread::ExitCode guASNowPlayingThread::Entry()
         FailCnt++;
         if( FailCnt > 2 )
         {
-            guLogError( wxT( "Reached max number of fail retry submitting to lastfm AudioScrobble service." ) );
-            FailCnt = 0;
-            while( !m_AudioScrobble->GetSessionId() && !TestDestroy() )
-            {
-                Sleep( guAS_SUBMIT_RETRY_TIMEOUT );
-                FailCnt++;
-                if( FailCnt > 2 )
-                    return 0;
-            }
+//            guLogError( wxT( "Reached max number of fail retry submitting to lastfm AudioScrobble service." ) );
+//            FailCnt = 0;
+//            while( !m_AudioScrobble->GetSessionId() && !TestDestroy() )
+//            {
+//                Sleep( guAS_SUBMIT_RETRY_TIMEOUT );
+//                FailCnt++;
+//                if( FailCnt > 2 )
+//                    return 0;
+//            }
+            break;
         }
         // If have not been destroyed wait 2 mins between submits.
         if( !TestDestroy() )
@@ -449,7 +657,7 @@ guASPlayedThread::~guASPlayedThread()
     if( !TestDestroy() )
     {
         if( m_AudioScrobble )
-            m_AudioScrobble->EndSubmitThread();
+            m_AudioScrobble->EndPlayedThread();
     }
 }
 
@@ -479,25 +687,26 @@ guASPlayedThread::ExitCode guASPlayedThread::Entry()
 //                event.SetInt( m_AudioScrobble->GetErrorCode() );
 //                wxPostEvent( wxTheApp->GetTopWindow(), event );
                 //
-                if( m_AudioScrobble->GetErrorCode() != guAS_ERROR_NOERROR )
-                {
-                    FailCnt++;
-                    if( FailCnt > 2 )
-                    {
-                        guLogError( wxT( "Reached max number of fail retry submitting to lastfm AudioScrobble service." ) );
-                        FailCnt = 0;
-                        while( !m_AudioScrobble->GetSessionId() && !TestDestroy() )
-                        {
-                            Sleep( guAS_SUBMIT_RETRY_TIMEOUT );
-                            FailCnt++;
-                            if( FailCnt > 2 )
-                                return 0;
-                        }
-                    }
-                }
-                else
-                    return 0;
-
+//                if( m_AudioScrobble->GetErrorCode() != guAS_ERROR_NOERROR )
+//                {
+//                    FailCnt++;
+//                    if( FailCnt > 2 )
+//                    {
+//                        guLogError( wxT( "Reached max number of fail retry submitting to lastfm AudioScrobble service." ) );
+//                        FailCnt = 0;
+//                        while( !m_AudioScrobble->GetSessionId() && !TestDestroy() )
+//                        {
+//                            Sleep( guAS_SUBMIT_RETRY_TIMEOUT );
+//                            FailCnt++;
+//                            if( FailCnt > 2 )
+//                                return 0;
+//                        }
+//                    }
+//                }
+//                else
+//                    return 0;
+                if( FailCnt++ > 2 )
+                    break;
                 Sleep( guAS_SUBMIT_RETRY_TIMEOUT ); // Wait 30 Secs between submit attempts
             }
             // if the submit was ok then delete the songs from cache
