@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -40,10 +40,10 @@ static void inline IndentPressedBitmap( wxRect * rect, int button_state )
 // -------------------------------------------------------------------------------- //
 guAuiTabArt::guAuiTabArt() : wxAuiDefaultTabArt()
 {
-    m_BgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_3DHILIGHT );
+    m_BgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOW );
     m_SelBgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_HIGHLIGHT );
-    m_TextFgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_INACTIVECAPTIONTEXT );
-    m_SelTextFgColour = wxSystemSettings::GetColour( wxSYS_COLOUR_BTNTEXT );
+    m_TextFgColor = wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT );
+    m_SelTextFgColour = wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOWTEXT );
 
     m_disabled_close_bmp = guImage( guIMAGE_INDEX_tiny_close_normal );
     m_active_close_bmp = guImage( guIMAGE_INDEX_tiny_close_highlight );
@@ -67,6 +67,42 @@ wxAuiTabArt * guAuiTabArt::Clone()
     return art;
 }
 
+// -------------------------------------------------------------------------------- //
+void guAuiTabArt::DrawBackground( wxDC &dc, wxWindow * wnd, const wxRect &rect )
+{
+    // draw background
+   wxColor top_color      = m_base_colour;
+   wxColor bottom_color   = wxAuiStepColour( m_base_colour, 140 );
+   wxRect r;
+
+   if( m_flags & wxAUI_NB_BOTTOM )
+       r = wxRect( rect.x, rect.y, rect.width + 2, rect.height );
+   // TODO: else if (m_flags &wxAUI_NB_LEFT) {}
+   // TODO: else if (m_flags &wxAUI_NB_RIGHT) {}
+   else //for wxAUI_NB_TOP
+       r = wxRect(rect.x, rect.y, rect.width+2, rect.height-3);
+
+    dc.GradientFillLinear( r, top_color, bottom_color, wxNORTH );
+
+   // draw base lines
+   dc.SetPen( m_border_pen );
+   int y = rect.GetHeight();
+   int w = rect.GetWidth();
+
+   if( m_flags & wxAUI_NB_BOTTOM )
+   {
+       dc.SetBrush( wxBrush( bottom_color ) );
+       dc.DrawRectangle( -1, 0, w + 2, 4 );
+   }
+   // TODO: else if (m_flags &wxAUI_NB_LEFT) {}
+   // TODO: else if (m_flags &wxAUI_NB_RIGHT) {}
+   else //for wxAUI_NB_TOP
+   {
+       dc.SetBrush( m_base_colour_brush );
+       dc.DrawRectangle( -1, y - 4, w + 2, 4 );
+   }
+}
+
 // -------------------------------------------------------------------------------- //
 void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &page,
         const wxRect & in_rect, int close_button_state, wxRect * out_tab_rect,
@@ -78,7 +114,7 @@ void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &pag
 
     // if the caption is empty, measure some temporary text
     wxString caption = page.caption;
-    if (caption.empty())
+    if( caption.empty() )
         caption = wxT("Xj");
 
     dc.SetFont(m_selected_font);
@@ -96,7 +132,9 @@ void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &pag
                                  close_button_state,
                                  x_extent);
 
-    wxCoord tab_height = m_tab_ctrl_height - 3;
+    wxCoord tab_height = m_tab_ctrl_height - 1;
+    if( !page.active )
+        tab_height -= 2;
     wxCoord tab_width = tab_size.x;
     wxCoord tab_x = in_rect.x;
     wxCoord tab_y = in_rect.y + in_rect.height - tab_height;
@@ -107,12 +145,12 @@ void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &pag
 
     // select pen, brush and font for the tab to be drawn
 
-    if (page.active)
+    if( page.active )
     {
-        dc.SetFont(m_selected_font);
+        dc.SetFont( m_selected_font );
         texty = selected_texty;
     }
-     else
+    else
     {
         dc.SetFont(m_normal_font);
         texty = normal_texty;
@@ -172,82 +210,34 @@ void guAuiTabArt::DrawTab(wxDC &dc, wxWindow * wnd, const wxAuiNotebookPage &pag
     int drawn_tab_height = border_points[0].y - border_points[1].y;
 
 
-    if (page.active)
+    if( page.active )
     {
-        wxRect r(tab_x, tab_y+1, tab_width, tab_height-3);
-
-        // start the gradent up a bit and leave the inside border inset
-        // by a pixel for a 3D look.  Only the top half of the inactive
-        // tab will have a slight gradient
-        r.x += 3;
-        r.y++;
-        r.width -= 4;
-        r.height /= 2;
-        r.height--;
-
-        // -- draw top gradient fill for glossy look
-        wxColor top_color = m_SelBgColor;
-        wxColor bottom_color = wxAuiStepColour(m_base_colour, 140);
-        dc.GradientFillLinear(r, bottom_color, top_color, wxNORTH);
-
-        r.y += r.height;
-        r.y--;
-
-        // -- draw bottom fill for glossy look
-        top_color = m_base_colour;
-        bottom_color = m_base_colour;
-        dc.GradientFillLinear(r, top_color, bottom_color, wxSOUTH);
-    }
-     else
-    {
-        // draw inactive tab
-
-        wxRect r(tab_x, tab_y+1, tab_width, tab_height-3);
-
-        // start the gradent up a bit and leave the inside border inset
-        // by a pixel for a 3D look.  Only the top half of the inactive
-        // tab will have a slight gradient
-        r.x += 3;
-        r.y++;
-        r.width -= 4;
-        r.height /= 2;
-        r.height--;
-
-        // -- draw top gradient fill for glossy look
-        wxColor top_color = m_base_colour;
-        wxColor bottom_color = wxAuiStepColour(top_color, 160);
-        dc.GradientFillLinear(r, bottom_color, top_color, wxNORTH);
-
-        r.y += r.height;
-        r.y--;
-
-        // -- draw bottom fill for glossy look
-        top_color = m_base_colour;
-        bottom_color = m_base_colour;
-        dc.GradientFillLinear(r, top_color, bottom_color, wxSOUTH);
+        dc.SetBrush( m_base_colour_brush );
+        dc.SetPen( * wxTRANSPARENT_PEN );
+        dc.DrawRectangle( tab_x + 1, tab_y + 2, tab_width - 1, tab_height - 3 );
     }
 
     // draw tab outline
-    dc.SetPen(m_border_pen);
-    dc.SetBrush(*wxTRANSPARENT_BRUSH);
-    dc.DrawPolygon(WXSIZEOF(border_points), border_points);
+    dc.SetPen( m_border_pen );
+    dc.SetBrush( * wxTRANSPARENT_BRUSH );
+    dc.DrawPolygon( WXSIZEOF( border_points ), border_points );
 
     // there are two horizontal grey lines at the bottom of the tab control,
     // this gets rid of the top one of those lines in the tab control
     if (page.active)
     {
-       if (m_flags &wxAUI_NB_BOTTOM)
-           dc.SetPen(wxPen(wxColour(wxAuiStepColour(m_base_colour, 170))));
-       // TODO: else if (m_flags &wxAUI_NB_LEFT) {}
-       // TODO: else if (m_flags &wxAUI_NB_RIGHT) {}
-       else //for wxAUI_NB_TOP
-           dc.SetPen(m_base_colour_pen);
-        dc.DrawLine(border_points[0].x+1,
-                    border_points[0].y,
-                    border_points[5].x,
-                    border_points[5].y);
-    }
+        if( m_flags & wxAUI_NB_BOTTOM )
+            dc.SetPen( wxPen( wxColour( wxAuiStepColour( m_base_colour, 170 ) ) ) );
+        // TODO: else if (m_flags &wxAUI_NB_LEFT) {}
+        // TODO: else if (m_flags &wxAUI_NB_RIGHT) {}
+        else //for wxAUI_NB_TOP
+           dc.SetPen( m_base_colour_pen );
 
+        dc.DrawLine( border_points[ 0 ].x + 1,
+                     border_points[ 0 ].y,
+                     border_points[ 5 ].x,
+                     border_points[ 5 ].y );
+    }
 
     int text_offset = tab_x + 8;
     int close_button_width = 0;
@@ -365,6 +355,51 @@ void guAuiNotebook::UpdateTabCtrlHeight()
     }
 }
 
+// -------------------------------------------------------------------------------- //
+wxString guAuiNotebook::GetPageId( wxAuiNotebookPage &page )
+{
+    static wxArrayString Names;
+    static wxArrayString Ids;
+
+    if( !Names.Count() )
+    {
+        Names.Add( _( "Library" ) );
+        Names.Add( _( "Radio" ) );
+        Names.Add( _( "Last.fm" ) );
+        Names.Add( _( "Lyrics" ) );
+        Names.Add( _( "PlayLists" ) );
+        Names.Add( _( "Podcasts" ) );
+        Names.Add( _( "Browser" ) );
+        Names.Add( _( "Files" ) );
+        Names.Add( wxT( "Jamendo" ) );
+        Names.Add( wxT( "Magnatune" ) );
+        Names.Add( _( "Tree" ) );
+    }
+
+    if( !Ids.Count() )
+    {
+        Ids.Add( wxT( "Library" ) );
+        Ids.Add( wxT( "Radio" ) );
+        Ids.Add( wxT( "Last.fm" ) );
+        Ids.Add( wxT( "Lyrics" ) );
+        Ids.Add( wxT( "PlayLists" ) );
+        Ids.Add( wxT( "Podcasts" ) );
+        Ids.Add( wxT( "Browser" ) );
+        Ids.Add( wxT( "Files" ) );
+        Ids.Add( wxT( "Jamendo" ) );
+        Ids.Add( wxT( "Magnatune" ) );
+        Ids.Add( wxT( "TreeView" ) );
+    }
+
+    int Index = Names.Index( page.caption );
+    if( Index != wxNOT_FOUND )
+    {
+        return Ids[ Index ];
+    }
+
+    return wxEmptyString;
+}
+
 // -------------------------------------------------------------------------------- //
 wxString guAuiNotebook::SavePerspective( void )
 {
@@ -400,7 +435,8 @@ wxString guAuiNotebook::SavePerspective( void )
                 tabs += wxT( "*" );
             else if( ( int ) p == tabframe->m_tabs->GetActivePage() )
                 tabs += wxT( "+" );
-            tabs += wxString::Format( wxT( "%02u[%s]" ), page_idx, page.caption.c_str() );
+            //tabs += wxString::Format( wxT( "%02u[%s]" ), page_idx, page.caption.c_str() );
+            tabs += wxString::Format( wxT( "%02u[%s]" ), page_idx, GetPageId( page ).c_str() );
         }
     }
     tabs += wxT( "@" );
