@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -168,14 +168,15 @@ class guAuiTabArt : public wxAuiDefaultTabArt
     virtual ~guAuiTabArt();
 
     wxAuiTabArt * Clone();
-    virtual void DrawTab(wxDC& dc,
-                         wxWindow* wnd,
-                         const wxAuiNotebookPage& pane,
-                         const wxRect& in_rect,
+    virtual void DrawTab( wxDC &dc,
+                         wxWindow * wnd,
+                         const wxAuiNotebookPage &pane,
+                         const wxRect &in_rect,
                          int close_button_state,
-                         wxRect* out_tab_rect,
-                         wxRect* out_button_rect,
-                         int* x_extent);
+                         wxRect * out_tab_rect,
+                         wxRect * out_button_rect,
+                         int * x_extent );
+    virtual void DrawBackground( wxDC &dc, wxWindow * wnd, const wxRect & rect );
 
 };
 
@@ -184,6 +185,7 @@ class guAuiNotebook : public wxAuiNotebook
 {
   protected :
 //    guAuiTabArt     m_AuiTabArt;
+    wxString    GetPageId( wxAuiNotebookPage &page );
 
   public:
 
