@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -48,17 +48,38 @@ class guConfig : public wxConfig
     bool            WriteStr( const wxString &KeyName, const wxString &Value, const wxString &Category = wxEmptyString );
     wxArrayString   ReadAStr( const wxString &Key, const wxString &Default, const wxString &Category = wxEmptyString );
     bool            WriteAStr( const wxString &Key, const wxArrayString &Value, const wxString &Category = wxEmptyString, bool ResetGroup = true );
+#if wxUSE_STL
+    bool            WriteAStr( const wxString &Key, const wxSortedArrayString &Value, const wxString &Category = wxEmptyString, bool ResetGroup = true );
+#endif
     wxArrayInt      ReadANum( const wxString &Key, const int Default, const wxString &Category = wxEmptyString );
     bool            WriteANum( const wxString &Key, const wxArrayInt &Value, const wxString &Category = wxEmptyString, bool ResetGroup = true );
 
     void            RegisterObject( wxEvtHandler * object );
     void            UnRegisterObject( wxEvtHandler * object );
-    void            SendConfigChangedEvent( void );
+    void            SendConfigChangedEvent( const int flags = 0 );
 
     bool            GetIgnoreLayouts( void ) { return m_IgnoreLayouts; }
     void            SetIgnoreLayouts( const bool ignorelayouts ) { m_IgnoreLayouts = ignorelayouts; };
 
 };
 
+// -------------------------------------------------------------------------------- //
+wxString inline escape_configlist_str( const wxString &val )
+{
+    wxString RetVal = val;
+    RetVal.Replace( wxT( ":" ), wxT( "_$&" ) );
+    RetVal.Replace( wxT( ";" ), wxT( "_&$" ) );
+    return RetVal;
+}
+
+// -------------------------------------------------------------------------------- //
+wxString inline unescape_configlist_str( const wxString &val )
+{
+    wxString RetVal = val;
+    RetVal.Replace( wxT( "_$&" ), wxT( ":" ) );
+    RetVal.Replace( wxT( "_&$" ), wxT( ";" ) );
+    return RetVal;
+}
+
 #endif
 // -------------------------------------------------------------------------------- //
