@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -48,7 +48,7 @@ WX_DEFINE_OBJARRAY(guCoverImageArray);
 
 // -------------------------------------------------------------------------------- //
 guCoverEditor::guCoverEditor( wxWindow* parent, const wxString &Artist, const wxString &Album ) :
-               wxDialog( parent, wxID_ANY, _( "Cover Editor" ), wxDefaultPosition, wxSize( 520, 430 ), wxDEFAULT_DIALOG_STYLE )
+               wxDialog( parent, wxID_ANY, _( "Cover Editor" ), wxDefaultPosition, wxSize( 520, 446 ), wxDEFAULT_DIALOG_STYLE )
 {
     wxStaticText* ArtistStaticText;
     wxStaticText* AlbumStaticText;
@@ -99,13 +99,13 @@ guCoverEditor::guCoverEditor( wxWindow* parent, const wxString &Artist, const wx
 
 	CoverSizer->Add( 0, 0, 1, wxEXPAND, 5 );
 
-	m_PrevButton = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_left ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
+	m_PrevButton = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_left ), wxDefaultPosition, wxSize( 32, 96 ), wxBU_AUTODRAW );
 	CoverSizer->Add( m_PrevButton, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );
 
 	m_CoverBitmap = new wxStaticBitmap( this, wxID_ANY, guImage( guIMAGE_INDEX_blank_cd_cover ), wxDefaultPosition, wxSize( -1,-1 ), 0 );
 	CoverSizer->Add( m_CoverBitmap, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5 );
 
-	m_NextButton = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_right ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
+	m_NextButton = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_right ), wxDefaultPosition, wxSize( 32, 96 ), wxBU_AUTODRAW );
 	CoverSizer->Add( m_NextButton, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );
 
 
@@ -134,6 +134,10 @@ guCoverEditor::guCoverEditor( wxWindow* parent, const wxString &Artist, const wx
 
 	MainSizer->Add( GaugeSizer, 0, wxEXPAND, 5 );
 
+	m_EmbedToFilesChkBox = new wxCheckBox( this, wxID_ANY, _( "Embed into tracks" ), wxDefaultPosition, wxDefaultSize, 0 );
+    m_EmbedToFilesChkBox->SetValue( Config->ReadBool( wxT( "EmbedToFiles" ), false, wxT( "General" ) ) );
+	MainSizer->Add( m_EmbedToFilesChkBox, 0, wxRIGHT|wxLEFT, 5 );
+
     wxStdDialogButtonSizer * ButtonsSizer;
     wxButton * ButtonsSizerOK;
     wxButton * ButtonsSizerCancel;
@@ -162,6 +166,8 @@ guCoverEditor::guCoverEditor( wxWindow* parent, const wxString &Artist, const wx
 	m_EngineChoice->Connect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guCoverEditor::OnEngineChanged ), NULL, this );
 
 	m_CoverBitmap->Connect( wxEVT_LEFT_DCLICK, wxMouseEventHandler( guCoverEditor::OnCoverLeftDClick ), NULL, this );
+	m_CoverBitmap->Connect( wxEVT_LEFT_DOWN, wxMouseEventHandler( guCoverEditor::OnCoverLeftClick ), NULL, this );
+	Connect( wxEVT_MOUSEWHEEL, wxMouseEventHandler( guCoverEditor::OnMouseWheel ), NULL, this );
 
 	m_PrevButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guCoverEditor::OnPrevButtonClick ), NULL, this );
 	m_NextButton->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guCoverEditor::OnNextButtonClick ), NULL, this );
@@ -180,6 +186,7 @@ guCoverEditor::~guCoverEditor()
 {
     guConfig * Config = ( guConfig * ) guConfig::Get();
     Config->WriteNum( wxT( "CoverSearchEngine" ), m_EngineChoice->GetSelection(), wxT( "General" ) );
+    Config->WriteBool( wxT( "EmbedToFiles" ), m_EmbedToFilesChkBox->GetValue(), wxT( "General" ) );
 
     m_DownloadThreadMutex.Lock();
     int index;
@@ -209,6 +216,7 @@ guCoverEditor::~guCoverEditor()
 	m_EngineChoice->Disconnect( wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( guCoverEditor::OnEngineChanged ), NULL, this );
 
 	m_CoverBitmap->Disconnect( wxEVT_LEFT_DCLICK, wxMouseEventHandler( guCoverEditor::OnCoverLeftDClick ), NULL, this );
+	m_CoverBitmap->Disconnect( wxEVT_LEFT_DOWN, wxMouseEventHandler( guCoverEditor::OnCoverLeftClick ), NULL, this );
 
 	m_PrevButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guCoverEditor::OnPrevButtonClick ), NULL, this );
 	m_NextButton->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guCoverEditor::OnNextButtonClick ), NULL, this );
@@ -290,6 +298,27 @@ void guCoverEditor::OnNextButtonClick( wxCommandEvent& event )
     }
 }
 
+// -------------------------------------------------------------------------------- //
+void guCoverEditor::OnMouseWheel( wxMouseEvent &event )
+{
+    int Rotation = event.GetWheelRotation() / event.GetWheelDelta() * -1;
+    int NewPos = m_CurrentImage + Rotation;
+
+    //guLogMessage( wxT( "MouseWheel %i => %i" ), m_CurrentImage, NewPos );
+
+    NewPos = wxMax( 0, wxMin( NewPos, ( int ) m_AlbumCovers.Count() - 1 ) );
+
+    if( NewPos != m_CurrentImage )
+    {
+        m_CurrentImage = NewPos;
+
+        m_NextButton->Enable( m_CurrentImage < ( int ) m_AlbumCovers.Count() - 1 );
+        m_PrevButton->Enable( m_CurrentImage );
+
+        UpdateCoverBitmap();
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 void guCoverEditor::OnAddCoverImage( wxCommandEvent &event )
 {
@@ -344,7 +373,24 @@ void guCoverEditor::UpdateCoverBitmap( void )
 // -------------------------------------------------------------------------------- //
 void guCoverEditor::OnCoverLeftDClick( wxMouseEvent &event )
 {
-    EndModal( wxID_OK );
+    if( event.m_x >= 75 && event.m_x <= 225 )
+        EndModal( wxID_OK );
+}
+
+// -------------------------------------------------------------------------------- //
+void guCoverEditor::OnCoverLeftClick( wxMouseEvent &event )
+{
+    wxCommandEvent CmdEvent;
+    if( event.m_x < 75 )
+    {
+        OnPrevButtonClick( CmdEvent );
+    }
+    else if( event.m_x > 225 )
+    {
+        OnNextButtonClick( CmdEvent );
+    }
+    //guLogMessage( wxT( "%i %i" ), event.m_x, event.m_y );
+    event.Skip();
 }
 
 // -------------------------------------------------------------------------------- //
