@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -35,6 +35,7 @@
 #include <wx/dialog.h>
 #include <wx/gauge.h>
 #include <wx/choice.h>
+#include <wx/checkbox.h>
 
 #define GUCOVERINFO_LINK    0
 #define GUCOVERINFO_SIZE    1
@@ -133,6 +134,8 @@ class guCoverEditor : public wxDialog
     guAutoPulseGauge *          m_Gauge;
     wxStaticText *              m_InfoTextCtrl;
 
+    wxCheckBox *                m_EmbedToFilesChkBox;
+
 	guCoverImageArray           m_AlbumCovers;
 	guFetchCoverLinksThread *   m_DownloadCoversThread;
 	guThreadArray               m_DownloadThreads;
@@ -144,6 +147,7 @@ class guCoverEditor : public wxDialog
 	void OnTextCtrlEnter( wxCommandEvent& event );
     void OnEngineChanged( wxCommandEvent& event );
 	void OnCoverLeftDClick( wxMouseEvent& event );
+	void OnCoverLeftClick( wxMouseEvent& event );
 	void OnPrevButtonClick( wxCommandEvent& event );
 	void OnNextButtonClick( wxCommandEvent& event );
 	void OnAddCoverImage( wxCommandEvent &event );
@@ -152,11 +156,14 @@ class guCoverEditor : public wxDialog
 	void EndDownloadCoverThread( guDownloadCoverThread * DownloadCoverThread );
 	void OnDownloadedLinks( wxCommandEvent &event );
 
+	void OnMouseWheel( wxMouseEvent &event );
+
   public:
 	guCoverEditor( wxWindow * parent, const wxString &Artist, const wxString &Album );// wxWindowID id = wxID_ANY, const wxString& title = wxT("Cover Editor"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize( -1,-1 ), long style = wxDEFAULT_DIALOG_STYLE );
 	~guCoverEditor();
     wxString    GetSelectedCoverUrl( void );
     wxImage *   GetSelectedCoverImage( void );
+    bool        EmbedToFiles( void ) { return m_EmbedToFilesChkBox->IsChecked(); }
 
   friend class guFetchCoverLinksThread;
   friend class guDownloadCoverThread;
