@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -225,7 +225,7 @@ bool guDbCache::SetContent( const wxString &url, const wxString &content )
   ExecuteUpdate( query );
 
   // delete the expired entries but call it only 2% of the times
-  if( guRandom( 1000 ) < 20 )
+  if( guRandom( 1000 ) < 7 )
     ClearExpired();
 
   return true;
@@ -237,16 +237,13 @@ void guDbCache::ClearExpired( void )
     // last.fm queries are kept only 7 days
     wxString query = wxString::Format( wxT( "DELETE FROM cache WHERE cache_time < %u AND cache_type = %u" ),
         wxDateTime::Now().GetTicks() - 604800, guDBCACHE_TYPE_TEXT );
-
     ExecuteUpdate( query );
 
     // Images are kept 30 days
-    query = wxString::Format( wxT( "DELETE FROM cache WHERE cache_time < %u AND cache_type = %u" ),
-        wxDateTime::Now().GetTicks() - 2592000, guDBCACHE_TYPE_TEXT );
-
+    query = wxString::Format( wxT( "DELETE FROM cache WHERE cache_time < %u" ), wxDateTime::Now().GetTicks() - 2592000 );
     ExecuteUpdate( query );
 
-    //guLogMessage( wxT( "Delete expired Cache elements done" ) );
+    guLogMessage( wxT( "Delete expired Cache elements done" ) );
 }
 
 // -------------------------------------------------------------------------------- //
