@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2009 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -31,19 +31,34 @@
 #include <wx/utils.h>
 #include <wx/filefn.h>
 #include <wx/dir.h>
+#include <wx/dynarray.h>
 
 // wxSqlite3
 #include "wx/wxsqlite3.h"
 
+#define GU_TRACKS_QUERYSTR   wxT( "SELECT song_id, song_name, song_genreid, song_genre, song_artistid, song_artist, "\
+               "song_albumartistid, song_albumartist, song_composerid, song_composer, song_albumid, song_album, "\
+               "song_pathid, song_path, song_filename, song_format, song_disk, song_number, song_year, song_comment, "\
+               "song_coverid, song_offset, song_length, song_bitrate, song_rating, song_playcount, "\
+               "song_addedtime, song_lastplay, song_filesize "\
+               "FROM songs " )
+
 // PLAYLISTS
-#define GUPLAYLIST_STATIC       0
-#define GUPLAYLIST_DYNAMIC      1
+#define guPLAYLIST_TYPE_STATIC       0
+#define guPLAYLIST_TYPE_DYNAMIC      1
+
+extern unsigned long DynPLDateOption2[];
+
+#define guTRACK_TYPE_STOP_HERE      0x80000000
 
 enum guTrackType {
-    guTRACK_TYPE_DB,
+    guTRACK_TYPE_DB = 0,
     guTRACK_TYPE_NOTDB,
     guTRACK_TYPE_RADIOSTATION,
-    guTRACK_TYPE_PODCAST
+    guTRACK_TYPE_PODCAST,
+    guTRACK_TYPE_JAMENDO,
+    guTRACK_TYPE_MAGNATUNE,
+    guTRACK_TYPE_IPOD
 };
 
 enum guTrackMode {
@@ -53,6 +68,14 @@ enum guTrackMode {
     guTRACK_MODE_RADIO
 };
 
+enum guRandomMode {
+    guRANDOM_MODE_TRACK,
+    guRANDOM_MODE_ALBUM
+};
+
+class guLibPanel;
+class guDbLibrary;
+
 // -------------------------------------------------------------------------------- //
 class guTrack
 {
@@ -64,14 +87,18 @@ class guTrack
     wxString        m_AlbumName;
     int             m_ArtistId;
     wxString        m_ArtistName;
+    int             m_AlbumArtistId;
+    wxString        m_AlbumArtist;
     int             m_GenreId;
     wxString        m_GenreName;
     int             m_PathId;
-    wxString        m_FileName;           // mp3 filename
+    wxString        m_Path;
+    wxString        m_FileName;           // mp3 path + filename
     unsigned int    m_FileSize;
     int             m_Number;             // the track num of the song into the album
     int             m_Year;               // the year of the song
     unsigned int    m_Length;             // the length of the song in seconds
+    int             m_Offset;
     int             m_Bitrate;
     int             m_Rating;
     int             m_PlayCount;
@@ -82,15 +109,33 @@ class guTrack
     wxString        m_Disk;
     wxString        m_Comments;
     wxString        m_Composer;
+    int             m_ComposerId;
+    wxString        m_Format;
     //
     guTrackMode     m_TrackMode;          // Indicate how the track was created
+    guLibPanel *    m_LibPanel;
 
     guTrack() {
+        m_LibPanel = NULL;
         m_Type = guTRACK_TYPE_DB;
         m_TrackMode = guTRACK_MODE_USER;
+        m_Bitrate = 0;
+        m_Number = 0;
+        m_Year = 0;
+        m_Length = 0;
+        m_Offset = 0;
+        m_Bitrate = 0;
+        m_Rating = wxNOT_FOUND;
+        m_PlayCount = 0;
+        m_LastPlay = 0;
+        m_AddedTime = 0;
+        m_CoverId = 0;
     };
 
     ~guTrack() {};
+
+    bool ReadFromFile( const wxString &file );
+
 };
 WX_DECLARE_OBJARRAY(guTrack, guTrackArray);
 
@@ -122,6 +167,7 @@ class guArrayListItem //: public wxObject
     void DelData( int m_Id ) { m_Data.Remove( m_Id ); };
     wxArrayInt GetData( void ) { return m_Data; };
     int GetId() { return m_Id; };
+    int Index( int id ) { return m_Data.Index( id ); }
 //    const wxString GetName( void ) { return m_Name; };
 //    void SetName( const wxString &NewName ) { m_Name = NewName; };
 };
@@ -155,6 +201,7 @@ class guAlbumItem : public guListItem
     wxString    m_CoverPath;
     wxBitmap *  m_Thumb;
     int         m_Year;
+    wxString    m_ArtistName;
 
     guAlbumItem() : guListItem()
     {
@@ -177,26 +224,17 @@ class guAlbumItem : public guListItem
 };
 WX_DECLARE_OBJARRAY(guAlbumItem,guAlbumItems);
 
-// -------------------------------------------------------------------------------- //
-class guRadioStation
-{
-  public :
-    int         m_Id;
-    long        m_SCId;
-    wxString    m_Name;
-    wxString    m_Link;
-    long        m_GenreId;
-    bool        m_IsUser;
-    wxString    m_Type;
-    long        m_BitRate;
-    long        m_Listeners;
+enum guRadioSource {
+    guRADIO_SOURCE_GENRE = 0,
+    guRADIO_SOURCE_USER,
+    guRADIO_SOURCE_SEARCH
 };
-WX_DECLARE_OBJARRAY(guRadioStation,guRadioStations);
 
 enum guTRACKS_ORDER {
     guTRACKS_ORDER_NUMBER,
     guTRACKS_ORDER_TITLE,
     guTRACKS_ORDER_ARTIST,
+    guTRACKS_ORDER_ALBUMARTIST,
     guTRACKS_ORDER_ALBUM,
     guTRACKS_ORDER_GENRE,
     guTRACKS_ORDER_COMPOSER,
@@ -207,20 +245,19 @@ enum guTRACKS_ORDER {
     guTRACKS_ORDER_RATING,
     guTRACKS_ORDER_PLAYCOUNT,
     guTRACKS_ORDER_LASTPLAY,
-    guTRACKS_ORDER_ADDEDDATE
+    guTRACKS_ORDER_ADDEDDATE,
+    guTRACKS_ORDER_FORMAT,
+    guTRACKS_ORDER_FILEPATH
 };
 
-#define guRADIOSTATIONS_ORDER_NAME        0
-#define guRADIOSTATIONS_ORDER_BITRATE     1
-#define guRADIOSTATIONS_ORDER_LISTENERS   2
-
 enum guALBUMS_ORDER {
     guALBUMS_ORDER_NAME = 0,
     guALBUMS_ORDER_YEAR,
     guALBUMS_ORDER_YEAR_REVERSE,
     guALBUMS_ORDER_ARTIST_NAME,
     guALBUMS_ORDER_ARTIST_YEAR,
-    guALBUMS_ORDER_ARTIST_YEAR_REVERSE
+    guALBUMS_ORDER_ARTIST_YEAR_REVERSE,
+    guALBUMS_ORDER_ADDEDTIME
 };
 
 // -------------------------------------------------------------------------------- //
@@ -233,77 +270,72 @@ class guAS_SubmitInfo //guAudioScrobbler_SubmitInfo
     wxString m_TrackName;
     int      m_PlayedTime; // UTC Time since 1/1/1970
     wxChar   m_Source;
-    wxChar   m_Ratting;
+    wxChar   m_Rating;
     int      m_TrackLen;
     int      m_TrackNum;
     int      m_MBTrackId;
 };
 WX_DECLARE_OBJARRAY(guAS_SubmitInfo, guAS_SubmitInfoArray);
 
-#define GULIBRARY_FILTER_LABELS     0
-#define GULIBRARY_FILTER_GENRES     1
-#define GULIBRARY_FILTER_ARTISTS    2
-#define GULIBRARY_FILTER_ALBUMS     3
-#define GULIBRARY_FILTER_SONGS      4
+enum guLIBRARY_FILTER {
+    guLIBRARY_FILTER_LABELS = 0,
+    guLIBRARY_FILTER_GENRES,
+    guLIBRARY_FILTER_COMPOSERS,
+    guLIBRARY_FILTER_ALBUMARTISTS,
+    guLIBRARY_FILTER_ARTISTS,
+    guLIBRARY_FILTER_YEARS,
+    guLIBRARY_FILTER_ALBUMS,
+    guLIBRARY_FILTER_RATINGS,
+    guLIBRARY_FILTER_PLAYCOUNTS,
+    guLIBRARY_FILTER_SONGS
+};
 
 class guDynPlayList;
 class guAlbumBrowserItemArray;
+class guTreeViewFilterArray;
+class guCurrentTrack;
 
 // -------------------------------------------------------------------------------- //
 class guDbLibrary : public guDb
 {
   protected :
-//    wxSQLite3Database  m_Db;
     wxArrayString      m_LibPaths;
-    guTrack            m_CurSong;
     bool               m_NeedUpdate;
-    //wxString           m_UpTag;
 
     // Library Filter Options
     wxArrayInt         m_GeFilters;
-    wxArrayInt         m_LaFilters;
-    wxArrayInt         m_ArFilters;
-    wxArrayInt         m_AlFilters;
+    wxArrayInt         m_LaFilters; // Label
+    wxArrayInt         m_CoFilters; // Composers
+    wxArrayInt         m_AAFilters; // AlbumArtist
+    wxArrayInt         m_ArFilters; // Artist
+    wxArrayInt         m_AlFilters; // Album
     wxArrayString      m_TeFilters; // Text string filters
-
-    // Radio Filters Options
-    wxArrayInt         m_RaGeFilters;
-    bool               m_RadioIsUser;
-    wxArrayInt         m_RaLaFilters;
-    wxArrayString      m_RaTeFilters;
+    wxArrayInt         m_YeFilters; // Year
+    wxArrayInt         m_RaFilters; // Ratting
+    wxArrayInt         m_PcFilters; // PlayCount
 
     guTRACKS_ORDER     m_TracksOrder;
     bool               m_TracksOrderDesc;
-    int                m_StationsOrder; // 0 -> Name, 1 -> BitRate, 2 -> Listeners
-    bool               m_StationsOrderDesc;
     int                m_AlbumsOrder;   // 0 ->
 
-//    guListItems         Labels;
-    guListItems         m_GenresCache;
-    guListItems         m_ArtistsCache;
-    guAlbumItems        m_AlbumsCache;
-    guListItems         m_PathsCache;
-
     wxArrayString       m_CoverSearchWords;
 
     // Podcasts
-    wxArrayInt          m_PodChFilters;
-    int                 m_PodcastOrder;
-    bool                m_PodcastOrderDesc;
+    //wxArrayInt          m_PodChFilters;
+    //int                 m_PodcastOrder;
+    //bool                m_PodcastOrderDesc;
 
     wxString            FiltersSQL( int Level );
 
 
     void inline         FillTrackFromDb( guTrack * Song, wxSQLite3ResultSet * dbRes );
-    int                 GetRadioFiltersCount( void ) const;
-    wxString            RadioFiltersSQL( void );
 
   public :
                         guDbLibrary();
                         guDbLibrary( const wxString &DbName );
+                        guDbLibrary( guDb * db );
                         ~guDbLibrary();
-//    int                 Open( const wxString &DbPath );
-//    int                 Close();
+
     virtual int         GetDbVersion( void );
 
     void                ConfigChanged( void );
@@ -311,55 +343,76 @@ class guDbLibrary : public guDb
     bool                NeedUpdate( void ) { return m_NeedUpdate; };
 
     void                DoCleanUp( void );
+    void                CleanItems( const wxArrayInt &tracks, const wxArrayInt &covers );
+    void                CleanFiles( const wxArrayString &files );
     virtual bool        CheckDbVersion( void );
-    void                LoadCache( void );
+    //void                LoadCache( void );
 
     int                 GetGenreId( wxString &GenreName );
+    int                 GetComposerId( wxString &composername, bool create = true );
     const wxString      GetArtistName( const int ArtistId );
     int                 GetArtistId( wxString &ArtistName, bool Create = true );
+    int                 GetAlbumArtistId( wxString &albumartist, bool create = true );
+    int                 AddCoverImage( const wxImage &image );
     int                 AddCoverFile( const wxString &coverfile, const wxString &coverhash = wxEmptyString );
     void                UpdateCoverFile( int coverid, const wxString &coverfile, const wxString &coverhash );
     int                 FindCoverFile( const wxString &DirName );
     int                 SetAlbumCover( const int AlbumId, const wxString & CoverPath, const wxString &coverhash = wxEmptyString );
+    int                 SetAlbumCover( const int AlbumId, const wxImage &image );
     bool                GetAlbumInfo( const int AlbumId, wxString * AlbumName, wxString * ArtistName, wxString * AlbumPath );
     int                 GetAlbumCoverId( const int AlbumId );
 
     wxString            GetCoverPath( const int CoverId );
     int                 GetAlbumId( wxString &AlbumName, const int ArtistId, const int PathId, const wxString &Path, const int coverid = 0 );
-    int                 GetSongId( wxString &filename, const int pathid );
-    int                 GetSongId( wxString &FileName, wxString &FilePath );
+    int                 GetSongId( wxString &filename, const int pathid, bool * created = NULL );
+    int                 GetSongId( wxString &FileName, wxString &FilePath, bool * created = NULL );
     wxArrayInt          GetLabelIds( const wxArrayString &Labels );
     wxArrayString       GetLabelNames( const wxArrayInt &LabelIds );
     int                 GetLabelId( int * LabelId, wxString &LabelName );
     int                 PathExists( const wxString &path );
     int                 GetPathId( wxString &PathValue );
-    int                 UpdateSong( void );
+    virtual int         UpdateSong( const guTrack &track, const bool allowrating = false );
+    int                 AddFiles( const wxArrayString &files );
 
-    void                GetGenres( guListItems * Genres, bool FullList = false );
-    void                GetLabels( guListItems * Labels, bool FullList = false );
-    void                GetRadioLabels( guListItems * Labels, bool FullList = false );
-    void                GetArtists( guListItems * Artists, bool FullList = false );
+    void                GetGenres( guListItems * Genres, const bool FullList = false );
+    void                GetLabels( guListItems * Labels, const bool FullList = false );
+    void                GetArtists( guListItems * Artists, const bool FullList = false );
+    void                GetYears( guListItems * items, const bool FullList = false );
+    void                GetRatings( guListItems * items, const bool FullList = false );
+    void                GetPlayCounts( guListItems * items, const bool FullList = false );
+    void                GetAlbumArtists( guListItems * items, const bool FullList = false );
+    void                GetComposers( guListItems * items, const bool FullList = false );
     void                SetAlbumsOrder( const int order );
     int                 GetAlbumsOrder( void ) { return m_AlbumsOrder; };
+    void                GetAlbums( guListItems * Albums, bool FullList = false );
     void                GetAlbums( guAlbumItems * Albums, bool FullList = false );
     wxArrayString       GetAlbumsPaths( const wxArrayInt &AlbumIds );
     int                 GetAlbums( guAlbumBrowserItemArray * items, guDynPlayList * filter,
-                                   const int start, const int count, const int order );
-    int                 GetAlbumsCount( guDynPlayList * filter );
+                            const wxArrayString &textfilters, const int start, const int count, const int order );
+    int                 GetAlbumsCount( guDynPlayList * filter, const wxArrayString &textfilters );
+    bool                GetAlbumDetails( const int albumid, int * year, int * trackcount );
     int                 GetAlbumYear( const int albumid );
     int                 GetAlbumTrackCount( const int albumid );
 
     void                GetPaths( guListItems * Paths, bool FullList = false );
 
     bool                GetSong( const int songid, guTrack * song );
+    void                GetSongsCounters( const guTreeViewFilterArray &filters, const wxArrayString &textfilters, wxLongLong * count, wxLongLong * len, wxLongLong * size );
+    int                 GetSongs( const guTreeViewFilterArray &filters, guTrackArray * Songs, const wxArrayString &textfilters, const int order, const bool orderdesc );
     int                 GetSongs( const wxArrayInt &SongIds, guTrackArray * Songs );
-    int                 GetSongs( guTrackArray * Songs );
+    int                 GetSongsCount( void );
+    int                 GetSongs( guTrackArray * Songs, const int start, const int end );
+    int                 GetSongsId( const int start );
+    wxString            GetSongsName( const int start );
     void                GetTracksCounters( wxLongLong * count, wxLongLong * len, wxLongLong * size );
+    void                DeleteLibraryTracks( const guTrackArray * tracks, const bool savedeleted );
+    bool                FindDeletedFile( const wxString &file, const bool create );
 
     void                SetSongsOrder( const guTRACKS_ORDER order );
     guTRACKS_ORDER      GetSongsOrder( void ) const;
     bool                GetSongsOrderDesc( void ) const;
-    void                UpdateSongs( guTrackArray * Songs );
+    void                UpdateSongs( const guTrackArray * Songs, const wxArrayInt &changedflags );
+    void                UpdateTracksWithValue( guTrackArray * tracks, const int fieldid, void * value );
     void                UpdateTrackLength( const int trackid, const int length );
     void                UpdateTrackBitRate( const int trackid, const int bitrate );
     int                 GetAlbumsSongs( const wxArrayInt &Albums, guTrackArray * Songs, bool ordertoedit = false );
@@ -368,8 +421,9 @@ class guDbLibrary : public guDb
     int                 GetArtistsSongs( const wxArrayInt &Artists, guTrackArray * Songs,
                                          const int count, const int filterallow, const int filterdeny );
     int                 GetArtistsAlbums( const wxArrayInt &Artists, wxArrayInt * Albums );
+    int                 GetAlbumArtistsAlbums( const wxArrayInt &albumartists, wxArrayInt * Albums );
     int                 GetGenresSongs( const wxArrayInt &Genres, guTrackArray * Songs );
-    int                 GetRandomTracks( guTrackArray * Tracks, const int count,
+    int                 GetRandomTracks( guTrackArray * Tracks, const int count, const int rndmode,
                                          const int allowplaylist, const int denyplaylist );
     int                 GetLabelsSongs( const wxArrayInt &Labels, guTrackArray * Songs );
     int                 AddLabel( wxString LabelName );
@@ -377,40 +431,54 @@ class guDbLibrary : public guDb
     int                 DelLabel( const int LabelId );
     wxArrayInt          GetLabels( void );
 
-    int                 GetRadioLabelsSongs( const wxArrayInt &Labels, guTrackArray * Songs );
-    int                 AddRadioLabel( wxString LabelName );
-    int                 SetRadioLabelName( const int LabelId, wxString NewName );
-    int                 DelRadioLabel( const int LabelId );
-    wxArrayInt          GetRadioLabels( void );
+    int                 GetStaticPlayList( const wxString &name );
+    virtual int         CreateStaticPlayList( const wxString &name, const wxArrayInt &tracks );
+    virtual int         UpdateStaticPlayList( const int plid, const wxArrayInt &tracks );
+    virtual int         AppendStaticPlayList( const int plid, const wxArrayInt &tracks );
+    virtual void        UpdateStaticPlayListFile( const int plid );
 
-    int                 CreateStaticPlayList( const wxString &name, const wxArrayInt &tracks );
-    int                 UpdateStaticPlayList( const int plid, const wxArrayInt &tracks );
-    int                 AppendStaticPlayList( const int plid, const wxArrayInt &tracks );
-    int                 DelPlaylistSetIds( const int plid, const wxArrayInt &setids );
+    virtual int         DelPlaylistSetIds( const int plid, const wxArrayInt &setids );
     int                 GetPlayListFiles( const int plid, wxFileDataObject * Files );
-    void                GetPlayLists( guListItems * PlayLists, const int type );
+    int                 GetPlayListsCount( void );
+    void                GetPlayLists( guListItems &playlists );
+    void                GetPlayLists( wxArrayString &playlistnames );
+    void                GetPlayLists( wxArrayInt &playlistids );
+    void                GetPlayLists( guListItems * PlayLists, const int type, const wxArrayString * textfilters = NULL );
     int                 GetPlayListSongIds( const int plid, wxArrayInt * tracks );
+    int                 GetPlayListSongs( const int plid, guTrackArray * tracks );
     int                 GetPlayListSongs( const int plid, const int pltype, guTrackArray * tracks,
                             wxLongLong * len, wxLongLong * size );
+    int                 GetPlayListSongs( const wxArrayInt &ids, const wxArrayInt &types, guTrackArray * tracks,
+                            wxLongLong * len, wxLongLong * size );
     int                 GetPlayListSetIds( const int plid, wxArrayInt * setids );
-    void                DeletePlayList( const int plid );
-    void                SetPlayListName( const int plid, const wxString &plname );
+    int                 GetPlayListSetIds( const wxArrayInt &plid, wxArrayInt * setids );
+    virtual void        DeletePlayList( const int plid );
+    virtual void        SetPlayListName( const int plid, const wxString &plname );
+    wxString            GetPlayListName( const int plid );
+    wxString            GetPlayListPath( const int plid );
+    void                SetPlayListPath( const int plid, const wxString &path );
     void                GetDynamicPlayList( const int plid, guDynPlayList * playlist );
-    int                 CreateDynamicPlayList( const wxString &name, guDynPlayList * playlist );
-    void                UpdateDynPlayList( const int plid, const guDynPlayList * playlist );
+    virtual int         CreateDynamicPlayList( const wxString &name, const guDynPlayList * playlist );
+    virtual void        UpdateDynamicPlayList( const int plid, const guDynPlayList * playlist );
     wxString            GetPlayListQuery( const int plid );
     int                 GetPlayListType( const int plid );
 
     void                SetLibPath( const wxArrayString &NewPaths );
-    int                 ReadFileTags( const char * filename );
-    void                UpdateImageFile( const char * filename );
+    int                 ReadFileTags( const wxString &filename );
+    void                UpdateImageFile( const char * filename, const char * saveto,
+                                         const int type = wxBITMAP_TYPE_JPEG, const int maxsize = wxNOT_FOUND );
 
     int                 GetFiltersCount() const;
-    void                SetTeFilters( const wxArrayString &NewTeFilters );
-    void                SetGeFilters( const wxArrayInt &NewGeFilters );
-    void                SetTaFilters( const wxArrayInt &NewTaFilters );
-    void                SetArFilters( const wxArrayInt &NewArFilters );
-    void                SetAlFilters( const wxArrayInt &NewAlFilters );
+    void                SetTeFilters( const wxArrayString &tefilters, const bool locked );
+    void                SetGeFilters( const wxArrayInt &filters, const bool locked  );
+    void                SetLaFilters( const wxArrayInt &filters, const bool locked );
+    void                SetArFilters( const wxArrayInt &filters, const bool locked );
+    void                SetAlFilters( const wxArrayInt &filters, const bool locked );
+    void                SetYeFilters( const wxArrayInt &filter, const bool locked );
+    void                SetRaFilters( const wxArrayInt &filter );
+    void                SetPcFilters( const wxArrayInt &filter );
+    void                SetCoFilters( const wxArrayInt &filter, const bool locked );
+    void                SetAAFilters( const wxArrayInt &filter, const bool locked );
 
     //
     guArrayListItems    GetArtistsLabels( const wxArrayInt &Artists );
@@ -420,21 +488,23 @@ class guDbLibrary : public guDb
     void                SetAlbumsLabels( const wxArrayInt &Albums, const wxArrayInt &Labels );
     void                SetSongsLabels( const wxArrayInt &Songs, const wxArrayInt &Labels );
 
-    void                UpdateArtistsLabels( const wxArrayInt &Artists, const wxArrayInt &Labels );
-    void                UpdateAlbumsLabels( const wxArrayInt &Albums, const wxArrayInt &Labels );
-    void                UpdateSongsLabels( const wxArrayInt &Songs, const wxArrayInt &Labels );
-    void                UpdateSongsLabel( const guTrackArray * tracks, const wxString &label, const wxString &newlabel );
+    virtual void        UpdateArtistsLabels( const guArrayListItems &labelsets );
+    virtual void        UpdateAlbumsLabels( const guArrayListItems &labelsets );
+    virtual void        UpdateSongsLabels( const guArrayListItems &labelsets );
+    virtual void        UpdateSongsLabel( const guTrackArray * tracks, const wxString &label, const wxString &newlabel );
 
-    void                SetTrackRating( const int songid, const int rating );
-    void                SetTracksRating( const wxArrayInt &songids, const int rating );
+    void                SetTrackRating( const int songid, const int rating, const bool writetags = false );
+    void                SetTracksRating( const wxArrayInt &songids, const int rating, const bool writetags = false );
+    void                SetTracksRating( const guTrackArray * tracks, const int rating, const bool writetags = false );
 
-    void                SetTrackPlayCount( const int songid, const int playcount );
+    void                SetTrackPlayCount( const int songid, const int playcount, const bool writetags = false );
 
 
-//    wxSQLite3ResultSet  ExecuteQuery(  const wxSQLite3StatementBuffer &query );
-//    int                 ExecuteUpdate( const wxString &query );
-//    int                 ExecuteUpdate( const wxSQLite3StatementBuffer &query );
-//    wxSQLite3ResultSet  ExecuteQuery( const wxString &query );
+    int                 GetYearsSongs( const wxArrayInt &Years, guTrackArray * Songs );
+    int                 GetRatingsSongs( const wxArrayInt &Ratings, guTrackArray * Songs );
+    int                 GetPlayCountsSongs( const wxArrayInt &PlayCounts, guTrackArray * Songs );
+    int                 GetComposersSongs( const wxArrayInt &Composers, guTrackArray * Songs );
+    int                 GetAlbumArtistsSongs( const wxArrayInt &albumartists, guTrackArray * tracks );
 
     wxBitmap *          GetCoverBitmap( const int coverid, const bool thumb = true );
     guCoverInfos        GetEmptyCovers( void );
@@ -443,40 +513,16 @@ class guDbLibrary : public guDb
     int                 FindArtist( const wxString &Artist );
     int                 FindAlbum( const wxString &Artist, const wxString &Album );
     int                 FindTrack( const wxString &Artist, const wxString &m_Name );
+    int                 GetTrackIndex( const int trackid );
     guTrack *           FindSong( const wxString &artist, const wxString &track,
                                   const int filterallow, const int filterdeny );
-    int                 FindTrackFile( const wxString &filename, guTrack * song );
-
-    //
-    // Radio support functions
-    //
-    void                    SetRaTeFilters( const wxArrayString &filters );
-    void                    SetRadioLabelsFilters( const wxArrayInt &filters );
-    void                    SetRadioGenresFilters( const wxArrayInt &filters );
-    void                    SetRadioIsUserFilter( bool isuserradio );
-    void                    GetRadioGenresList( guListItems * RadioGenres, const wxArrayInt &GenreIds );
-    void                    GetRadioGenres( guListItems * RadioGenres, bool AllowFilter = true );
-    void                    SetRadioGenres( const wxArrayString &Genres );
-    int                     GetRadioStations( guRadioStations * stations );
-    void                    GetRadioCounter( wxLongLong * count );
-    int                     GetUserRadioStations( guRadioStations * stations );
-    void                    SetRadioStation( const guRadioStation * RadioStation );
-    bool                    GetRadioStation( const int id, guRadioStation * radiostation );
-    void                    SetRadioStations( const guRadioStations * RadioStations );
-    guArrayListItems        GetStationsLabels( const wxArrayInt &Stations );
-    void                    SetRadioStationsLabels( const wxArrayInt &Stations, const wxArrayInt &Labels );
-    int                     DelRadioStations( const wxArrayInt &RadioGenresIds );
-    int                     DelRadioStation( const int radioid );
-    void                    SetRadioStationsOrder( int OrderValue );
-
-    int                     AddRadioGenre( wxString GenreName );
-    int                     SetRadioGenreName( const int GenreId, wxString GenreName );
-    int                     DelRadioGenre( const int GenreId );
+    int                 FindTrackFile( const wxString &filename, guTrack * track );
+    int                 FindTrackId( const int trackid, guTrack * track );
 
     //
     // AudioScrobbler functions
     //
-    bool                    AddCachedPlayedSong( const guTrack &Song );
+    bool                    AddCachedPlayedSong( const guCurrentTrack &Song );
     guAS_SubmitInfoArray    GetCachedPlayedSongs( int MaxCount = 10 );
     bool                    DeleteCachedPlayedSongs( const guAS_SubmitInfoArray &SubmitInfo );
 
@@ -496,9 +542,9 @@ class guDbLibrary : public guDb
     int                     GetPodcastChannelId( const int id, guPodcastChannel * channel = NULL );
     void                    DelPodcastChannel( const int id );
 
-    int                     GetPodcastItems( guPodcastItemArray * items );
-    void                    GetPodcastCounters( wxLongLong * count, wxLongLong * len, wxLongLong * size );
-    int                     GetPodcastItems( const wxArrayInt &ids, guPodcastItemArray * items );
+    int                     GetPodcastItems( guPodcastItemArray * items, const wxArrayInt &filters, const int order, const bool desc );
+    void                    GetPodcastCounters( const wxArrayInt &filters, wxLongLong * count, wxLongLong * len, wxLongLong * size );
+    int                     GetPodcastItems( const wxArrayInt &ids, guPodcastItemArray * items, const int order, const bool desc );
     void                    SavePodcastItem( const int channelid, guPodcastItem * item, bool onlynew = false );
     void                    SavePodcastItems( const int channelid, guPodcastItemArray * items, bool onlynew = false );
     void                    SetPodcastItemStatus( const int itemid, const int status );
@@ -509,11 +555,15 @@ class guDbLibrary : public guDb
     int                     GetPodcastItemFile( const wxString &filename, guPodcastItem * item = NULL );
     void                    DelPodcastItem( const int itemid );
     void                    DelPodcastItems( const int channelid );
-    void                    SetPodcastChannelFilters( const wxArrayInt &filters );
-    void                    SetPodcastOrder( int order );
+    //void                    SetPodcastChannelFilters( const wxArrayInt &filters );
+    //void                    SetPodcastOrder( int order );
     int                     GetPendingPodcasts( guPodcastItemArray * items );
     int                     GetPodcastFiles( const wxArrayInt &channelid, wxFileDataObject * files );
 
+    // File Browser related functions
+    void                    UpdateTrackFileName( const wxString &oldname, const wxString &newname );
+    void                    UpdatePaths( const wxString &oldpath, const wxString &newpath );
+
 };
 
 // -------------------------------------------------------------------------------- //
@@ -526,6 +576,74 @@ int guListItemSearch( const guListItems &Items, int StartPos, int EndPos, int m_
 wxString guListItemsGetName( const guListItems &Items, int m_Id );
 wxArrayInt GetArraySameItems( const wxArrayInt &Source, const wxArrayInt &Oper );
 wxArrayInt GetArrayDiffItems( const wxArrayInt &Source, const wxArrayInt &Oper );
+wxString TextFilterToSQL( const wxArrayString &TeFilters );
+wxString LabelFilterToSQL( const wxArrayInt &LaFilters );
+
+// -------------------------------------------------------------------------------- //
+wxString inline ArrayIntToStrList( const wxArrayInt &Data )
+{
+  int index;
+  int count;
+  wxString RetVal = wxT( "(" );
+  count = Data.Count();
+  for( index = 0; index < count; index++ )
+  {
+    RetVal += wxString::Format( wxT( "%u," ), Data[ index ]  );
+  }
+  if( RetVal.Length() > 1 )
+    RetVal = RetVal.RemoveLast() + wxT( ")" );
+  else
+    RetVal += wxT( ")" );
+  return RetVal;
+}
+
+// -------------------------------------------------------------------------------- //
+wxString inline ArrayToFilter( const wxArrayInt &Filters, const wxString &VarName )
+{
+  return VarName + wxT( " IN " ) + ArrayIntToStrList( Filters );
+}
+
+// -------------------------------------------------------------------------------- //
+void inline guDbLibrary::FillTrackFromDb( guTrack * Song, wxSQLite3ResultSet * dbRes )
+{
+/*
+#define GU_TRACKS_QUERYSTR   wxT( "SELECT song_id, song_name, song_genreid, song_genre, song_artistid, song_artist, "\
+               "song_albumartistid, song_albumartist, song_composerid, song_composer, song_albumid, song_album, "\
+               "song_pathid, song_path, song_filename, song_format, song_disk, song_number, song_year, song_comment, "\
+               "song_coverid, song_offset, song_length, song_bitrate, song_rating, song_playcount, "\
+               "song_addedtime, song_lastplay, song_filesize "\
+               "FROM songs " )
+*/
+  Song->m_SongId        = dbRes->GetInt( 0 );
+  Song->m_SongName      = dbRes->GetString( 1 );
+  Song->m_GenreId       = dbRes->GetInt( 2 );
+  Song->m_GenreName     = dbRes->GetString( 3 );
+  Song->m_ArtistId      = dbRes->GetInt( 4 );
+  Song->m_ArtistName    = dbRes->GetString( 5 );
+  Song->m_AlbumArtistId = dbRes->GetInt( 6 );
+  Song->m_AlbumArtist   = dbRes->GetString( 7 );
+  Song->m_ComposerId    = dbRes->GetInt( 8 );
+  Song->m_Composer      = dbRes->GetString( 9 );
+  Song->m_AlbumId       = dbRes->GetInt( 10 );
+  Song->m_AlbumName     = dbRes->GetString( 11 );
+  Song->m_PathId        = dbRes->GetInt( 12 );
+  Song->m_Path          = dbRes->GetString( 13 );
+  Song->m_FileName      = Song->m_Path + dbRes->GetString( 14 );
+  Song->m_Format        = dbRes->GetString( 15 );
+  Song->m_Disk          = dbRes->GetString( 16 );
+  Song->m_Number        = dbRes->GetInt( 17 );
+  Song->m_Year          = dbRes->GetInt( 18 );
+  Song->m_Comments      = dbRes->GetString( 19 );
+  Song->m_CoverId       = dbRes->GetInt( 20 );
+  Song->m_Offset        = dbRes->GetInt( 21 );
+  Song->m_Length        = dbRes->GetInt( 22 );
+  Song->m_Bitrate       = dbRes->GetInt( 23 );
+  Song->m_Rating        = dbRes->GetInt( 24 );
+  Song->m_PlayCount     = dbRes->GetInt( 25 );
+  Song->m_AddedTime     = dbRes->GetInt( 26 );
+  Song->m_LastPlay      = dbRes->GetInt( 27 );
+  Song->m_FileSize      = dbRes->GetInt( 28 );
+}
 
 #endif
 // -------------------------------------------------------------------------------- //
