@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,9 +21,11 @@
 #include "DynamicPlayList.h"
 
 #include "Images.h"
+#include "Config.h"
 #include "Utils.h"
 
 WX_DEFINE_OBJARRAY(guFilterItemArray);
+WX_DEFINE_OBJARRAY(guDynPlayListArray);
 
 wxArrayString m_FilterFieldChoices;
 wxArrayString m_FilterTextOptionChoices;
@@ -31,6 +33,7 @@ wxArrayString m_FilterLabelOptionChoices;
 wxArrayString m_FilterNumberOptionChoices;
 wxArrayString m_FilterYearOptionChoices;
 wxArrayString m_FilterDateOptionChoices;
+wxArrayString m_FilterBoolOptionChoices;
 wxArrayString m_LimitChoices;
 wxArrayString m_SortChoices;
 wxArrayString m_FilterDateOption2Choices;
@@ -55,6 +58,11 @@ void inline InitArrayStrings( void )
         m_FilterFieldChoices.Add( _( "Play Count" ) );
         m_FilterFieldChoices.Add( _( "Last Play Time" ) );
         m_FilterFieldChoices.Add( _( "Added Date" ) );
+        m_FilterFieldChoices.Add( _( "Track Number" ) );
+        m_FilterFieldChoices.Add( _( "Bitrate" ) );
+        m_FilterFieldChoices.Add( _( "Size" ) );
+        m_FilterFieldChoices.Add( _( "Disk" ) );
+        m_FilterFieldChoices.Add( _( "With Cover" ) );
 //    }
 //
 //    if( !m_FilterTextOptionChoices.Count() )
@@ -100,6 +108,11 @@ void inline InitArrayStrings( void )
         m_FilterDateOptionChoices.Add( _( "before the last" ) );
 //    }
 //
+//    if( !m_FilterBoolOptionChoices.Count() )
+//    {
+        m_FilterBoolOptionChoices.Add( _( "false" ) );
+        m_FilterBoolOptionChoices.Add( _( "true" ) );
+//    }
 //    if( !m_LimitChoices.Count() )
 //    {
         m_LimitChoices.Add( _( "Tracks" ) );
@@ -174,6 +187,7 @@ void guFilterItem::SetFilterLabel( void )
         case guDYNAMIC_FILTER_TYPE_COMPOSER :
         case guDYNAMIC_FILTER_TYPE_COMMENT :
         case guDYNAMIC_FILTER_TYPE_PATH :
+        case guDYNAMIC_FILTER_TYPE_DISK :
         {
             m_Label += m_FilterTextOptionChoices[ m_Option ] + wxT( " " );
             m_Label += m_Text;
@@ -196,6 +210,9 @@ void guFilterItem::SetFilterLabel( void )
 
         case guDYNAMIC_FILTER_TYPE_RATING : // Numbers
         case guDYNAMIC_FILTER_TYPE_PLAYCOUNT :
+        case guDYNAMIC_FILTER_TYPE_TRACKNUMBER :
+        case guDYNAMIC_FILTER_TYPE_BITRATE :
+        case guDYNAMIC_FILTER_TYPE_SIZE :
         {
             m_Label += m_FilterNumberOptionChoices[ m_Option ];
             m_Label += wxString::Format( wxT( " %u" ), m_Number );
@@ -215,30 +232,18 @@ void guFilterItem::SetFilterLabel( void )
             m_Label += m_FilterDateOptionChoices[ m_Option ];
             m_Label += wxString::Format( wxT( " %u " ), m_Number );
             m_Label += m_FilterDateOption2Choices[ m_Option2 ];
+            break;
+        }
+
+        case guDYNAMIC_FILTER_TYPE_HASARTWORK :
+        {
+            m_Label += m_FilterBoolOptionChoices[ m_Option ];
         }
     }
 }
 
 // -------------------------------------------------------------------------------- //
 // guDynPlayList
-// -------------------------------------------------------------------------------- //
-wxString inline escape_dynplaylist_str( const wxString &val )
-{
-    wxString RetVal = val;
-    RetVal.Replace( wxT( ":" ), wxT( "_$&" ) );
-    RetVal.Replace( wxT( ";" ), wxT( "_&$" ) );
-    return RetVal;
-}
-
-// -------------------------------------------------------------------------------- //
-wxString inline unescape_dynplaylist_str( const wxString &val )
-{
-    wxString RetVal = val;
-    RetVal.Replace( wxT( "_$&" ), wxT( ":" ) );
-    RetVal.Replace( wxT( "_&$" ), wxT( ";" ) );
-    return RetVal;
-}
-
 // -------------------------------------------------------------------------------- //
 wxString guDynPlayList::ToString( void )
 {
@@ -246,7 +251,7 @@ wxString guDynPlayList::ToString( void )
 
     RetVal += wxString::Format( wxT( "%i:%s:%i:%i:%i:%i:%i:%i:%i:" ),
         m_Id,
-        escape_dynplaylist_str( m_Name ).c_str(), // Need to escape the ':'
+        escape_configlist_str( m_Name ).c_str(), // Need to escape the ':'
         m_Limited,
         m_LimitValue,
         m_LimitType,
@@ -263,10 +268,10 @@ wxString guDynPlayList::ToString( void )
         RetVal += wxString::Format( wxT( "{%i;%i;%s;%i;%i;%s}:" ),
             FilterItem.m_Type,
             FilterItem.m_Option,
-            escape_dynplaylist_str( FilterItem.m_Text ).c_str(),
+            escape_configlist_str( FilterItem.m_Text ).c_str(),
             FilterItem.m_Number,
             FilterItem.m_Option2,
-            escape_dynplaylist_str( FilterItem.m_Label ).c_str() );
+            escape_configlist_str( FilterItem.m_Label ).c_str() );
     }
 
     return RetVal;
@@ -299,7 +304,7 @@ void ReadFilterFromString( guFilterItem * filteritem, wxString &filterstr )
 
             case 2 :
             {
-                filteritem->m_Text = unescape_dynplaylist_str( Val );
+                filteritem->m_Text = unescape_configlist_str( Val );
                 break;
             }
 
@@ -317,7 +322,7 @@ void ReadFilterFromString( guFilterItem * filteritem, wxString &filterstr )
 
             case 5 :
             {
-                filteritem->m_Label = unescape_dynplaylist_str( Val );
+                filteritem->m_Label = unescape_configlist_str( Val );
                 break;
             }
         }
@@ -346,6 +351,7 @@ void guDynPlayList::FromString( const wxString &playlist )
 {
     wxString Fields = playlist;
     int Field = 0;
+    m_Filters.Empty();
 
     while( Fields.Length() )
     {
@@ -370,7 +376,7 @@ void guDynPlayList::FromString( const wxString &playlist )
 
             case 2 :
             {
-                m_Name = unescape_dynplaylist_str( Val );
+                m_Name = unescape_configlist_str( Val );
                 break;
             }
 
@@ -665,7 +671,6 @@ void guDynPlayListEditor::FillPlayListEditData( void )
         m_PlayList->m_Sorted = m_SortCheckBox->IsChecked();
         m_PlayList->m_SortType = m_SortChoice->GetSelection();
         m_PlayList->m_SortDesc = m_DescCheckBox->IsChecked();
-        m_PlayList->m_AnyOption = m_AddOnAnyCheckBox->IsChecked();
     }
     else
     {
@@ -675,8 +680,8 @@ void guDynPlayListEditor::FillPlayListEditData( void )
         m_PlayList->m_Sorted = false;
         m_PlayList->m_SortType = 0;
         m_PlayList->m_SortDesc = false;
-        m_PlayList->m_AnyOption = false;
     }
+    m_PlayList->m_AnyOption = m_AddOnAnyCheckBox->IsChecked();
 }
 
 // -------------------------------------------------------------------------------- //
@@ -690,7 +695,8 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
                     m_FilterLabelOptionChoice->GetSelection(),
                     FilterItem.m_Text = m_FilterText->GetValue() );
     }
-    else if( FilterType < guDYNAMIC_FILTER_TYPE_YEAR )
+    else if( ( FilterType < guDYNAMIC_FILTER_TYPE_YEAR ) ||
+             ( FilterType == guDYNAMIC_FILTER_TYPE_DISK ) )
     {
         FilterItem.Set( FilterType,
                     m_FilterTextOptionChoice->GetSelection(),
@@ -714,7 +720,8 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
                         0 );
 
     }
-    else if( FilterType > guDYNAMIC_FILTER_TYPE_PLAYCOUNT )
+    else if( ( FilterType == guDYNAMIC_FILTER_TYPE_LASTPLAY ) ||
+             ( FilterType == guDYNAMIC_FILTER_TYPE_ADDEDDATE ) )
     {
         unsigned long value = 0;
         m_FilterText->GetValue().ToULong( &value );
@@ -723,7 +730,7 @@ guFilterItem guDynPlayListEditor::GetFilterItem( void )
                         value,
                         m_FilterDateOption2Choice->GetSelection() );
     }
-    else
+    else // FilterType == guDYNAMIC_FILTER_TYPE_YEAR
     {
         unsigned long value = 0;
         m_FilterText->GetValue().ToULong( &value );
@@ -744,10 +751,8 @@ bool guDynPlayListEditor::FilterHasChanged( void )
 // -------------------------------------------------------------------------------- //
 void guDynPlayListEditor::UpdateEditor( int FilterType )
 {
-    int index;
-    int count;
-
-    if( FilterType == guDYNAMIC_FILTER_TYPE_LENGTH )
+    if( ( FilterType == guDYNAMIC_FILTER_TYPE_LENGTH ) ||
+        ( FilterType == guDYNAMIC_FILTER_TYPE_HASARTWORK ) )
     {
         //
         m_FilterAdd->Enable( true );
@@ -758,6 +763,8 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         m_FilterAdd->Enable( false );
     }
 
+    m_FilterText->Enable( FilterType != guDYNAMIC_FILTER_TYPE_HASARTWORK );
+
     if( FilterType == guDYNAMIC_FILTER_TYPE_RATING )
     {
         m_FilterText->Show( false );
@@ -785,7 +792,8 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         m_LengthSeconds->Show( true );
 
     }
-    else if( FilterType > guDYNAMIC_FILTER_TYPE_PLAYCOUNT )
+    else if( ( FilterType == guDYNAMIC_FILTER_TYPE_LASTPLAY ) ||
+             ( FilterType == guDYNAMIC_FILTER_TYPE_ADDEDDATE ) )
     {
         m_FilterText->Show( true );
         m_FilterTextOptionChoice->Show( true );
@@ -811,6 +819,19 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
         m_LengthSeparator2->Show( false );
         m_LengthSeconds->Show( false );
     }
+    else if( FilterType == guDYNAMIC_FILTER_TYPE_HASARTWORK )
+    {
+        m_FilterText->Show( true );
+        m_FilterTextOptionChoice->Show( true );
+        m_FilterLabelOptionChoice->Show( false );
+        m_FilterRating->Show( false );
+        m_FilterDateOption2Choice->Show( false );
+        m_LengthHours->Show( false );
+        m_LengthSeparator1->Show( false );
+        m_LengthMinutes->Show( false );
+        m_LengthSeparator2->Show( false );
+        m_LengthSeconds->Show( false );
+    }
     else
     {
         m_FilterText->Show( true );
@@ -827,37 +848,27 @@ void guDynPlayListEditor::UpdateEditor( int FilterType )
 
     m_FilterTextOptionChoice->Clear();
 
-    if( FilterType < guDYNAMIC_FILTER_TYPE_YEAR )
+    if( ( FilterType < guDYNAMIC_FILTER_TYPE_YEAR ) ||
+        ( FilterType == guDYNAMIC_FILTER_TYPE_DISK ) )
     {
-        count = m_FilterTextOptionChoices.Count();
-        for( index = 0; index < count; index++ )
-        {
-            m_FilterTextOptionChoice->Append( m_FilterTextOptionChoices[ index ] );
-        }
+          m_FilterTextOptionChoice->Append( m_FilterTextOptionChoices );
     }
     else if( FilterType == guDYNAMIC_FILTER_TYPE_YEAR )
     {
-        count = m_FilterYearOptionChoices.Count();
-        for( index = 0; index < count; index++ )
-        {
-            m_FilterTextOptionChoice->Append( m_FilterYearOptionChoices[ index ] );
-        }
+        m_FilterTextOptionChoice->Append( m_FilterYearOptionChoices );
     }
-    else if( FilterType < guDYNAMIC_FILTER_TYPE_LASTPLAY )
+    else if( ( FilterType == guDYNAMIC_FILTER_TYPE_LASTPLAY ) ||
+             ( FilterType == guDYNAMIC_FILTER_TYPE_ADDEDDATE ) )
     {
-        count = m_FilterNumberOptionChoices.Count();
-        for( index = 0; index < count; index++ )
-        {
-            m_FilterTextOptionChoice->Append( m_FilterNumberOptionChoices[ index ] );
-        }
+        m_FilterTextOptionChoice->Append( m_FilterDateOptionChoices );
     }
-    else if( FilterType > guDYNAMIC_FILTER_TYPE_PLAYCOUNT )
+    else if( FilterType == guDYNAMIC_FILTER_TYPE_HASARTWORK )
     {
-        count = m_FilterDateOptionChoices.Count();
-        for( index = 0; index < count; index++ )
-        {
-            m_FilterTextOptionChoice->Append( m_FilterDateOptionChoices[ index ] );
-        }
+        m_FilterTextOptionChoice->Append( m_FilterBoolOptionChoices );
+    }
+    else //if( FilterType < guDYNAMIC_FILTER_TYPE_LASTPLAY )
+    {
+        m_FilterTextOptionChoice->Append( m_FilterNumberOptionChoices );
     }
     m_FilterTextOptionChoice->SetSelection( 0 );
     m_FilterLabelOptionChoice->SetSelection( 0 );
@@ -879,7 +890,9 @@ void guDynPlayListEditor::OnFiltersSelected( wxCommandEvent &event )
             m_FilterLabelOptionChoice->SetSelection( FilterItem->m_Option );
         else
             m_FilterTextOptionChoice->SetSelection( FilterItem->m_Option );
-        if( FilterItem->m_Type < guDYNAMIC_FILTER_TYPE_YEAR )
+
+        if( ( FilterItem->m_Type < guDYNAMIC_FILTER_TYPE_YEAR ) ||
+            ( FilterItem->m_Type == guDYNAMIC_FILTER_TYPE_DISK ) )
         {
             m_FilterText->SetValue( FilterItem->m_Text );
         }
@@ -939,13 +952,15 @@ void guDynPlayListEditor::OnFilterFieldSelected( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 void guDynPlayListEditor::OnFilterOptionSelected( wxCommandEvent &event )
 {
-    bool IsNotSetLabel = m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LENGTH ||
+    bool IsNotSetLabel = ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LENGTH ) ||
+                         ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_HASARTWORK ) ||
                           ( ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LABEL ) &&
                           ( m_FilterLabelOptionChoice->GetSelection() == guDYNAMIC_FILTER_OPTION_LABELS_NOTSET ) );
+    //guLogMessage( wxT( "IsNotSetLabel: %i" ), IsNotSetLabel );
     m_FilterText->Enable( !IsNotSetLabel );
     if( IsNotSetLabel )
         m_FilterText->SetValue( wxEmptyString );
-    m_FilterAdd->Enable( IsNotSetLabel );
+    m_FilterAdd->Enable( IsNotSetLabel || !m_FilterText->IsEmpty() );
 
     if( m_CurFilter != wxNOT_FOUND )
     {
@@ -968,15 +983,16 @@ void guDynPlayListEditor::OnFilterDateOption2Selected( wxCommandEvent &event )
 void guDynPlayListEditor::OnFilterTextChanged( wxCommandEvent& event )
 {
     int FilterType = m_FilterFieldChoice->GetSelection();
-    if( FilterType > guDYNAMIC_FILTER_TYPE_PATH )
+    if( ( FilterType > guDYNAMIC_FILTER_TYPE_PATH ) &&
+        ( FilterType != guDYNAMIC_FILTER_TYPE_DISK ) )
     {
         unsigned long value = 0;
-        if( !m_FilterText->GetValue().IsEmpty() && !m_FilterText->GetValue().ToULong( &value ) )
+        if( !m_FilterText->IsEmpty() && !m_FilterText->GetValue().ToULong( &value ) )
             m_FilterText->SetValue( wxEmptyString );
     }
 
 
-    m_FilterAdd->Enable( !m_FilterText->GetValue().IsEmpty() ||
+    m_FilterAdd->Enable( !m_FilterText->IsEmpty() ||
                          ( ( m_FilterFieldChoice->GetSelection() == guDYNAMIC_FILTER_TYPE_LABEL ) &&
                          ( m_FilterLabelOptionChoice->GetSelection() == guDYNAMIC_FILTER_OPTION_LABELS_NOTSET ) ) );
 
