@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -106,36 +106,43 @@ int guGoogleCoverFetcher::ExtractImagesInfo( wxString &content, int count )
     int ImageIndex = 0;
 
     int StrPos = content.Find( wxT( "dyn.setResults([[" ) );
+    if( StrPos == wxNOT_FOUND )
+        return 0;
+    content = content.Mid( StrPos + 14 );
+    StrPos = content.Find( wxT( "]]);" ) );
+    if( StrPos == wxNOT_FOUND )
+        return 0;
+    content = content.Mid( 0, StrPos );
 
-    if( StrPos != wxNOT_FOUND )
-        StrPos += 14;
+    StrPos = 0;
     //guLogMessage( wxT( "Content:\n%s" ), Content.c_str() );
     while( ( StrPos != wxNOT_FOUND ) && !m_MainThread->TestDestroy() )
     {
         content = content.Mid( StrPos + 3 );
         StrPos = content.Find( wxT( "],[" ) );
-        if( StrPos == wxNOT_FOUND )
-          return 0; //break;
-        //guLogMessage( wxT( "%s" ), Content.Mid( 0, StrPos ).c_str() );
+
+        //guLogMessage( wxT( "%s" ), content.Mid( 0, StrPos ).c_str() );
         wxHtmlEntitiesParser EntitiesParser;
         GoogleImage = ExtractImageInfo( EntitiesParser.Parse( content.Mid( 0, StrPos ) ) );
-        //RetVal.Add( CurImage );
-        CurImage.Empty();
-        if( GoogleImage[ GOOGLE_COVERINFO_LINK ].IsEmpty() )
+        if( GoogleImage.Count() >= GOOGLE_COVERINFO_SIZE )
         {
-            GoogleImage[ GOOGLE_COVERINFO_LINK ] = ExtractCoverFromGoogleLink( GoogleImage[ 0 ] );
-        }
+            //RetVal.Add( CurImage );
+            CurImage.Empty();
+            if( GoogleImage[ GOOGLE_COVERINFO_LINK ].IsEmpty() )
+            {
+                GoogleImage[ GOOGLE_COVERINFO_LINK ] = ExtractCoverFromGoogleLink( GoogleImage[ 0 ] );
+            }
 
-        if( !GoogleImage[ GOOGLE_COVERINFO_LINK ].IsEmpty() )
-        {
-            CurImage.Add( GoogleImage[ GOOGLE_COVERINFO_LINK ] );
-            CurImage.Add( GoogleImage[ GOOGLE_COVERINFO_SIZE ] );
-            m_CoverLinks->Add( CurImage );
-            ImageIndex++;
-            if( ImageIndex == count )
-                break;
+            if( !GoogleImage[ GOOGLE_COVERINFO_LINK ].IsEmpty() )
+            {
+                CurImage.Add( GoogleImage[ GOOGLE_COVERINFO_LINK ] );
+                CurImage.Add( GoogleImage[ GOOGLE_COVERINFO_SIZE ] );
+                m_CoverLinks->Add( CurImage );
+                ImageIndex++;
+                if( ImageIndex == count )
+                    break;
+            }
         }
-
         //guLogMessage( wxT( "Pos: %u" ), StrPos );
     }
     return ImageIndex;
@@ -147,7 +154,7 @@ int guGoogleCoverFetcher::AddCoverLinks( int pagenum )
     wxString SearchString = wxString::Format( wxT( "\"%s\" \"%s\"" ), m_Artist.c_str(), m_Album.c_str() );
     //guLogMessage( wxT( "URL: %u %s" ), m_CurrentPage, m_SearchString.c_str() );
     wxString SearchUrl = wxString::Format( GOOGLE_IMAGES_SEARCH_URL, guURLEncode( SearchString ).c_str(), ( pagenum * GOOGLE_COVERS_PER_PAGE ) );
-    //guLogMessage( wxT( "URL: %u %s" ), pagenum, SearchUrl.c_str() );
+    guLogMessage( wxT( "URL: %u %s" ), pagenum, SearchUrl.c_str() );
     if( !m_MainThread->TestDestroy() )
     {
         //printf( "Buffer:\n%s\n", Buffer );
@@ -156,7 +163,7 @@ int guGoogleCoverFetcher::AddCoverLinks( int pagenum )
         {
             if( !m_MainThread->TestDestroy() )
             {
-                //guLogMessage( Content );
+                //guLogMessage( wxT( "Google:====>>>>\n%s\n<<<<====" ), Content.c_str() );
                 return ExtractImagesInfo( Content, GOOGLE_COVERS_PER_PAGE );
             }
         }
