@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -37,6 +37,7 @@
 #include "./images/edit_delete.h"
 #include "./images/edit.h"
 #include "./images/exit.h"
+#include "./images/filter.h"
 #include "./images/guayadeque.h"
 #include "./images/guayadeque_taskbar.h"
 #include "./images/lastfm_as_off.h"
@@ -62,6 +63,7 @@
 #include "./images/tiny_edit_clear.h"
 #include "./images/tiny_edit.h"
 #include "./images/tiny_edit_copy.h"
+#include "./images/tiny_filter.h"
 #include "./images/tiny_left.h"
 #include "./images/tiny_net_radio.h"
 #include "./images/tiny_numerate.h"
@@ -91,6 +93,18 @@
 #include "./images/pref_podcasts.h"
 #include "./images/pref_record.h"
 #include "./images/pref_crossfader.h"
+#include "./images/pref_jamendo.h"
+#include "./images/pref_magnatune.h"
+#include "./images/pref_accelerators.h"
+//
+#include "./images/loc_library.h"
+#include "./images/loc_portable_device.h"
+#include "./images/loc_net_radio.h"
+#include "./images/loc_podcast.h"
+#include "./images/loc_magnatune.h"
+#include "./images/loc_jamendo.h"
+#include "./images/loc_lastfm.h"
+#include "./images/loc_lyrics.h"
 //
 #include "./images/tiny_close_normal.h"
 #include "./images/tiny_close_highlight.h"
@@ -153,6 +167,8 @@
 #include "./images/player_normal_love.h"
 #include "./images/player_normal_ban.h"
 #include "./images/player_tiny_light_play.h"
+#include "./images/player_tiny_light_stop.h"
+#include "./images/player_tiny_red_stop.h"
 //
 #include "./images/star_normal_tiny.h"
 #include "./images/star_normal_mid.h"
@@ -160,7 +176,9 @@
 #include "./images/star_highlight_tiny.h"
 #include "./images/star_highlight_mid.h"
 #include "./images/star_highlight_big.h"
-
+//
+#include "./images/tiny_crossfade.h"
+#include "./images/tiny_gapless.h"
 
 // -------------------------------------------------------------------------------- //
 typedef struct {
@@ -187,6 +205,7 @@ guImage_Item guImage_Items[] = {
     GUIMAGE( guImage_edit_delete,                   wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_edit,                          wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_exit,                          wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_filter,                        wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_guayadeque,                    wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_guayadeque_taskbar,            wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_lastfm_as_off,                 wxBITMAP_TYPE_PNG ),
@@ -212,6 +231,7 @@ guImage_Item guImage_Items[] = {
     GUIMAGE( guImage_musicbrainz,	                wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_edit,                     wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_edit_copy,	            wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_tiny_filter,	                wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_search_again,             wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_numerate,	                wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_edit_clear,               wxBITMAP_TYPE_PNG ),
@@ -241,6 +261,18 @@ guImage_Item guImage_Items[] = {
     GUIMAGE( guImage_pref_podcasts,                 wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_pref_record,                   wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_pref_crossfader,		    wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_pref_jamendo,		    wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_pref_magnatune,		    wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_pref_accelerators,		    wxBITMAP_TYPE_PNG ),
+    //
+    GUIMAGE( guImage_loc_library,		    wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_portable_device,	        wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_net_radio,	                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_podcast,	                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_magnatune,	                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_jamendo,	                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_lastfm,	                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_loc_lyrics,			        wxBITMAP_TYPE_PNG ),
     //
     GUIMAGE( guImage_tiny_close_normal,		        wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_tiny_close_highlight,	        wxBITMAP_TYPE_PNG ),
@@ -303,13 +335,18 @@ guImage_Item guImage_Items[] = {
     GUIMAGE( guImage_player_normal_love,            wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_player_normal_ban,             wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_player_tiny_light_play,        wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_player_tiny_light_stop,        wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_player_tiny_red_stop,          wxBITMAP_TYPE_PNG ),
     //
     GUIMAGE( guImage_star_normal_tiny,              wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_star_normal_mid,               wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_star_normal_big,               wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_star_highlight_tiny,           wxBITMAP_TYPE_PNG ),
     GUIMAGE( guImage_star_highlight_mid,            wxBITMAP_TYPE_PNG ),
-    GUIMAGE( guImage_star_highlight_big,            wxBITMAP_TYPE_PNG )
+    GUIMAGE( guImage_star_highlight_big,            wxBITMAP_TYPE_PNG ),
+    //
+    GUIMAGE( guImage_tiny_crossfade,                wxBITMAP_TYPE_PNG ),
+    GUIMAGE( guImage_tiny_gapless,                  wxBITMAP_TYPE_PNG )
 };
 
 
