@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -259,10 +259,11 @@ class guFetchAlbumInfoThread : public guFetchLastFMInfoThread
 {
   protected:
     guDbCache *             m_DbCache;
+    int                     m_Start;
     wxString                m_ArtistName;
 
   public:
-    guFetchAlbumInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname );
+    guFetchAlbumInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const int startpage );
     ~guFetchAlbumInfoThread();
 
     virtual ExitCode Entry();
@@ -277,9 +278,10 @@ class guFetchTopTracksInfoThread : public guFetchLastFMInfoThread
     guDbCache *             m_DbCache;
     wxString                m_ArtistName;
     int                     m_ArtistId;
+    int                     m_Start;
 
   public:
-    guFetchTopTracksInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname );
+    guFetchTopTracksInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const int startpage );
     ~guFetchTopTracksInfoThread();
 
     virtual ExitCode Entry();
@@ -288,14 +290,31 @@ class guFetchTopTracksInfoThread : public guFetchLastFMInfoThread
 };
 
 // -------------------------------------------------------------------------------- //
-class guFetchSimilarArtistInfoThread : public guFetchLastFMInfoThread
+class guFetchArtistInfoThread : public guFetchLastFMInfoThread
 {
   private:
     guDbCache *             m_DbCache;
     wxString                m_ArtistName;
 
   public:
-    guFetchSimilarArtistInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname );
+    guFetchArtistInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname );
+    ~guFetchArtistInfoThread();
+
+    virtual ExitCode Entry();
+
+    friend class guDownloadArtistImageThread;
+};
+
+// -------------------------------------------------------------------------------- //
+class guFetchSimilarArtistInfoThread : public guFetchLastFMInfoThread
+{
+  private:
+    guDbCache *             m_DbCache;
+    wxString                m_ArtistName;
+    int                     m_Start;
+
+  public:
+    guFetchSimilarArtistInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const int startpage );
     ~guFetchSimilarArtistInfoThread();
 
     virtual ExitCode Entry();
@@ -304,16 +323,34 @@ class guFetchSimilarArtistInfoThread : public guFetchLastFMInfoThread
 };
 
 // -------------------------------------------------------------------------------- //
-class guFetchTrackInfoThread : public guFetchLastFMInfoThread
+class guFetchEventsInfoThread : public guFetchLastFMInfoThread
+{
+  private:
+    guDbCache *             m_DbCache;
+    wxString                m_ArtistName;
+    int                     m_Start;
+
+  public:
+    guFetchEventsInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const int startpage );
+    ~guFetchEventsInfoThread();
+
+    virtual ExitCode Entry();
+
+    friend class guDownloadArtistImageThread;
+};
+
+// -------------------------------------------------------------------------------- //
+class guFetchSimTracksInfoThread : public guFetchLastFMInfoThread
 {
   private:
     guDbCache *             m_DbCache;
     wxString                m_ArtistName;
     wxString                m_TrackName;
+    int                     m_Start;
 
   public:
-    guFetchTrackInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const wxChar * trackname );
-    ~guFetchTrackInfoThread();
+    guFetchSimTracksInfoThread( guLastFMPanel * lastfmpanel, guDbCache * dbcache, const wxChar * artistname, const wxChar * trackname, const int stargpage );
+    ~guFetchSimTracksInfoThread();
 
     virtual ExitCode Entry();
 
@@ -343,7 +380,6 @@ class guLastFMInfoCtrl : public wxPanel
     virtual void        CreateControls( wxWindow * parent );
     virtual void        OnPlayClicked( wxCommandEvent &event );
     virtual void        OnEnqueueClicked( wxCommandEvent &event );
-    virtual void        OnEnqueueAsNextClicked( wxCommandEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
     virtual void        OnSongSelectName( wxCommandEvent &event );
     virtual void        OnArtistSelectName( wxCommandEvent &event );
@@ -414,8 +450,6 @@ class guAlbumInfoCtrl : public guLastFMInfoCtrl
     guLastFMAlbumInfo * m_Info;
 
     virtual void        OnContextMenu( wxContextMenuEvent& event );
-//    void                OnSearchLinkClicked( wxCommandEvent &event );
-    //virtual void        OnClick( wxMouseEvent &event );
     virtual wxString    GetSearchText( void );
     virtual wxString    GetItemUrl( void );
     virtual void        CreateContextMenu( wxMenu * Menu );
@@ -475,7 +509,6 @@ class guTrackInfoCtrl : public guLastFMInfoCtrl
     virtual void        CreateContextMenu( wxMenu * Menu );
     virtual wxString    GetSearchText( void );
     virtual wxString    GetItemUrl( void );
-    //void                OnClick( wxMouseEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
     virtual void        OnSongSelectName( wxCommandEvent &event );
     virtual void        OnArtistSelectName( wxCommandEvent &event );
@@ -506,7 +539,6 @@ class guTopTrackInfoCtrl : public guLastFMInfoCtrl
     virtual void        CreateContextMenu( wxMenu * Menu );
     virtual wxString    GetSearchText( void );
     virtual wxString    GetItemUrl( void );
-    //void                OnClick( wxMouseEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
     virtual void        OnSongSelectName( wxCommandEvent &event );
     virtual void        OnArtistSelectName( wxCommandEvent &event );
@@ -537,7 +569,6 @@ class guEventInfoCtrl : public guLastFMInfoCtrl
     virtual void        CreateContextMenu( wxMenu * Menu );
     virtual wxString    GetSearchText( void );
     virtual wxString    GetItemUrl( void );
-    //void                OnClick( wxMouseEvent &event );
     virtual int         GetSelectedTracks( guTrackArray * tracks );
 
     virtual wxString    GetBitmapImageUrl( void ) { return m_Info ? m_Info->m_ImageUrl : wxT( "" ); };
@@ -571,17 +602,23 @@ class guLastFMPanel : public wxScrolledWindow
 	wxString                m_LongBio;
 	bool                    m_UpdateEnabled;
 
+	guFetchArtistInfoThread *           m_ArtistsUpdateThread;
+	wxMutex                             m_ArtistsUpdateThreadMutex;
+
 	guFetchAlbumInfoThread *            m_AlbumsUpdateThread;
 	wxMutex                             m_AlbumsUpdateThreadMutex;
 
 	guFetchTopTracksInfoThread *        m_TopTracksUpdateThread;
 	wxMutex                             m_TopTracksUpdateThreadMutex;
 
-	guFetchSimilarArtistInfoThread *    m_ArtistsUpdateThread;
-	wxMutex                             m_ArtistsUpdateThreadMutex;
+	guFetchSimilarArtistInfoThread *    m_SimArtistsUpdateThread;
+	wxMutex                             m_SimArtistsUpdateThreadMutex;
 
-	guFetchTrackInfoThread *            m_TracksUpdateThread;
-	wxMutex                             m_TracksUpdateThreadMutex;
+	guFetchSimTracksInfoThread *        m_SimTracksUpdateThread;
+	wxMutex                             m_SimTracksUpdateThreadMutex;
+
+	guFetchEventsInfoThread *           m_EventsUpdateThread;
+	wxMutex                             m_EventsUpdateThreadMutex;
 
     // TODO : Check if its really necesary
 	wxMutex                             m_UpdateEventsMutex;
@@ -607,27 +644,54 @@ class guLastFMPanel : public wxScrolledWindow
 	bool                                m_ShowAlbums;
     wxStaticText *                      m_AlbumsStaticText;
 	wxGridSizer *                       m_AlbumsSizer;
-    guAlbumInfoCtrlArray                m_AlbumInfoCtrls;
+    guAlbumInfoCtrlArray                m_AlbumsInfoCtrls;
+    wxStaticText *                      m_AlbumsRangeLabel;
+    wxBitmapButton *                    m_AlbumsPrevBtn;
+    wxBitmapButton *                    m_AlbumsNextBtn;
+    int                                 m_AlbumsCount;
+    int                                 m_AlbumsPageStart;
 
 	bool                                m_ShowTopTracks;
     wxStaticText *                      m_TopTracksStaticText;
 	wxGridSizer *                       m_TopTracksSizer;
 	guTopTrackInfoCtrlArray             m_TopTrackInfoCtrls;
+    wxStaticText *                      m_TopTracksRangeLabel;
+    wxBitmapButton *                    m_TopTracksPrevBtn;
+    wxBitmapButton *                    m_TopTracksNextBtn;
+    int                                 m_TopTracksCount;
+    int                                 m_TopTracksPageStart;
 
-	bool                                m_ShowArtists;
-    wxStaticText *                      m_ArtistsStaticText;
-	wxGridSizer *                       m_ArtistsSizer;
-	guSimilarArtistInfoCtrlArray        m_ArtistInfoCtrls;
+	bool                                m_ShowSimArtists;
+    wxStaticText *                      m_SimArtistsStaticText;
+	wxGridSizer *                       m_SimArtistsSizer;
+	guSimilarArtistInfoCtrlArray        m_SimArtistsInfoCtrls;
+    wxStaticText *                      m_SimArtistsRangeLabel;
+    wxBitmapButton *                    m_SimArtistsPrevBtn;
+    wxBitmapButton *                    m_SimArtistsNextBtn;
+    int                                 m_SimArtistsCount;
+    int                                 m_SimArtistsPageStart;
 
-	bool                                m_ShowTracks;
-    wxStaticText *                      m_TracksStaticText;
-	wxGridSizer *                       m_TracksSizer;
-	guTrackInfoCtrlArray                m_TrackInfoCtrls;
+	bool                                m_ShowSimTracks;
+    wxStaticText *                      m_SimTracksStaticText;
+	wxGridSizer *                       m_SimTracksSizer;
+	guTrackInfoCtrlArray                m_SimTracksInfoCtrls;
+    wxStaticText *                      m_SimTracksRangeLabel;
+    wxBitmapButton *                    m_SimTracksPrevBtn;
+    wxBitmapButton *                    m_SimTracksNextBtn;
+    int                                 m_SimTracksCount;
+    int                                 m_SimTracksPageStart;
 
 	bool                                m_ShowEvents;
     wxStaticText *                      m_EventsStaticText;
 	wxGridSizer *                       m_EventsSizer;
 	guEventInfoCtrlArray                m_EventsInfoCtrls;
+    wxStaticText *                      m_EventsRangeLabel;
+    wxBitmapButton *                    m_EventsPrevBtn;
+    wxBitmapButton *                    m_EventsNextBtn;
+    int                                 m_EventsCount;
+    int                                 m_EventsPageStart;
+
+    wxStaticText *                      m_ContextMenuObject;
 
 
 	// TODO : Check if its really necesary
@@ -640,8 +704,6 @@ class guLastFMPanel : public wxScrolledWindow
     void    OnUpdateTrackItem( wxCommandEvent &event );
     void    OnUpdateEventItem( wxCommandEvent &event );
 
-//    void    OnShowMoreLinkClicked( wxHyperlinkEvent &event );
-//	  void    OnHtmlLinkClicked( wxHtmlLinkEvent& event );
 	void    OnArInfoTitleDClicked( wxMouseEvent &event );
     void    OnTopAlbumsTitleDClick( wxMouseEvent &event );
     void    OnTopTracksTitleDClick( wxMouseEvent &event );
@@ -649,22 +711,57 @@ class guLastFMPanel : public wxScrolledWindow
 	void    OnSimTrTitleDClick( wxMouseEvent &event );
 	void    OnEventsTitleDClick( wxMouseEvent &event );
 
+    void    SetTopAlbumsVisible( const bool dolayout = false );
+    void    SetTopTracksVisible( const bool dolayout = false );
+    void    SetSimArtistsVisible( const bool dolayout = false );
+    void    SetSimTracksVisible( const bool dolayout = false );
+    void    SetEventsVisible( const bool dolayout = false );
+
+
 	void    OnUpdateChkBoxClick( wxCommandEvent &event );
     void    OnPrevBtnClick( wxCommandEvent &event );
     void    OnNextBtnClick( wxCommandEvent &event );
     void    OnReloadBtnClick( wxCommandEvent &event );
     void    OnTextUpdated( wxCommandEvent& event );
-    void    OnSearchBtnClick( wxCommandEvent &event );
-
-//	void    OnAlbumTextClicked( wxMouseEvent &event );
-//	void    OnArtistTextClicked( wxMouseEvent &event );
-//	void    OnTrackTextClicked( wxMouseEvent &event );
-//	void    OnAlbumTextRightClicked( wxMouseEvent &event );
-//	void    OnArtistTextRightClicked( wxMouseEvent &event );
-//	void    OnTrackTextRightClicked( wxMouseEvent &event );
+    void    OnTextCtrlKeyDown( wxKeyEvent &event );
+    void    OnSearchSelected( wxCommandEvent &event );
 
     void    UpdateTrackChangeButtons( void );
 
+    void    UpdateAlbumsRangeLabel( void );
+    void    OnAlbumsCountUpdated( wxCommandEvent &event );
+    void    OnAlbumsPrevClicked( wxCommandEvent &event );
+    void    OnAlbumsNextClicked( wxCommandEvent &event );
+
+    void    UpdateTopTracksRangeLabel( void );
+    void    OnTopTracksCountUpdated( wxCommandEvent &event );
+    void    OnTopTracksPrevClicked( wxCommandEvent &event );
+    void    OnTopTracksNextClicked( wxCommandEvent &event );
+
+    void    UpdateSimArtistsRangeLabel( void );
+    void    OnSimArtistsCountUpdated( wxCommandEvent &event );
+    void    OnSimArtistsPrevClicked( wxCommandEvent &event );
+    void    OnSimArtistsNextClicked( wxCommandEvent &event );
+
+    void    UpdateSimTracksRangeLabel( void );
+    void    OnSimTracksCountUpdated( wxCommandEvent &event );
+    void    OnSimTracksPrevClicked( wxCommandEvent &event );
+    void    OnSimTracksNextClicked( wxCommandEvent &event );
+
+    void    UpdateEventsRangeLabel( void );
+    void    OnEventsCountUpdated( wxCommandEvent &event );
+    void    OnEventsPrevClicked( wxCommandEvent &event );
+    void    OnEventsNextClicked( wxCommandEvent &event );
+
+    void    OnContextMenu( wxContextMenuEvent& event );
+
+    void    OnPlayClicked( wxCommandEvent &event );
+    void    OnEnqueueClicked( wxCommandEvent &event );
+    void    OnSaveClicked( wxCommandEvent &event );
+    void    OnCopyToClicked( wxCommandEvent &event );
+
+    void    GetContextMenuTracks( guTrackArray * tracks );
+
   public :
             guLastFMPanel( wxWindow * parent, guDbLibrary * db,
                 guDbCache * dbcache, guPlayerPanel * playerpanel );
@@ -674,14 +771,16 @@ class guLastFMPanel : public wxScrolledWindow
     void    AppendTrackChangeInfo( const guTrackChangeInfo * trackchangeinfo );
 	void    ShowCurrentTrack( void );
 	void    SetUpdateEnable( bool value );
-	void    UpdateLayout( void );
+	void    UpdateLayout( void ) { m_MainSizer->FitInside( this ); }
 	void    OnDropFiles( const wxArrayString &files );
 
     friend class guFetchLastFMInfoThread;
+    friend class guFetchArtistInfoThread;
     friend class guFetchAlbumInfoThread;
     friend class guFetchTopTracksInfoThread;
     friend class guFetchSimilarArtistInfoThread;
-    friend class guFetchTrackInfoThread;
+    friend class guFetchSimTracksInfoThread;
+    friend class guFetchEventsInfoThread;
     friend class guDownloadImageThread;
 };
 
