@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -36,9 +36,60 @@ int inline GetFileLastChangeTime( const wxString &FileName )
     return St.st_ctime;
 }
 
+// -------------------------------------------------------------------------------- //
+bool inline IsFileSymbolicLink( const wxString &FileName )
+{
+    wxStructStat St;
+    wxLstat( FileName, &St );
+    return S_ISLNK( St.st_mode );
+}
+
+// -------------------------------------------------------------------------------- //
+guLibUpdateThread::guLibUpdateThread( guLibPanel * libpanel, int gaugeid, const wxString &scanpath )
+{
+    m_LibPanel = libpanel;
+    m_Db = libpanel->GetDb();
+    m_MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
+    m_GaugeId = gaugeid;
+    m_ScanPath = scanpath;
+
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    //m_CoverSearchWords = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+    m_CoverSearchWords = libpanel->GetCoverSearchWords();
+    int Index;
+    int Count = m_CoverSearchWords.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        m_CoverSearchWords[ Index ].MakeLower();
+    }
+
+    if( scanpath.IsEmpty() )
+    {
+        m_LibPaths = libpanel->GetLibraryPaths();
+        wxString PlaylistPath = libpanel->GetPlaylistPath();
+        if( !PlaylistPath.IsEmpty() )
+            m_LibPaths.Add( PlaylistPath );
+
+        CheckSymLinks( m_LibPaths );
+    }
+
+    m_LastUpdate = libpanel->LastUpdate();
+    //guLogMessage( wxT( "LastUpdate: %s" ), LastTime.Format().c_str() );
+    m_ScanAddPlayLists = Config->ReadBool( wxT( "ScanAddPlayLists" ), true, wxT( "General" ) );
+    m_ScanEmbeddedCovers = Config->ReadBool( wxT( "ScanEmbeddedCovers" ), true, wxT( "General" ) );
+    m_ScanSymlinks = Config->ReadBool( wxT( "ScanSymlinks" ), false, wxT( "General" ) );
+
+    if( Create() == wxTHREAD_NO_ERROR )
+    {
+        SetPriority( WXTHREAD_DEFAULT_PRIORITY );
+        Run();
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 guLibUpdateThread::guLibUpdateThread( guDbLibrary * db, int gaugeid, const wxString &scanpath )
 {
+    m_LibPanel = NULL;
     m_Db = db;
     m_MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
     m_GaugeId = gaugeid;
@@ -46,6 +97,7 @@ guLibUpdateThread::guLibUpdateThread( guDbLibrary * db, int gaugeid, const wxStr
 
     guConfig * Config = ( guConfig * ) guConfig::Get();
     m_CoverSearchWords = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+
     if( scanpath.IsEmpty() )
     {
         m_LibPaths = Config->ReadAStr( wxT( "LibPath" ), wxEmptyString, wxT( "LibPaths" ) );
@@ -57,10 +109,11 @@ guLibUpdateThread::guLibUpdateThread( guDbLibrary * db, int gaugeid, const wxStr
     //guLogMessage( wxT( "LastUpdate: %s" ), LastTime.Format().c_str() );
     m_ScanAddPlayLists = Config->ReadBool( wxT( "ScanAddPlayLists" ), true, wxT( "General" ) );
     m_ScanEmbeddedCovers = Config->ReadBool( wxT( "ScanEmbeddedCovers" ), true, wxT( "General" ) );
+    m_ScanSymlinks = Config->ReadBool( wxT( "ScanSymlinks" ), false, wxT( "General" ) );
 
     if( Create() == wxTHREAD_NO_ERROR )
     {
-        SetPriority( WXTHREAD_DEFAULT_PRIORITY );
+        SetPriority( WXTHREAD_DEFAULT_PRIORITY + 10 );
         Run();
     }
 }
@@ -68,21 +121,29 @@ guLibUpdateThread::guLibUpdateThread( guDbLibrary * db, int gaugeid, const wxStr
 // -------------------------------------------------------------------------------- //
 guLibUpdateThread::~guLibUpdateThread()
 {
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-    if( Config )
-    {
-        wxDateTime Now = wxDateTime::Now();
-        Config->WriteNum( wxT( "LastUpdate" ), Now.GetTicks(), wxT( "General" ) );
-        Config->Flush();
-    }
-
     wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_LIBRARY_UPDATED );
-    event.SetEventObject( ( wxObject * ) this );
-    wxPostEvent( wxTheApp->GetTopWindow(), event );
+    if( !TestDestroy() )
+    {
+        if( m_LibPanel )
+        {
+            m_LibPanel->SetLastUpdate();
+        }
+        else
+        {
+            guConfig * Config = ( guConfig * ) guConfig::Get();
+            wxDateTime Now = wxDateTime::Now();
+            Config->WriteNum( wxT( "LastUpdate" ), Now.GetTicks(), wxT( "General" ) );
+            Config->Flush();
+        }
+
+        event.SetEventObject( ( wxObject * ) this );
+        event.SetClientData( ( void * ) m_LibPanel );
+        wxPostEvent( m_MainFrame, event );
+    }
     //
-    event.SetId( ID_GAUGE_REMOVE );
+    event.SetId( ID_STATUSBAR_GAUGE_REMOVE );
     event.SetInt( m_GaugeId );
-    wxPostEvent( wxTheApp->GetTopWindow(), event );
+    wxPostEvent( m_MainFrame, event );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -115,48 +176,56 @@ int guLibUpdateThread::ScanDirectory( wxString dirname, bool includedir )
     if( Dir.GetFirst( &FileName, wxEmptyString, wxDIR_FILES | wxDIR_DIRS ) )
     {
       do {
-        if( ( FileName[ 0 ] != '.' ) )
+        if( FileName[ 0 ] == '.' )
+          continue;
+
+        if ( !m_ScanSymlinks && IsFileSymbolicLink( dirname + FileName ) )
+            continue;
+
+        int FileDate = GetFileLastChangeTime( dirname + FileName );
+        if( Dir.Exists( dirname + FileName ) )
         {
-          int FileDate = GetFileLastChangeTime( dirname + FileName );
-          if( Dir.Exists( dirname + FileName ) )
-          {
-            //guLogMessage( wxT( "Scanning dir '%s' : FileDate: %u  -> %u\n%u Tracks found" ), ( dirname + FileName ).c_str(), m_LastUpdate, FileDate, m_TrackFiles.Count() );
-            ScanDirectory( dirname + FileName, includedir || ( FileDate > m_LastUpdate ) );
+          //guLogMessage( wxT( "Scanning dir '%s' : FileDate: %u  -> %u\n%u Tracks found" ), ( dirname + FileName ).c_str(), m_LastUpdate, FileDate, m_TrackFiles.Count() );
+          ScanDirectory( dirname + FileName, includedir || ( FileDate > m_LastUpdate ) );
 
-            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_GAUGE_SETMAX );
-            event.SetInt( m_GaugeId );
-            event.SetExtraLong( m_TrackFiles.Count() );
-            wxPostEvent( m_MainFrame, event );
-          }
-          else
+          wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_STATUSBAR_GAUGE_SETMAX );
+          event.SetInt( m_GaugeId );
+          event.SetExtraLong( m_TrackFiles.Count() );
+          wxPostEvent( m_MainFrame, event );
+        }
+        else
+        {
+          //guLogMessage( wxT( "%s (%i): FileDate: %u  -> %u" ), ( dirname + FileName ).c_str(), includedir, m_LastUpdate, FileDate );
+          if( includedir || ( FileDate > m_LastUpdate ) )
           {
-            //guLogMessage( wxT( "%s (%i): FileDate: %u  -> %u" ), ( dirname + FileName ).c_str(), includedir, m_LastUpdate, FileDate );
-            if( includedir || ( FileDate > m_LastUpdate ) )
+            LowerFileName = FileName.Lower();
+
+            if( guIsValidAudioFile( LowerFileName ) )
             {
-              LowerFileName = FileName.Lower();
-
-              if( guIsValidAudioFile( LowerFileName ) )
+              if( !m_Db->FindDeletedFile( dirname + FileName, false ) )
               {
-                if( !m_Db->FindDeletedFile( dirname + FileName, false ) )
-                {
-                    m_TrackFiles.Add( dirname + FileName );
-                    if( m_ScanEmbeddedCovers && FirstAudioFile.IsEmpty() )
-                        FirstAudioFile = dirname + FileName;
-                }
-              }
-              else if( guIsValidImageFile( LowerFileName ) )
-              {
-                if( SearchCoverWords( LowerFileName, m_CoverSearchWords ) )                //guLogMessage( wxT( "Adding image '%s'" ), wxString( dirname + FileName ).c_str() );
-                {
-                    m_ImageFiles.Add( dirname + FileName );
-                }
-              }
-              else if( m_ScanAddPlayLists && guPlayListFile::IsValidPlayList( LowerFileName ) )
-              {
-                  FoundCover = true;
-                  m_PlayListFiles.Add( dirname + FileName );
+                m_TrackFiles.Add( dirname + FileName );
+                if( m_ScanEmbeddedCovers && FirstAudioFile.IsEmpty() )
+                    FirstAudioFile = dirname + FileName;
               }
             }
+            else if( guIsValidImageFile( LowerFileName ) )
+            {
+              if( SearchCoverWords( LowerFileName, m_CoverSearchWords ) )
+              {
+                //guLogMessage( wxT( "Adding image '%s'" ), wxString( dirname + FileName ).c_str() );
+                m_ImageFiles.Add( dirname + FileName );
+              }
+            }
+            else if( m_ScanAddPlayLists && guPlayListFile::IsValidPlayList( LowerFileName ) )
+            {
+              FoundCover = true;
+              m_PlayListFiles.Add( dirname + FileName );
+            }
+            //else
+            //{
+            //    guLogMessage( wxT( "Unknown file: '%s'"), ( dirname + FileName ).c_str() );
+            //}
           }
         }
       } while( !TestDestroy() && Dir.GetNext( &FileName ) );
@@ -179,10 +248,10 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
 {
     int index;
     int count;
-    wxCommandEvent evtup( wxEVT_COMMAND_MENU_SELECTED, ID_GAUGE_UPDATE );
+    wxCommandEvent evtup( wxEVT_COMMAND_MENU_SELECTED, ID_STATUSBAR_GAUGE_UPDATE );
     evtup.SetInt( m_GaugeId );
 
-    wxCommandEvent evtmax( wxEVT_COMMAND_MENU_SELECTED, ID_GAUGE_SETMAX );
+    wxCommandEvent evtmax( wxEVT_COMMAND_MENU_SELECTED, ID_STATUSBAR_GAUGE_SETMAX );
     evtmax.SetInt( m_GaugeId );
 
     if( m_ScanPath.IsEmpty() )
@@ -198,7 +267,7 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
         index = 0;
         while( !TestDestroy() && ( index < count ) )
         {
-            //guLogMessage( wxT( "Doing Library Update in %s" ), m_LibPaths[ index ].c_str() );
+            guLogMessage( wxT( "Doing Library Update in %s" ), m_LibPaths[ index ].c_str() );
             ScanDirectory( m_LibPaths[ index ] );
             index++;
         }
@@ -212,6 +281,8 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
     count = m_TrackFiles.Count();
     if( count )
     {
+        m_Db->ExecuteUpdate( wxT( "BEGIN TRANSACTION;" ) );
+
         index = 0;
         evtmax.SetExtraLong( count );
         wxPostEvent( m_MainFrame, evtmax );
@@ -222,21 +293,43 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
                 break;
 
             //guLogMessage( wxT( "Scanning: '%s'" ), m_TrackFiles[ index ].c_str() );
-            m_Db->ReadFileTags( m_TrackFiles[ index ].char_str() );
+            m_Db->ReadFileTags( m_TrackFiles[ index ] );
             //Sleep( 1 );
             index++;
             evtup.SetExtraLong( index );
             wxPostEvent( m_MainFrame, evtup );
         }
+
+        m_Db->ExecuteUpdate( wxT( "COMMIT TRANSACTION;" ) );
     }
 
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-    wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
-    wxString CoverName = ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+
+    wxString CoverName;
+    if( m_LibPanel )
+    {
+        CoverName = m_LibPanel->GetCoverName();
+    }
+    else
+    {
+        guConfig * Config = ( guConfig * ) guConfig::Get();
+        wxArrayString SearchCovers = Config->ReadAStr( wxT( "Word" ), wxEmptyString, wxT( "CoverSearch" ) );
+        CoverName = ( SearchCovers.Count() ? SearchCovers[ 0 ] : wxT( "cover" ) ) + wxT( ".jpg" );
+    }
+    int CoverType = wxBITMAP_TYPE_JPEG;
+    if( m_LibPanel )
+    {
+        CoverType = m_LibPanel->GetCoverType();
+    }
+    int CoverMaxSize = wxNOT_FOUND;
+    if( m_LibPanel )
+    {
+        CoverMaxSize = m_LibPanel->GetCoverMaxSize();
+    }
 
     count = m_ImageFiles.Count();
     if( count )
     {
+        m_Db->ExecuteUpdate( wxT( "BEGIN TRANSACTION;" ) );
         index = 0;
         evtmax.SetExtraLong( count );
         wxPostEvent( m_MainFrame, evtmax );
@@ -246,16 +339,20 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
             if( ( index >= count ) )
                 break;
 
-            m_Db->UpdateImageFile( m_ImageFiles[ index ].ToUTF8(), CoverName.ToUTF8() );
+            m_Db->UpdateImageFile( m_ImageFiles[ index ].ToUTF8(), CoverName.ToUTF8(), CoverType, CoverMaxSize );
             index++;
             evtup.SetExtraLong( index );
             wxPostEvent( m_MainFrame, evtup );
         }
+
+        m_Db->ExecuteUpdate( wxT( "COMMIT TRANSACTION;" ) );
     }
 
     count = m_PlayListFiles.Count();
     if( count )
     {
+        m_Db->ExecuteUpdate( wxT( "BEGIN TRANSACTION;" ) );
+
         index = 0;
         evtmax.SetExtraLong( count );
         wxPostEvent( m_MainFrame, evtmax );
@@ -282,12 +379,12 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
                 }
                 if( PlayListIds.Count() )
                 {
-                    if( m_Db->GetStaticPlayList( wxFileNameFromPath( m_PlayListFiles[ index ] ).BeforeLast( wxT( '.' ) ) ) == wxNOT_FOUND )
+                    if( m_Db->GetStaticPlayList( m_PlayListFiles[ index ] ) == wxNOT_FOUND )
                     {
-                        m_Db->CreateStaticPlayList( wxFileNameFromPath( m_PlayListFiles[ index ] ).BeforeLast( wxT( '.' ) ),
-                                    PlayListIds );
+                        m_Db->CreateStaticPlayList( m_PlayListFiles[ index ], PlayListIds );
 
                         wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYLIST_UPDATED );
+                        evt.SetClientData( ( void * ) m_LibPanel );
                         wxPostEvent( wxTheApp->GetTopWindow(), evt );
                     }
                 }
@@ -297,6 +394,8 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
             evtup.SetExtraLong( index );
             wxPostEvent( m_MainFrame, evtup );
         }
+
+        m_Db->ExecuteUpdate( wxT( "COMMIT TRANSACTION;" ) );
     }
 
     //
@@ -312,9 +411,24 @@ guLibUpdateThread::ExitCode guLibUpdateThread::Entry()
 
 // -------------------------------------------------------------------------------- //
 // guLibCleanThread
+// -------------------------------------------------------------------------------- //
+guLibCleanThread::guLibCleanThread( guLibPanel * libpanel )
+{
+    m_LibPanel = libpanel;
+    m_Db = libpanel->GetDb();
+    m_MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
+
+    if( Create() == wxTHREAD_NO_ERROR )
+    {
+        SetPriority( WXTHREAD_DEFAULT_PRIORITY );
+        Run();
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 guLibCleanThread::guLibCleanThread( guDbLibrary * db )
 {
+    m_LibPanel = NULL;
     m_Db = db;
     m_MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
 
@@ -332,6 +446,7 @@ guLibCleanThread::~guLibCleanThread()
     {
         wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_LIBRARY_CLEANFINISHED );
         event.SetEventObject( ( wxObject * ) this );
+        event.SetClientData( ( void * ) m_LibPanel );
         wxPostEvent( m_MainFrame, event );
     }
 }
@@ -345,8 +460,16 @@ guLibCleanThread::ExitCode guLibCleanThread::Entry()
     wxSQLite3ResultSet dbRes;
     wxString FileName;
 
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-    wxArrayString LibPaths = Config->ReadAStr( wxT( "LibPath" ), wxEmptyString, wxT( "LibPaths" ) );
+    wxArrayString LibPaths;
+    if( m_LibPanel )
+    {
+        LibPaths = m_LibPanel->GetLibraryPaths();
+    }
+    else
+    {
+        guConfig * Config = ( guConfig * ) guConfig::Get();
+        LibPaths = Config->ReadAStr( wxT( "LibPath" ), wxEmptyString, wxT( "LibPaths" ) );
+    }
 
     if( !TestDestroy() )
     {
@@ -358,7 +481,7 @@ guLibCleanThread::ExitCode guLibCleanThread::Entry()
 
         while( !TestDestroy() && dbRes.NextRow() )
         {
-            FileName = dbRes.GetString( 2 ) + wxT( "/" ) + dbRes.GetString( 1 );
+            FileName = dbRes.GetString( 2 ) + dbRes.GetString( 1 );
             //guLogMessage( wxT( "Checking %s" ), FileName.c_str() );
 
             if( !wxFileExists( FileName ) || !CheckFileLibPath( LibPaths, FileName ) )
@@ -401,13 +524,8 @@ guLibCleanThread::ExitCode guLibCleanThread::Entry()
                 }
 
                 // Delete all posible orphan entries
-//                Queries.Add( wxT( "DELETE FROM genres WHERE genre_id NOT IN ( SELECT DISTINCT song_genreid FROM songs );" ) );
-//                Queries.Add( wxT( "DELETE FROM artists WHERE artist_id NOT IN ( SELECT DISTINCT song_artistid FROM songs );" ) );
-//                Queries.Add( wxT( "DELETE FROM composers WHERE composer_id NOT IN ( SELECT DISTINCT song_composerid FROM songs );" ) );
-//                Queries.Add( wxT( "DELETE FROM albums WHERE album_id NOT IN ( SELECT DISTINCT song_albumid FROM songs );" ) );
                 Queries.Add( wxT( "DELETE FROM covers WHERE cover_id NOT IN ( SELECT DISTINCT song_coverid FROM songs );" ) );
-//                Queries.Add( wxT( "DELETE FROM paths WHERE path_id NOT IN ( SELECT DISTINCT song_pathid FROM songs );" ) );
-                Queries.Add( wxT( "DELETE FROM plsets WHERE plset_type = 0 AND plset_songid NOT IN ( SELECT DISTINCT song_id FROM songs );" ) );
+                Queries.Add( wxT( "DELETE FROM plsets WHERE plset_type = 0 AND plset_songid > 0 AND plset_songid NOT IN ( SELECT DISTINCT song_id FROM songs );" ) );
                 Queries.Add( wxT( "DELETE FROM settags WHERE settag_songid NOT IN ( SELECT DISTINCT song_id FROM songs );" ) );
 
                 int Index;
