@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -652,14 +652,58 @@ void guListView::OnMouse( wxMouseEvent &event )
     event.Skip();
 }
 
-//// -------------------------------------------------------------------------------- //
-//void guListView::OnSysColorChanged( wxSysColourChangedEvent &event )
-//{
-//    guLogMessage( wxT( "The sys color changed" ) );
-//
-//    m_Attr.LoadSysColors();
-//    event.Skip();
-//}
+// -------------------------------------------------------------------------------- //
+int FindColumnId( const guListViewColumnArray * columns, const int id )
+{
+    int index;
+    int count = columns->Count();
+    for( index = 0; index < count; index++ )
+    {
+        if( ( * columns )[ index ].m_Id == id )
+            return index;
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
+bool  guListView::GetColumnData( const int id, int * index, int * width, bool * enabled )
+{
+    int ColPos = FindColumnId( m_Columns, id );
+    if( ColPos != wxNOT_FOUND )
+    {
+        * index   = ColPos;
+        * width   = ( * m_Columns )[ ColPos ].m_Width;
+        * enabled = ( * m_Columns )[ ColPos ].m_Enabled;
+        return true;
+    }
+    return false;
+}
+
+// -------------------------------------------------------------------------------- //
+bool  guListView::SetColumnData( const int id, const int index, const int width, const bool enabled, const bool refresh )
+{
+    int ColPos = FindColumnId( m_Columns, id );
+    if( ColPos != wxNOT_FOUND )
+    {
+        if( ColPos != index )
+        {
+            guListViewColumn * Column = m_Columns->Detach( ColPos );
+            m_Columns->Insert( Column, index );
+        }
+        ( * m_Columns )[ index ].m_Width = width;
+        ( * m_Columns )[ index ].m_Enabled = enabled;
+        if( refresh )
+        {
+            m_Header->RefreshWidth();
+            Refresh();
+        }
+        return true;
+    }
+    return false;
+}
+
+
+
 
 // -------------------------------------------------------------------------------- //
 // guListViewClient
@@ -1515,19 +1559,6 @@ guListViewColEdit::~guListViewColEdit()
 	m_DownBitmapBtn->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guListViewColEdit::OnDownBtnClick ), NULL, this );
 }
 
-// -------------------------------------------------------------------------------- //
-int FindColumnId( const guListViewColumnArray * columns, const int id )
-{
-    int index;
-    int count = columns->Count();
-    for( index = 0; index < count; index++ )
-    {
-        if( ( * columns )[ index ].m_Id == id )
-            return index;
-    }
-    return wxNOT_FOUND;
-}
-
 // -------------------------------------------------------------------------------- //
 void guListViewColEdit::UpdateColumns( void )
 {
@@ -1672,7 +1703,6 @@ void guListViewDropFilesThread::AddDropFiles( const wxString &dirname )
     wxString FileName;
     //wxString SavedDir( wxGetCwd() );
 
-    //guLogMessage( wxT( "Adding drop item: '%s'" ), dirname.c_str() );
     if( wxDirExists( dirname ) )
     {
         wxString DirName = dirname;
@@ -1711,12 +1741,10 @@ guListViewDropFilesThread::ExitCode guListViewDropFilesThread::Entry()
 {
     int index;
     int Count = m_Files.Count();
-    int Length = 0;
     for( index = 0; index < Count; ++index )
     {
         if( TestDestroy() )
             return 0;
-        Length += m_Files[ index ].Length();
         AddDropFiles( m_Files[ index ] );
     }
 
@@ -1747,7 +1775,6 @@ guListViewDropTarget::~guListViewDropTarget()
 // -------------------------------------------------------------------------------- //
 bool guListViewDropTarget::OnDropFiles( wxCoord x, wxCoord y, const wxArrayString &files )
 {
-    //guLogMessage( wxT( "guListViewDropTarget::OnDropFiles" ) );
     // We are moving items inside this object.
     if( m_ListView->m_DragSelfItems )
     {
