@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -228,13 +228,12 @@ class guListViewClient : public wxVListBox
     guListViewClient( wxWindow * parent, const int flags, guListViewColumnArray * columns, guListViewAttr * attr );
     ~guListViewClient();
 
-    void SetItemHeigth( const int height );
+    void            SetItemHeigth( const int height );
 
-    void SetHScrollbar( const int width );
-    virtual void SetScrollPos( int orientation, int pos, bool refresh = true );
+    void            SetHScrollbar( const int width );
+    virtual void    SetScrollPos( int orientation, int pos, bool refresh = true );
     //int  GetHScrollPosition( void );
 
-
     friend class guListView;
     friend class guListViewHeader;
     friend class guListViewClientTimer;
@@ -355,6 +354,9 @@ class guListView : public wxScrolledWindow
     int                     GetColumnId( const int col ) const { return m_Columns->Item( col ).m_Id; }
     void                    SetColumnImage( const int col, const int imageindex );
 
+    bool                    GetColumnData( const int id, int * index, int * width, bool * enabled );
+    bool                    SetColumnData( const int id, const int index, const int width, const bool enabled, const bool refresh = false );
+
     wxRect                  GetClientScreenRect( void ) { return m_ListBox->GetScreenRect(); }
 
     bool                    IsAllowedColumnSelect( void ) const { return m_ColSelect; }
