@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -32,12 +32,22 @@
 #include <wx/image.h>
 #include <wx/tooltip.h>
 #include <wx/stdpaths.h>
+#include <wx/utils.h>
+
+#include <wx/debugrpt.h>
 
 IMPLEMENT_APP(guMainApp);
 
 // -------------------------------------------------------------------------------- //
 guMainApp::guMainApp() : wxApp()
 {
+    m_Db = NULL;
+    m_DbCache = NULL;
+
+#if wxUSE_ON_FATAL_EXCEPTION    // Thanks TheBigRed
+        wxHandleFatalExceptions();
+#endif
+
     if( !wxDirExists( wxGetHomeDir() + wxT( "/.guayadeque" ) ) )
     {
         wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque" ), 0770 );
@@ -50,6 +60,26 @@ guMainApp::guMainApp() : wxApp()
         guLogMessage( wxT( "Created the lyrics directory" ) );
     }
 
+    if( !wxDirExists( wxGetHomeDir() + wxT( "/.guayadeque/Jamendo" ) ) )
+    {
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Jamendo" ), 0770 );
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Jamendo/Covers" ), 0770 );
+        guLogMessage( wxT( "Created the Jamendo directory" ) );
+    }
+
+    if( !wxDirExists( wxGetHomeDir() + wxT( "/.guayadeque/Magnatune" ) ) )
+    {
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Magnatune" ), 0770 );
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Magnatune/Covers" ), 0770 );
+        guLogMessage( wxT( "Created the Magnatune directory" ) );
+    }
+
+    if( !wxDirExists( wxGetHomeDir() + wxT( "/.guayadeque/Devices" ) ) )
+    {
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Devices" ), 0770 );
+        guLogMessage( wxT( "Created the Devices directory" ) );
+    }
+
     if( !wxFileExists( wxGetHomeDir() + wxT( "/.guayadeque/guayadeque.conf" ) ) )
     {
         if( wxFileExists( wxT( "/usr/share/guayadeque/guayadeque.default.conf" ) ) )
@@ -80,8 +110,30 @@ guMainApp::guMainApp() : wxApp()
         guLogMessage( wxT( "Created the default equalizers file" ) );
     }
 
+    if( !wxFileExists( wxGetHomeDir() + wxT( "/.guayadeque/lyrics_sources.xml" ) ) )
+    {
+        if( wxFileExists( wxT( "/usr/share/guayadeque/lyrics_sources.xml" ) ) )
+        {
+            wxCopyFile( wxT( "/usr/share/guayadeque/lyrics_sources.xml" ),
+                        wxGetHomeDir() + wxT( "/.guayadeque/lyrics_sources.xml" ), false );
+        }
+        else if( wxFileExists( wxT( "/usr/local/share/guayadeque/lyrics_sources.xml" ) ) )
+        {
+            wxCopyFile( wxT( "/usr/local/share/guayadeque/lyrics_sources.xml" ),
+                        wxGetHomeDir() + wxT( "/.guayadeque/lyrics_sources.xml" ), false );
+        }
+        guLogMessage( wxT( "Created the default lyrics sources file" ) );
+    }
+
+    if( !wxDirExists( wxGetHomeDir() + wxT( "/.guayadeque/Layouts" ) ) )
+    {
+        wxMkdir( wxGetHomeDir() + wxT( "/.guayadeque/Layouts" ), 0770 );
+        guLogMessage( wxT( "Created the Layouts directory" ) );
+    }
+
     m_Config = new guConfig();
     guConfig::Set( m_Config );
+
 }
 
 // -------------------------------------------------------------------------------- //
@@ -122,6 +174,7 @@ bool SendFilesByMPRIS( const int argc, wxChar * argv[] )
     DBusConnection * dbconn;
     DBusMessage * dbmsg, * dbreply;
     DBusMessageIter dbiter;
+    DBusMessageIter dbitertracks;
 
     dbus_error_init( &dberr );
     dbconn = dbus_bus_get( DBUS_BUS_SESSION, &dberr );
@@ -133,65 +186,98 @@ bool SendFilesByMPRIS( const int argc, wxChar * argv[] )
          return false;
     }
 
+
     dbmsg = dbus_message_new_method_call( GUAYADEQUE_MPRIS_SERVICENAME,
                                           GUAYADEQUE_MPRIS_TRACKLIST_PATH,
                                           GUAYADEQUE_MPRIS_INTERFACE,
-                                          "AddTrack" );
+                                          "AddTracks" );
     if( dbmsg == NULL )
     {
          guLogError( wxT( "Couldn’t create a DBusMessage" ) );
          return false;
     }
 
+    dbus_message_iter_init_append( dbmsg, &dbiter );
+    dbus_message_iter_open_container( &dbiter, DBUS_TYPE_ARRAY, "s", &dbitertracks );
+
     wxString FilePath;
-    bool PlayTrack;
     int index;
     for( index = 1; index < argc; index++ )
     {
-        PlayTrack = ( index == 1 );
         FilePath = argv[ index ];
         //guLogMessage( wxT( "Trying to add file '%s'" ), argv[ index ] );
 
-        dbus_message_iter_init_append( dbmsg, &dbiter );
-
-        //dbus_message_iter_append_basic( &dbiter, DBUS_TYPE_STRING, &FilePath.char_str() );
-        Append_String( &dbiter, FilePath.char_str() );
-        dbus_message_iter_append_basic( &dbiter, DBUS_TYPE_BOOLEAN, &PlayTrack );
-
-        dbus_error_init( &dberr );
-
-        dbreply = dbus_connection_send_with_reply_and_block( dbconn, dbmsg, 5000, &dberr );
-        if( dbus_error_is_set( &dberr ) )
-        {
-              guLogMessage( wxT( "Error adding file %s" ), FilePath.c_str() );
-              printf( "Error getting a reply: %s\n", dberr.message );
-              dbus_message_unref( dbmsg );
-              dbus_error_free( &dberr );
-              return false;
-        }
-
-        dbus_message_unref( dbreply );
-
-        /* Don’t need this anymore */
-        dbus_message_unref( dbmsg );
+        Append_String( &dbitertracks, FilePath.char_str() );
     }
 
-    // Send the Play
-    dbmsg = dbus_message_new_method_call( GUAYADEQUE_MPRIS_SERVICENAME,
-                                          GUAYADEQUE_MPRIS_TRACKLIST_PATH,
-                                          GUAYADEQUE_MPRIS_INTERFACE,
-                                          "Play" );
+    dbus_message_iter_close_container( &dbiter, &dbitertracks );
+
+    dbus_bool_t PlayTrack = false;
+    dbus_message_iter_append_basic( &dbiter, DBUS_TYPE_BOOLEAN, &PlayTrack );
+
     dbreply = dbus_connection_send_with_reply_and_block( dbconn, dbmsg, 5000, &dberr );
     if( dbus_error_is_set( &dberr ) )
     {
-          guLogMessage( wxT( "Error sending Play" ) );
-          printf( "Error getting a reply: %s\n", dberr.message );
+          guLogMessage( wxT( "Error adding files: '%s'" ), wxString( dberr.message, wxConvUTF8 ).c_str() );
+          dbus_message_unref( dbmsg );
           dbus_error_free( &dberr );
+          return false;
     }
-    dbus_message_unref( dbreply );
+
+    if( dbreply )
+        dbus_message_unref( dbreply );
+
+    /* Don’t need this anymore */
+    dbus_message_unref( dbmsg );
+
+    dbus_connection_unref( dbconn );
+
+    return true;
+}
+
+// -------------------------------------------------------------------------------- //
+bool MakeWindowVisible( void )
+{
+    DBusError dberr;
+    DBusConnection * dbconn;
+    DBusMessage * dbmsg, * dbreply;
+
+    dbus_error_init( &dberr );
+    dbconn = dbus_bus_get( DBUS_BUS_SESSION, &dberr );
+
+    if( dbus_error_is_set( &dberr ) )
+    {
+         printf( "getting session bus failed: %s\n", dberr.message );
+         dbus_error_free( &dberr );
+         return false;
+    }
+
+
+    dbmsg = dbus_message_new_method_call( "org.mpris.MediaPlayer2.guayadeque",
+                                          "/org/mpris/MediaPlayer2",
+                                          "org.mpris.MediaPlayer2",
+                                          "Raise" );
+    if( dbmsg == NULL )
+    {
+         guLogError( wxT( "Couldn’t create a DBusMessage" ) );
+         return false;
+    }
+
+    dbreply = dbus_connection_send_with_reply_and_block( dbconn, dbmsg, 5000, &dberr );
+    if( dbus_error_is_set( &dberr ) )
+    {
+          guLogMessage( wxT( "Error showing window" ) );
+          dbus_message_unref( dbmsg );
+          dbus_error_free( &dberr );
+          return false;
+    }
+
+    if( dbreply )
+        dbus_message_unref( dbreply );
+
+    /* Don’t need this anymore */
     dbus_message_unref( dbmsg );
 
-    dbus_connection_close( dbconn );
     dbus_connection_unref( dbconn );
 
     return true;
@@ -205,6 +291,7 @@ bool guMainApp::OnInit()
     wxLog::SetActiveTarget( new wxLogStderr( stdout ) );
 
     const wxString AppName = wxString::Format( wxT( ".guayadeque/.guayadeque-%s" ), wxGetUserId().c_str() );
+    //guLogMessage( wxT( "Init: %s" ), AppName.c_str() );
     m_SingleInstanceChecker = new wxSingleInstanceChecker( AppName );
     if( m_SingleInstanceChecker->IsAnotherRunning() )
     {
@@ -221,7 +308,9 @@ bool guMainApp::OnInit()
             }
         }
 
-        guLogError( wxT( "Another program instance is already running, aborting." ) );
+        MakeWindowVisible();
+
+        guLogMessage( wxT( "Another program instance is already running, aborting." ) );
         return false;
     }
 
@@ -331,3 +420,15 @@ int guMainApp::OnExit()
 }
 
 // -------------------------------------------------------------------------------- //
+void guMainApp::OnFatalException()
+{
+    wxDebugReport Report;
+    wxDebugReportPreviewStd Preview;
+
+    Report.AddAll();
+
+    if( Preview.Show( Report ) )
+        Report.Process();
+}
+
+// -------------------------------------------------------------------------------- //
