@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -57,6 +57,14 @@ enum guRecordQuality {
     guRECORD_QUALITY_VERY_LOW
 };
 
+enum guOutputDeviceSink {
+    guOUTPUT_DEVICE_AUTOMATIC,
+    guOUTPUT_DEVICE_GCONF,
+    guOUTPUT_DEVICE_ALSA,
+    guOUTPUT_DEVICE_PULSEAUDIO,
+    guOUTPUT_DEVICE_OSS
+};
+
 // -------------------------------------------------------------------------------- //
 class guLevelInfo
 {
@@ -322,6 +330,9 @@ class guMediaCtrl : public wxEvtHandler
     guFaderPlayBinArray     m_FaderPlayBins;
     guFaderPlayBin *        m_CurrentPlayBin;
 
+    int                     m_OutputDevice;
+    wxString                m_OutputDeviceName;
+    bool                    m_ForceGapless;
     int                     m_FadeOutTime;
     int                     m_FadeInTime;
     double                  m_FadeInVolStart;
@@ -338,6 +349,12 @@ class guMediaCtrl : public wxEvtHandler
 
     int                     m_LastError;
 
+    int                     m_BufferSize;
+
+    bool                    m_ReplayGainMode;
+    double                  m_ReplayGainPreAmp;
+    //double                  m_ReplayGainFallback;
+
     bool                    RemovePlayBin( guFaderPlayBin * playbin );
 
     void                    FadeOutDone( guFaderPlayBin * faderplaybin );
@@ -364,8 +381,12 @@ class guMediaCtrl : public wxEvtHandler
     wxFileOffset    Length( void );
 
     bool            IsBuffering( void ) { return ( m_CurrentPlayBin && m_CurrentPlayBin->IsBuffering() ); }
+    int             BufferSize( void ) { return m_BufferSize; }
     bool            IsRecording( void ) { return m_IsRecording; }
 
+    bool            ForceGapless( void ) { return m_ForceGapless; }
+    void            ForceGapless( const bool forcegapless ) { m_ForceGapless = forcegapless; }
+
     double          GetVolume( void ) { return m_Volume; }
     bool            SetVolume( double volume );
 
@@ -396,6 +417,9 @@ class guMediaCtrl : public wxEvtHandler
     void            DisableRecord( void );
     bool            SetRecordFileName( const wxString &filename );
 
+    int             OutputDevice( void ) { return m_OutputDevice; }
+    wxString        OutputDeviceName( void ) { return m_OutputDeviceName; }
+
     friend class guFaderPlayBin;
 };
 
