@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -101,9 +101,9 @@ int GetXmlTrackLength( wxXmlNode * XmlNode )
     {
         if( XmlNode->GetName() == wxT( "duration" ) )
         {
-            int RetVal;
-            XmlNode->GetNodeContent().ToLong( ( long int * ) &RetVal );
-            return RetVal;
+            long RetVal;
+            XmlNode->GetNodeContent().ToLong( &RetVal );
+            return ( int ) RetVal;
         }
         XmlNode = XmlNode->GetNext();
     }
@@ -121,7 +121,9 @@ void ReadXmlTrack( wxXmlNode * XmlNode, guMBTrack * track )
         }
         else if( XmlNode->GetName() == wxT( "duration" ) )
         {
-            XmlNode->GetNodeContent().ToLong( ( long int * ) &track->m_Length );
+            long LongValue;
+            XmlNode->GetNodeContent().ToLong( &LongValue );
+            track->m_Length = LongValue;
         }
         else if( XmlNode->GetName() == wxT( "artist" ) )
         {
@@ -306,7 +308,6 @@ void ReadXmlReleases( wxXmlNode * XmlNode, guMBReleaseArray * mbreleases )
 // -------------------------------------------------------------------------------- //
 void guMusicBrainz::GetReleases( guMBReleaseArray * mbreleases, const wxString &artist, const wxString &title )
 {
-    wxASSERT( mbrelease );
     wxString QueryUrl = wxString::Format( wxT( guMUSICBRAINZ_URL_RELEASE_BY_TEXT
                 "&releasetypes=Official" ),
                 guURLEncode( artist ).c_str(), guURLEncode( title ).c_str() );
@@ -333,7 +334,6 @@ void guMusicBrainz::GetReleases( guMBReleaseArray * mbreleases, const wxString &
 // -------------------------------------------------------------------------------- //
 void guMusicBrainz::GetRelease( guMBRelease * mbrelease, const wxString &releaseid )
 {
-    wxASSERT( mbrelease );
     wxString QueryUrl = wxString::Format( wxT( guMUSICBRAINZ_URL_RELEASE_ID
                 "&inc=tracks+artist+release-events" ), releaseid.c_str() );
 
