@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -107,9 +107,11 @@ guNewPodcastChannelSelector::guNewPodcastChannelSelector( wxWindow * parent ) :
 	DirectoryTopSizer->Add( m_DirectoryInfoStaticText, 1, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxLEFT, 5 );
 
 	m_FilterDirectory = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_tiny_search ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
+	m_FilterDirectory->SetToolTip( _( "Search in the podcast directoy" ) );
 	DirectoryTopSizer->Add( m_FilterDirectory, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5 );
 
 	m_DirectoryReload = new wxBitmapButton( this, wxID_ANY, guImage( guIMAGE_INDEX_tiny_reload ), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
+	m_DirectoryReload->SetToolTip( _( "Refresh the podcast directory list" ) );
 	DirectoryTopSizer->Add( m_DirectoryReload, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 5 );
 
 	DirectoryStaticSizer->Add( DirectoryTopSizer, 0, wxEXPAND, 5 );
@@ -203,9 +205,6 @@ int guNewPodcastChannelSelector::ReadNewPodcastChannel( wxXmlNode * XmlNode, guN
 // -------------------------------------------------------------------------------- //
 int guNewPodcastChannelSelector::ReadNewPodcastChannels( wxXmlNode * XmlNode )
 {
-    wxASSERT( XmlNode );
-    wxASSERT( podcastarray );
-
     guNewPodcastCategory * NewPodcastChannel = NULL;
 
     while( XmlNode && XmlNode->GetName() == wxT( "outline" ) )
@@ -246,8 +245,6 @@ int guNewPodcastChannelSelector::ReadNewPodcastChannels( wxXmlNode * XmlNode )
 // -------------------------------------------------------------------------------- //
 int guNewPodcastChannelSelector::LoadNewPodcastsFromXml( const wxString &filename )
 {
-    wxASSERT( podcastarray );
-
     wxFFileInputStream ins( filename );
     wxXmlDocument XmlDoc( ins );
     wxXmlNode * XmlNode = XmlDoc.GetRoot();
@@ -276,7 +273,7 @@ void guNewPodcastChannelSelector::LoadPodcastDirectory( void )
     guConfig * Config = ( guConfig * ) guConfig::Get();
 
     wxString PodcastsPath = Config->ReadStr( wxT( "Path" ),
-                                wxGetHomeDir() + wxT( ".guayadeque/Podcasts" ), wxT( "Podcasts" ) );
+                                wxGetHomeDir() + wxT( "/.guayadeque/Podcasts" ), wxT( "Podcasts" ) );
 
     wxSetCursor( * wxHOURGLASS_CURSOR );
     wxTheApp->Yield();
@@ -380,7 +377,7 @@ void guNewPodcastChannelSelector::OnReloadDirectoryClicked( wxCommandEvent &even
     guConfig * Config = ( guConfig * ) guConfig::Get();
 
     wxString PodcastsPath = Config->ReadStr( wxT( "Path" ),
-                                wxGetHomeDir() + wxT( ".guayadeque/Podcasts" ), wxT( "Podcasts" ) );
+                                wxGetHomeDir() + wxT( "/.guayadeque/Podcasts" ), wxT( "Podcasts" ) );
 
     wxRemoveFile( PodcastsPath + wxT( "/Podcast.Directory.xml" ) );
 
