@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@
 // -------------------------------------------------------------------------------- //
 #include "PLSoListBox.h"
 
+#include "Accelerators.h"
 #include "Config.h" // Configuration
 #include "Commands.h"
 #include "Images.h"
@@ -32,10 +33,13 @@
 
 // -------------------------------------------------------------------------------- //
 guPLSoListBox::guPLSoListBox( wxWindow * parent, guDbLibrary * db, wxString confname, int style ) :
-             guSoListBox( parent, db, confname, style | guLISTVIEW_ALLOWDRAG | guLISTVIEW_ALLOWDROP | guLISTVIEW_DRAGSELFITEMS )
+             guSoListBox( parent, NULL, db, confname, style | guLISTVIEW_ALLOWDRAG | guLISTVIEW_ALLOWDROP | guLISTVIEW_DRAGSELFITEMS )
 {
-    m_PLId = wxNOT_FOUND;
-    m_PLType = wxNOT_FOUND;
+    //m_PLId = wxNOT_FOUND;
+    //m_PLType = wxNOT_FOUND;
+    m_LibPanel = NULL;
+
+    CreateAcceleratorTable();
 
     ReloadItems();
 }
@@ -45,33 +49,45 @@ guPLSoListBox::~guPLSoListBox()
 {
 }
 
-//// -------------------------------------------------------------------------------- //
-//void guPLSoListBox::ItemsCheckRange( const int start, const int end )
-//{
-////    guLogMessage( wxT( "guPLSoListBox::ItemsCheckRange( %i, %i )" ), start, end );
-////    if( m_ItemsFirst != start || m_ItemsLast != end )
-////    {
-////        m_Items.Empty();
-////
-////        if( m_PLId > 0 )
-////        {
-////            m_Db->GetPlayListSongs( m_PLId, m_PLType, &m_Items, &m_TracksLength, &m_TracksSize );
-////            m_Db->GetPlayListSetIds( m_PLId, &m_PLSetIds );
-////        }
-////
-////        m_ItemsFirst = start;
-////        m_ItemsLast = end;
-////    }
-//}
+// -------------------------------------------------------------------------------- //
+void guPLSoListBox::CreateAcceleratorTable( void )
+{
+    wxAcceleratorTable AccelTable;
+    wxArrayInt AliasAccelCmds;
+    wxArrayInt RealAccelCmds;
+
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SAVE );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITLABELS );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITTRACKS );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SEARCH );
+
+    RealAccelCmds.Add( ID_SONG_SAVETOPLAYLIST );
+    RealAccelCmds.Add( ID_SONG_EDITLABELS );
+    RealAccelCmds.Add( ID_SONG_EDITTRACKS );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    RealAccelCmds.Add( ID_PLAYLIST_SEARCH );
+
+    if( guAccelDoAcceleratorTable( AliasAccelCmds, RealAccelCmds, AccelTable ) )
+    {
+        SetAcceleratorTable( AccelTable );
+    }
+}
 
 // -------------------------------------------------------------------------------- //
 void guPLSoListBox::GetItemsList( void )
 {
     m_PLSetIds.Empty();
-    if( m_PLId > 0 )
+    if( m_PLIds.Count() )
     {
-        m_Db->GetPlayListSongs( m_PLId, m_PLType, &m_Items, &m_TracksLength, &m_TracksSize );
-        m_Db->GetPlayListSetIds( m_PLId, &m_PLSetIds );
+        m_Db->GetPlayListSongs( m_PLIds, m_PLTypes, &m_Items, &m_TracksLength, &m_TracksSize );
+        m_Db->GetPlayListSetIds( m_PLIds, &m_PLSetIds );
     }
     else
     {
@@ -86,8 +102,21 @@ void guPLSoListBox::GetItemsList( void )
 // -------------------------------------------------------------------------------- //
 void guPLSoListBox::SetPlayList( int plid, int pltype )
 {
-    m_PLId = plid;
-    m_PLType = pltype;
+    m_PLIds.Empty();
+    m_PLTypes.Empty();
+    if( plid != wxNOT_FOUND )
+    {
+        m_PLIds.Add( plid );
+        m_PLTypes.Add( pltype );
+    }
+    ReloadItems();
+}
+
+// -------------------------------------------------------------------------------- //
+void guPLSoListBox::SetPlayList( const wxArrayInt &ids, const wxArrayInt &types )
+{
+    m_PLIds = ids;
+    m_PLTypes = types;
     ReloadItems();
 }
 
@@ -100,9 +129,12 @@ void guPLSoListBox::CreateContextMenu( wxMenu * Menu ) const
     {
         guSoListBox::CreateContextMenu( Menu );
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE, _( "Remove from PlayList" ), _( "Delete the current selected tracks" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_del ) );
-        Menu->Insert( 7, MenuItem );
+        if( m_PLTypes.Count() == 1 && m_PLTypes[ 0 ] == guPLAYLIST_TYPE_STATIC )
+        {
+            MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE, _( "Remove from PlayList" ), _( "Delete the current selected tracks" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_del ) );
+            Menu->Insert( 7, MenuItem );
+        }
     }
 
 }
@@ -123,7 +155,7 @@ void guPLSoListBox::OnKeyDown( wxKeyEvent &event )
 // -------------------------------------------------------------------------------- //
 void guPLSoListBox::OnDropFile( const wxString &filename )
 {
-    if( m_PLId > 0 && m_PLType == GUPLAYLIST_STATIC )
+    if( ( m_PLIds.Count() == 1 ) && m_PLTypes[ 0 ] == guPLAYLIST_TYPE_STATIC )
     {
         //guLogMessage( wxT( "Adding file '%s'" ), filename.c_str() );
         if( guIsValidAudioFile( filename ) )
@@ -147,7 +179,7 @@ void guPLSoListBox::OnDropEnd( void )
     int count;
     wxArrayInt ItemIds;
 
-    if( m_PLId > 0 && m_PLType == GUPLAYLIST_STATIC )
+    if( ( m_PLIds.Count() == 1 ) && ( m_PLTypes[ 0 ] == guPLAYLIST_TYPE_STATIC ) )
     {
         if( m_DropIds.Count() )
         {
@@ -170,7 +202,8 @@ void guPLSoListBox::OnDropEnd( void )
             }
 
             // Save it to the database
-            m_Db->UpdateStaticPlayList( m_PLId, ItemIds );
+            m_Db->UpdateStaticPlayList( m_PLIds[ 0 ], ItemIds );
+            m_Db->UpdateStaticPlayListFile( m_PLIds[ 0 ] );
 
             m_DropIds.Clear();
         }
@@ -185,7 +218,7 @@ void guPLSoListBox::MoveSelection( void )
     wxArrayInt   MoveIndex;
     wxArrayInt   ItemIds;
 
-    if( m_PLType != GUPLAYLIST_STATIC )
+    if( ( m_PLIds.Count() != 1 ) || ( m_PLTypes[ 0 ] != guPLAYLIST_TYPE_STATIC ) )
         return;
 
     // Copy the elements we are going to move
@@ -229,7 +262,8 @@ void guPLSoListBox::MoveSelection( void )
     }
 
     // Save it to the database
-    m_Db->UpdateStaticPlayList( m_PLId, ItemIds );
+    m_Db->UpdateStaticPlayList( m_PLIds[ 0 ], ItemIds );
+    m_Db->UpdateStaticPlayListFile( m_PLIds[ 0 ] );
 
     ReloadItems();
 }
