@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -36,12 +36,14 @@ wxString LenToString( int Len );
 
 class guPlayerPanel;
 class guPlayList;
+class guMainFrame;
 
 // -------------------------------------------------------------------------------- //
 class guPlayList : public guListView
 {
   private :
     guDbLibrary *   m_Db;
+    guMainFrame *   m_MainFrame;
     guPlayerPanel * m_PlayerPanel;
     guTrackArray    m_Items;
     wxMutex         m_ItemsMutex;
@@ -55,6 +57,7 @@ class guPlayList : public guListView
     wxString        m_LastSearch;
 
     wxBitmap *      m_PlayBitmap;
+    wxBitmap *      m_StopBitmap;
     wxBitmap *      m_NormalStar;
     wxBitmap *      m_SelectStar;
     wxCoord         m_SecondLineOffset;
@@ -84,6 +87,7 @@ class guPlayList : public guListView
     void                        OnEditLabelsClicked( wxCommandEvent &event );
     void                        OnEditTracksClicked( wxCommandEvent &event );
     void                        OnSearchClicked( wxCommandEvent &event );
+    void                        OnStopAtEnd( wxCommandEvent &event ) { StopAtEnd(); }
     void                        OnSearchLinkClicked( wxCommandEvent &event );
     void                        OnCommandClicked( wxCommandEvent &event );
     wxString                    GetSearchText( int item ) const;
@@ -91,9 +95,13 @@ class guPlayList : public guListView
     void                        OnSelectTrack( wxCommandEvent &event );
     void                        OnSelectArtist( wxCommandEvent &event );
     void                        OnSelectAlbum( wxCommandEvent &event );
+    void                        OnSelectAlbumArtist( wxCommandEvent &event );
+    void                        OnSelectComposer( wxCommandEvent &event );
     void                        OnSelectYear( wxCommandEvent &event );
     void                        OnSelectGenre( wxCommandEvent &event );
 
+    void                        CreateAcceleratorTable( void );
+
   protected:
     virtual void                OnKeyDown( wxKeyEvent &event );
     virtual void                DrawItem( wxDC &dc, const wxRect &rect, const int row, const int col ) const;
@@ -108,15 +116,20 @@ class guPlayList : public guListView
     void                        OnDeleteFromLibrary( wxCommandEvent &event );
     void                        OnDeleteFromDrive( wxCommandEvent &event );
 
+    void                        OnSetRating( wxCommandEvent &event );
+
+    void                        SetTrackRating( const guTrack &track, const int rating );
+    void                        SetTracksRating( const guTrackArray &tracks, const int rating );
+
   public :
-    guPlayList( wxWindow * parent, guDbLibrary * db, guPlayerPanel * playerpanel = NULL );
+    guPlayList( wxWindow * parent, guDbLibrary * db, guPlayerPanel * playerpanel = NULL, guMainFrame * mainframe = NULL );
     ~guPlayList();
 
     void                        SetPlayerPanel( guPlayerPanel * playerpanel ) { m_PlayerPanel = playerpanel; }
 
     void                        AddItem( const guTrack &NewItem, const int pos = wxNOT_FOUND );
     //void                        AddItem( const guTrack * NewItem );
-    void                        AddPlayListItem( const wxString &FileName, bool AddPath = false, const int pos = wxNOT_FOUND );
+    void                        AddPlayListItem( const wxString &FileName, const int aftercurrent, const int pos );
 
     virtual void                ReloadItems( bool reset = true );
 
@@ -136,10 +149,10 @@ class guPlayList : public guListView
     void                        ClearItems();
     long                        GetLength( void ) const;
     wxString                    GetLengthStr( void ) const;
-    void                        AddToPlayList( const guTrackArray &newitems, const bool deleteold = false, const bool aftercurrent = false );
+    void                        AddToPlayList( const guTrackArray &newitems, const bool deleteold = false, const int aftercurrent = 0 ); //guINSERT_AFTER_CURRENT_NONE
     void                        SetPlayList( const guTrackArray &NewItems );
     wxString                    FindCoverFile( const wxString &DirName );
-    void                        Randomize( void );
+    void                        Randomize( const bool isplaying );
     int                         GetCaps();
     void                        RemoveItem( int itemnum );
 
@@ -148,6 +161,9 @@ class guPlayList : public guListView
 
     bool                        StartPlaying( void ) { return m_StartPlaying; }
 
+    void                        StopAtEnd( void );
+    void                        ClearStopAtEnd( void );
+
 
   friend class guAddDropFilesThread;
   friend class guPlayListDropTarget;
