@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,7 +21,7 @@
 #include "PlayListAppend.h"
 
 // -------------------------------------------------------------------------------- //
-guPlayListAppend::guPlayListAppend( wxWindow * parent, guDbLibrary * db, wxArrayInt * tracks, guListItems * plitems ) :
+guPlayListAppend::guPlayListAppend( wxWindow * parent, guDbLibrary * db, const wxArrayInt * tracks, guListItems * plitems ) :
     wxDialog( parent, wxID_ANY, _( "Save to Playlist" ), wxDefaultPosition, wxSize( 387,198 ), wxDEFAULT_DIALOG_STYLE )
 {
     wxStaticText *              PosLabel;
@@ -31,9 +31,6 @@ guPlayListAppend::guPlayListAppend( wxWindow * parent, guDbLibrary * db, wxArray
     wxButton *                  StdBtnSizerOK;
     wxButton *                  StdBtnSizerCancel;
 
-    wxASSERT( db );
-    wxASSERT( tracks );
-
     m_Db = db;
     m_Tracks = tracks;
 
@@ -126,7 +123,7 @@ int FindPlayListItem( guListItems * items, const wxString &playlistname )
 int guPlayListAppend::GetSelectedPlayList( void )
 {
     int Selection = m_PlayListComboBox->GetSelection();
-    if( Selection == wxNOT_FOUND && !m_PlayListComboBox->GetValue().IsEmpty() )
+    if( Selection == wxNOT_FOUND && !m_PlayListComboBox->IsEmpty() )
     {
         Selection = FindPlayListItem( m_PlayListItems, m_PlayListComboBox->GetValue().Lower().Trim().Trim( false ) );
     }
