@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -39,16 +39,16 @@
 class guPlayListAppend : public wxDialog
 {
   protected:
-    wxChoice *      m_PosChoice;
-    wxComboBox *    m_PlayListComboBox;
-    wxStaticText *  m_TracksStaticText;
+    wxChoice *              m_PosChoice;
+    wxComboBox *            m_PlayListComboBox;
+    wxStaticText *          m_TracksStaticText;
 
-    guDbLibrary *   m_Db;
-    wxArrayInt *    m_Tracks;
-    guListItems *   m_PlayListItems;
+    guDbLibrary *           m_Db;
+    const wxArrayInt *      m_Tracks;
+    guListItems *           m_PlayListItems;
 
   public:
-    guPlayListAppend( wxWindow * parent, guDbLibrary * db, wxArrayInt * songs, guListItems * items );
+    guPlayListAppend( wxWindow * parent, guDbLibrary * db, const wxArrayInt * songs, guListItems * items );
     ~guPlayListAppend();
 
     int         GetSelectedPosition( void );
