@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -33,6 +33,22 @@
 
 WX_DEFINE_OBJARRAY(guStationPlayList);
 
+// -------------------------------------------------------------------------------- //
+// guStationPlayListItem
+// -------------------------------------------------------------------------------- //
+wxString guStationPlayListItem::GetLocation( const bool relative, const wxString &pathbase )
+{
+    if( relative )
+    {
+        wxFileName FileName( m_Location );
+        if( FileName.MakeRelativeTo( pathbase ) )
+        {
+            return FileName.GetFullPath();
+        }
+    }
+    return m_Location;
+}
+
 // -------------------------------------------------------------------------------- //
 guPlayListFile::guPlayListFile( const wxString &uri )
 {
@@ -127,26 +143,26 @@ bool guPlayListFile::Load( const wxString &uri )
 }
 
 // -------------------------------------------------------------------------------- //
-bool guPlayListFile::Save( const wxString &filename )
+bool guPlayListFile::Save( const wxString &filename, const bool relative )
 {
     if( filename.Lower().EndsWith( wxT( ".pls" ) ) )
     {
-        return WritePlsFile( filename );
+        return WritePlsFile( filename, relative );
     }
     else if( filename.Lower().EndsWith( wxT( ".xspf" ) ) )
     {
-        return WriteXspfFile( filename );
+        return WriteXspfFile( filename, relative );
     }
     else if( filename.Lower().EndsWith( wxT( ".asx" ) ) )
     {
-        return WriteAsxFile( filename );
+        return WriteAsxFile( filename, relative );
     }
     else
     {
         wxString FileName = filename;
         if( !FileName.Lower().EndsWith( wxT( ".m3u" ) ) )
             FileName += wxT( ".m3u" );
-        return WriteM3uFile( FileName );
+        return WriteM3uFile( FileName, relative );
     }
     return false;
 }
@@ -180,7 +196,7 @@ bool guPlayListFile::ReadPlsStream( wxInputStream &playlist, const wxString &pat
                             Config->Read( wxString::Format( wxT( "Title%u" ), Index ), &Title );
 
                             wxURI Uri( Location );
-                            if( Uri.HasScheme() || path.IsEmpty() )
+                            if( Location.StartsWith( wxT( "/" ) ) || Uri.HasScheme() || path.IsEmpty() )
                             {
                                 m_PlayList.Add( new guStationPlayListItem( Location, Title ) );
                             }
@@ -256,7 +272,7 @@ bool guPlayListFile::ReadM3uStream( wxInputStream &playlist, const wxString &pat
             {
 
                 wxURI Uri( Lines[ Index ] );
-                if( Uri.HasScheme() || path.IsEmpty() )
+                if( Lines[ Index ].StartsWith( wxT( "/" ) ) || Uri.HasScheme() || path.IsEmpty() )
                 {
                     m_PlayList.Add( new guStationPlayListItem( Lines[ Index ], ItemName ) );
                 }
@@ -443,7 +459,7 @@ bool guPlayListFile::ReadAsxFile( const wxString &filename )
 }
 
 // -------------------------------------------------------------------------------- //
-bool guPlayListFile::WritePlsFile( const wxString &filename )
+bool guPlayListFile::WritePlsFile( const wxString &filename, const bool relative )
 {
     wxFile PlsFile;
     if( PlsFile.Create( filename, true ) && PlsFile.IsOpened() )
@@ -453,20 +469,22 @@ bool guPlayListFile::WritePlsFile( const wxString &filename )
         PlsFile.Write( wxString::Format( wxT( "numberofentries=%u\n" ), Count ) );
         for( int Index = 0; Index < Count; Index++ )
         {
-            PlsFile.Write( wxString::Format( wxT( "File%u=%s\n" ), Index + 1, m_PlayList[ Index ].m_Location.c_str() ) );
-            PlsFile.Write( wxString::Format( wxT( "Title%u=%s\n" ), Index + 1, m_PlayList[ Index ].m_Name.c_str() ) );
+            PlsFile.Write( wxString::Format( wxT( "File%u=%s\n" ), Index + 1,
+                                m_PlayList[ Index ].GetLocation( relative, wxPathOnly( filename ) ).c_str() ) );
+            PlsFile.Write( wxString::Format( wxT( "Title%u=%s\n" ), Index + 1,
+                                m_PlayList[ Index ].m_Name.c_str() ) );
         }
         PlsFile.Close();
     }
     else
     {
-        guLogError( wxT( "Could not open the m3ufile '%s'" ), filename.c_str() );
+        guLogError( wxT( "Could not open the plsfile '%s'" ), filename.c_str() );
     }
     return false;
 }
 
 // -------------------------------------------------------------------------------- //
-bool guPlayListFile::WriteM3uFile( const wxString &filename )
+bool guPlayListFile::WriteM3uFile( const wxString &filename, const bool relative )
 {
     wxFile M3uFile;
     if( M3uFile.Create( filename, true ) && M3uFile.IsOpened() )
@@ -475,7 +493,7 @@ bool guPlayListFile::WriteM3uFile( const wxString &filename )
         int Count = m_PlayList.Count();
         for( int Index = 0; Index < Count; Index++ )
         {
-            M3uFile.Write( m_PlayList[ Index ].m_Location );
+            M3uFile.Write( m_PlayList[ Index ].GetLocation( relative, wxPathOnly( filename ) ) );
             M3uFile.Write( wxT( "\n" ) );
         }
         M3uFile.Close();
@@ -488,7 +506,7 @@ bool guPlayListFile::WriteM3uFile( const wxString &filename )
 }
 
 // -------------------------------------------------------------------------------- //
-bool guPlayListFile::WriteXspfFile( const wxString &filename )
+bool guPlayListFile::WriteXspfFile( const wxString &filename, const bool relative )
 {
     wxXmlDocument OutXml;
     //OutXml.SetRoot(  );
@@ -507,7 +525,7 @@ bool guPlayListFile::WriteXspfFile( const wxString &filename )
         wxXmlNode * TrackNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "track" ) );
 
         wxXmlNode * LocationNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "location" ) );
-        wxXmlNode * LocationNodeVal = new wxXmlNode( wxXML_TEXT_NODE, wxT( "location" ), m_PlayList[ Index ].m_Location );
+        wxXmlNode * LocationNodeVal = new wxXmlNode( wxXML_TEXT_NODE, wxT( "location" ), m_PlayList[ Index ].GetLocation( relative, wxPathOnly( filename ) ) );
         LocationNode->AddChild( LocationNodeVal );
 
         wxXmlNode * TitleNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "title" ) );
@@ -525,7 +543,7 @@ bool guPlayListFile::WriteXspfFile( const wxString &filename )
 }
 
 // -------------------------------------------------------------------------------- //
-bool guPlayListFile::WriteAsxFile( const wxString &filename )
+bool guPlayListFile::WriteAsxFile( const wxString &filename, const bool relative )
 {
     wxXmlDocument OutXml;
     //OutXml.SetRoot(  );
@@ -542,7 +560,7 @@ bool guPlayListFile::WriteAsxFile( const wxString &filename )
         wxXmlNode * EntryNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "entry" ) );
 
         wxXmlNode * RefNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "ref" ) );
-        wxXmlProperty * RefNodeVal = new wxXmlProperty( wxT( "href" ), m_PlayList[ Index ].m_Location, NULL );
+        wxXmlProperty * RefNodeVal = new wxXmlProperty( wxT( "href" ), m_PlayList[ Index ].GetLocation( relative, wxPathOnly( filename ) ), NULL );
         RefNode->SetProperties( RefNodeVal );
 
         wxXmlNode * TitleNode = new wxXmlNode( wxXML_ELEMENT_NODE, wxT( "title" ) );
