@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 #ifndef PLAYLISTPANEL_H
 #define PLAYLISTPANEL_H
 
+#include "AuiManagedPanel.h"
 #include "DbLibrary.h"
 #include "PlayerPanel.h"
 #include "PLSoListBox.h"
@@ -54,6 +55,7 @@
 #define     guPANEL_PLAYLIST_VISIBLE_DEFAULT   ( 0 )
 
 class guPLNamesDropTarget;
+class guPlayListPanel;
 
 // -------------------------------------------------------------------------------- //
 // guPLNamesTreeCtrl
@@ -61,12 +63,13 @@ class guPLNamesDropTarget;
 class guPLNamesTreeCtrl : public wxTreeCtrl
 {
   protected :
-    wxArrayString   m_TextSearchFilter;
-    guDbLibrary *   m_Db;
-    wxImageList *   m_ImageList;
-    wxTreeItemId    m_RootId;
-    wxTreeItemId    m_StaticId;
-    wxTreeItemId    m_DynamicId;
+    guDbLibrary *       m_Db;
+    guPlayListPanel *   m_PlayListPanel;
+    wxArrayString       m_TextSearchFilter;
+    wxImageList *       m_ImageList;
+    wxTreeItemId        m_RootId;
+    wxTreeItemId        m_StaticId;
+    wxTreeItemId        m_DynamicId;
 
     wxTreeItemId    m_DragOverItem;
     wxArrayInt      m_DropIds;
@@ -79,8 +82,11 @@ class guPLNamesTreeCtrl : public wxTreeCtrl
     void            OnDropEnd( void );
     void            OnKeyDown( wxKeyEvent &event );
 
+    void            OnConfigUpdated( wxCommandEvent &event );
+    void            CreateAcceleratorTable( void );
+
   public :
-    guPLNamesTreeCtrl( wxWindow * parent, guDbLibrary * db );
+    guPLNamesTreeCtrl( wxWindow * parent, guDbLibrary * db, guPlayListPanel * playlistpanel );
     ~guPLNamesTreeCtrl();
 
     void            ReloadItems( void );
@@ -132,11 +138,9 @@ class guPLNamesDropTarget : public wxFileDropTarget
 };
 
 // -------------------------------------------------------------------------------- //
-class guPlayListPanel : public wxPanel
+class guPlayListPanel : public guAuiManagedPanel
 {
   protected :
-    wxAuiManager        m_AuiManager;
-
     guDbLibrary *       m_Db;
     guPlayerPanel *     m_PlayerPanel;
 
@@ -147,13 +151,16 @@ class guPlayListPanel : public wxPanel
     wxSearchCtrl *      m_InputTextCtrl;
 
     wxTimer             m_TextChangedTimer;
-    unsigned int        m_VisiblePanels;
+    wxString            m_ExportLastFolder;
+
+    bool                m_InstantSearchEnabled;
+    bool                m_EnterSelectSearchEnabled;
 
     void                OnPLNamesSelected( wxTreeEvent &event );
     void                OnPLNamesActivated( wxTreeEvent &event );
     void                OnPLNamesPlay( wxCommandEvent &event );
     void                OnPLNamesEnqueue( wxCommandEvent &event );
-    void                OnPLNamesEnqueueAsNext( wxCommandEvent &event );
+
     void                OnPLNamesNewPlaylist( wxCommandEvent &event );
     void                OnPLNamesEditPlaylist( wxCommandEvent &event );
     void                OnPLNamesRenamePlaylist( wxCommandEvent &event );
@@ -163,14 +170,10 @@ class guPlayListPanel : public wxPanel
     void                OnPLNamesImport( wxCommandEvent &event );
     void                OnPLNamesExport( wxCommandEvent &event );
 
-    void                OnPLTracksActivated( wxListEvent &event );
-    void                OnPLTracksDeleteClicked( wxCommandEvent &event );
+    virtual void        OnPLTracksActivated( wxListEvent &event );
     void                OnPLTracksPlayClicked( wxCommandEvent &event );
-    void                OnPLTracksPlayAllClicked( wxCommandEvent &event );
     void                OnPLTracksQueueClicked( wxCommandEvent &event );
-    void                OnPLTracksQueueAsNextClicked( wxCommandEvent &event );
-    void                OnPLTracksQueueAllClicked( wxCommandEvent &event );
-    void                OnPLTracksQueueAllAsNextClicked( wxCommandEvent &event );
+    void                OnPLTracksDeleteClicked( wxCommandEvent &event );
     void                OnPLTracksEditLabelsClicked( wxCommandEvent &event );
     void                OnPLTracksEditTracksClicked( wxCommandEvent &event );
     void                OnPLTracksCopyToClicked( wxCommandEvent &event );
@@ -185,22 +188,32 @@ class guPlayListPanel : public wxPanel
     void                OnPLTracksSelectAlbum( wxCommandEvent &event );
 
     void                OnSearchActivated( wxCommandEvent &event );
-    //void                OnSearchSelected( wxCommandEvent &event );
+    void                OnSearchSelected( wxCommandEvent &event );
     void                OnSearchCancelled( wxCommandEvent &event );
 
     void                OnTextChangedTimer( wxTimerEvent &event );
 
-    void                OnPaneClose( wxAuiManagerEvent &event );
-
     void                DeleteCurrentPlayList( void );
 
     void                OnPLTracksDeleteLibrary( wxCommandEvent &event );
     void                OnPLTracksDeleteDrive( wxCommandEvent &event );
 
+    virtual void        NormalizeTracks( guTrackArray * tracks, const bool isdrag = false ) {};
+    virtual void        SendPlayListUpdatedEvent( void );
+
+    void                OnGoToSearch( wxCommandEvent &event );
+    bool                DoTextSearch( void );
+
+    void                OnSetAllowDenyFilter( wxCommandEvent &event );
+
+    void                OnConfigUpdated( wxCommandEvent &event );
+
   public :
     guPlayListPanel( wxWindow * parent, guDbLibrary * db, guPlayerPanel * playerpanel );
     ~guPlayListPanel();
 
+    virtual void        InitPanelData( void );
+
     void                PlayListUpdated( void );
 
     bool                GetPlayListCounters( wxLongLong * count, wxLongLong * len, wxLongLong * size );
@@ -208,9 +221,11 @@ class guPlayListPanel : public wxPanel
     void inline         UpdatedTracks( const guTrackArray * tracks ) { m_PLTracksListBox->UpdatedTracks( tracks ); };
     void inline         UpdatedTrack( const guTrack * track ) { m_PLTracksListBox->UpdatedTrack( track ); };
 
-    bool                IsPanelShown( const int panelid ) const;
-    void                ShowPanel( const int panelid, bool show );
+    virtual int         GetListViewColumnCount( void ) { return guSONGS_COLUMN_COUNT; }
+    virtual bool        GetListViewColumnData( const int id, int * index, int * width, bool * enabled ) { return m_PLTracksListBox->GetColumnData( id, index, width, enabled ); }
+    virtual bool        SetListViewColumnData( const int id, const int index, const int width, const bool enabled, const bool refresh = false ) { return m_PLTracksListBox->SetColumnData( id, index, width, enabled, refresh ); }
 
+    friend class guPLNamesTreeCtrl;
 };
 
 #endif
