@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -50,12 +50,12 @@ guPlayerFilters::guPlayerFilters( wxWindow * parent, guDbLibrary * db ) :
 
 	wxStaticText * FiltersDenyLabel = new wxStaticText( this, wxID_ANY, _( "Deny:" ), wxDefaultPosition, wxDefaultSize, 0 );
 	FiltersDenyLabel->Wrap( -1 );
-	FiltersFlexSizer->Add( FiltersDenyLabel, 0, wxALIGN_CENTER_VERTICAL|wxBOTTOM|wxRIGHT|wxLEFT, 5 );
+	FiltersFlexSizer->Add( FiltersDenyLabel, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxLEFT, 5 );
 
 	wxArrayString m_FilterDenyChoiceChoices;
 	m_FilterDenyChoice = new wxChoice( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, m_FilterDenyChoiceChoices, 0 );
 	m_FilterDenyChoice->SetSelection( 0 );
-	FiltersFlexSizer->Add( m_FilterDenyChoice, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxBOTTOM|wxRIGHT, 5 );
+	FiltersFlexSizer->Add( m_FilterDenyChoice, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL|wxRIGHT, 5 );
 
 	FiltersMainSizer->Add( FiltersFlexSizer, 1, wxEXPAND, 5 );
 
@@ -105,8 +105,8 @@ void guPlayerFilters::UpdateFilters( void )
     m_FilterPlayLists.Empty();
 
     m_FilterPlayLists.Add( new guListItem( wxNOT_FOUND, _( "All" ) ) );
-    m_Db->GetPlayLists( &m_FilterPlayLists, GUPLAYLIST_STATIC );
-    m_Db->GetPlayLists( &m_FilterPlayLists, GUPLAYLIST_DYNAMIC );
+    m_Db->GetPlayLists( &m_FilterPlayLists, guPLAYLIST_TYPE_STATIC );
+    m_Db->GetPlayLists( &m_FilterPlayLists, guPLAYLIST_TYPE_DYNAMIC );
 
     wxArrayString ChoiceItems;
     int Index;
@@ -134,3 +134,15 @@ void guPlayerFilters::EnableFilters( const bool enable )
 }
 
 // -------------------------------------------------------------------------------- //
+void guPlayerFilters::SetAllowFilterId( const int id )
+{
+    m_FilterAllowChoice->SetSelection( GetListItemsIdIndex( m_FilterPlayLists, id ) );
+}
+
+// -------------------------------------------------------------------------------- //
+void guPlayerFilters::SetDenyFilterId( const int id )
+{
+    m_FilterDenyChoice->SetSelection( GetListItemsIdIndex( m_FilterPlayLists, id ) );
+}
+
+// -------------------------------------------------------------------------------- //
