@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -27,6 +27,7 @@
 #include "Utils.h"
 
 #include <wx/arrimpl.cpp>
+#include <wx/uri.h>
 
 WX_DEFINE_OBJARRAY(guPodcastChannelArray);
 WX_DEFINE_OBJARRAY(guPodcastItemArray);
@@ -40,12 +41,12 @@ int StrLengthToInt( const wxString &length )
     {
         // 1:02:03:04
         wxString Rest = length.Strip( wxString::both );
-        int element;
+        long element;
         int FactorIndex = 0;
         int RetVal = 0;
         int Factor[] = { 1, 60, 3600, 86400 };
         do {
-            Rest.AfterLast( wxT( ':' ) ).ToLong( ( long * ) &element );
+            Rest.AfterLast( wxT( ':' ) ).ToLong( &element );
             if( !element )
                 break;
             RetVal += ( Factor[ FactorIndex ] * element );
@@ -162,6 +163,8 @@ bool guPodcastChannel::ReadXml( wxXmlNode * XmlNode )
                 {
                     guPodcastItem * PodcastItem = new guPodcastItem( XmlNode->GetChildren() );
                     //PodcastItem->ReadXml( XmlNode->GetChildren() );
+                    if( PodcastItem->m_Author.IsEmpty() )
+                        PodcastItem->m_Author = m_Author;
                     //guLogMessage( wxT( "Item Length: %i" ), PodcastItem->m_Length );
                     m_Items.Add( PodcastItem );
                 }
@@ -227,8 +230,6 @@ void guPodcastChannel::CheckLogo( void )
 // -------------------------------------------------------------------------------- //
 int guPodcastChannel::GetUpdateItems( guDbLibrary * db, guPodcastItemArray * items )
 {
-  wxASSERT( items );
-
   wxString query;
   wxSQLite3ResultSet dbRes;
 
@@ -421,7 +422,7 @@ void guPodcastChannel::CheckDeleteItems( guDbLibrary * db )
                 return;
         }
 
-        query += wxString::Format( wxT( "AND podcastitem_addeddate < %u " ), DeleteTime.GetTicks() );
+        query += wxString::Format( wxT( "AND podcastitem_time < %u " ), DeleteTime.GetTicks() );
 
         //
         if( Config->ReadBool( wxT( "DeletePlayed" ), false, wxT( "Podcasts" ) ) )
@@ -452,9 +453,9 @@ void guPodcastChannel::Update( guDbLibrary * db, guMainFrame * mainframe )
         // Save only the new items in the channel
         db->SavePodcastChannel( this, true );
 
-        CheckDownloadItems( db, mainframe );
-
         CheckDeleteItems( db );
+
+        CheckDownloadItems( db, mainframe );
     }
 }
 
@@ -508,9 +509,12 @@ void guPodcastItem::ReadXml( wxXmlNode * XmlNode )
         else if( XmlNode->GetName() == wxT( "enclosure" ) )
         {
             XmlNode->GetPropVal( wxT( "url" ), &m_Enclosure );
+
             wxString LenStr;
             XmlNode->GetPropVal( wxT( "length" ), &LenStr );
-            LenStr.ToULong( ( unsigned long * ) &m_FileSize );
+            unsigned long ULongVal;
+            LenStr.ToULong( &ULongVal );
+            m_FileSize = ULongVal;
         }
         else if( m_Summary.IsEmpty() && ( ( XmlNode->GetName() == wxT( "itunes:summary" ) ) ||
                  ( XmlNode->GetName() == wxT( "description" ) ) ) )
@@ -573,8 +577,6 @@ void guPodcastDownloadQueueThread::SendUpdateEvent( guPodcastItem * podcastitem
 // -------------------------------------------------------------------------------- //
 void guPodcastDownloadQueueThread::AddPodcastItems( guPodcastItemArray * items, bool priority )
 {
-    wxASSERT( items );
-
     int Index;
     int Count = items->Count();
     if( Count )
@@ -605,8 +607,6 @@ void guPodcastDownloadQueueThread::AddPodcastItems( guPodcastItemArray * items,
 // -------------------------------------------------------------------------------- //
 int guPodcastDownloadQueueThread::FindPodcastItem( guPodcastItem * podcastitem )
 {
-    wxASSERT( podcastitem );
-
     int Index;
     int Count = m_Items.Count();
     for( Index = 0; Index < Count; Index++ )
@@ -621,8 +621,6 @@ int guPodcastDownloadQueueThread::FindPodcastItem( guPodcastItem * podcastitem )
 // -------------------------------------------------------------------------------- //
 void guPodcastDownloadQueueThread::RemovePodcastItems( guPodcastItemArray * items )
 {
-    wxASSERT( items );
-
     int Index;
     int ItemPos;
     int Count = items->Count();
@@ -687,10 +685,10 @@ guPodcastDownloadQueueThread::ExitCode guPodcastDownloadQueueThread::Entry()
                 wxDateTime PodcastTime;
                 PodcastTime.Set( ( time_t ) PodcastItem->m_Time );
 
-                wxFileName PodcastFile = wxFileName( m_PodcastsPath + wxT( "/" ) +
+                wxFileName PodcastFile( m_PodcastsPath + wxT( "/" ) +
                                             PodcastItem->m_Channel + wxT( "/" ) +
-                                            PodcastTime.Format( wxT( "%Y%m%d%H%M%S-" ) ) +
-                                            Uri.GetPath().AfterLast( wxT( '/' ) ) );
+                                            //PodcastTime.Format( wxT( "%Y%m%d%H%M%S-" ) ) +
+                                            Uri.BuildUnescapedURI().AfterLast( wxT( '/' ) ) );
                 if( PodcastFile.Normalize( wxPATH_NORM_ALL|wxPATH_NORM_CASE ) )
                 {
                     PodcastItem->m_FileName = PodcastFile.GetFullPath();
