@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@
 // -------------------------------------------------------------------------------- //
 #include "PodcastsPanel.h"
 
+#include "Accelerators.h"
 #include "AuiDockArt.h"
 #include "ChannelEditor.h"
 #include "Commands.h"
@@ -40,7 +41,7 @@
 // guPostcastPanel
 // -------------------------------------------------------------------------------- //
 guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame * mainframe, guPlayerPanel * playerpanel ) :
-    wxPanel( parent, wxID_ANY, wxDefaultPosition, wxSize( 672,586 ), wxTAB_TRAVERSAL )
+    guAuiManagedPanel( parent )
 {
     m_Db = db;
     m_MainFrame = mainframe;
@@ -65,6 +66,8 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
     guConfig * Config = ( guConfig * ) guConfig::Get();
     Config->RegisterObject( this );
 
+    InitPanelData();
+
     // Check that the directory to store podcasts are created
     m_PodcastsPath = Config->ReadStr( wxT( "Path" ), wxGetHomeDir() + wxT( "/.guayadeque/Podcasts" ), wxT( "Podcasts" ) );
     if( !wxDirExists( m_PodcastsPath ) )
@@ -72,35 +75,8 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
         wxMkdir( m_PodcastsPath, 0770 );
     }
 
-    m_AuiManager.SetManagedWindow( this );
-    m_AuiManager.SetArtProvider( new guAuiDockArt() );
-    m_AuiManager.SetFlags( wxAUI_MGR_ALLOW_FLOATING |
-                           wxAUI_MGR_TRANSPARENT_DRAG |
-                           wxAUI_MGR_TRANSPARENT_HINT );
-    wxAuiDockArt * AuiDockArt = m_AuiManager.GetArtProvider();
-    AuiDockArt->SetColour( wxAUI_DOCKART_INACTIVE_CAPTION_TEXT_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_INACTIVECAPTIONTEXT ) );
-    AuiDockArt->SetColour( wxAUI_DOCKART_ACTIVE_CAPTION_TEXT_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_CAPTIONTEXT ) );
-
-    AuiDockArt->SetColour( wxAUI_DOCKART_ACTIVE_CAPTION_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_ACTIVEBORDER ) );
-
-    AuiDockArt->SetColour( wxAUI_DOCKART_ACTIVE_CAPTION_GRADIENT_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_3DSHADOW ) );
-
-    AuiDockArt->SetColour( wxAUI_DOCKART_INACTIVE_CAPTION_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_INACTIVEBORDER ) );
-
-    AuiDockArt->SetColour( wxAUI_DOCKART_INACTIVE_CAPTION_GRADIENT_COLOUR,
-            wxSystemSettings::GetColour( wxSYS_COLOUR_3DSHADOW ) );
-
-    AuiDockArt->SetColour( wxAUI_DOCKART_GRADIENT_TYPE,
-            wxAUI_GRADIENT_VERTICAL );
-
     m_VisiblePanels = Config->ReadNum( wxT( "PodVisiblePanels" ), guPANEL_PODCASTS_VISIBLE_DEFAULT, wxT( "Positions" ) );
 
-
 	ChannelsPanel = new wxPanel( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
 	wxBoxSizer * ChannelsMainSizer;
 	ChannelsMainSizer = new wxBoxSizer( wxVERTICAL );
@@ -125,7 +101,7 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
 	MainPodcastsSizer = new wxBoxSizer( wxVERTICAL );
 
 	m_PodcastsListBox = new guPodcastListBox( PodcastsPanel, m_Db );
-	MainPodcastsSizer->Add( m_PodcastsListBox, 1, wxALL|wxEXPAND, 1 );
+	MainPodcastsSizer->Add( m_PodcastsListBox, 1, wxEXPAND, 5 );
 
 	PodcastsPanel->SetSizer( MainPodcastsSizer );
 	PodcastsPanel->Layout();
@@ -252,7 +228,7 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
 	m_DetailScrolledWindow->SetSizer( m_DetailFlexGridSizer );
 	m_DetailScrolledWindow->Layout();
 	m_DetailFlexGridSizer->Fit( m_DetailScrolledWindow );
-	DetailSizer->Add( m_DetailScrolledWindow, 1, wxEXPAND | wxALL, 5 );
+	DetailSizer->Add( m_DetailScrolledWindow, 1, wxEXPAND|wxALL, 5 );
 
 	m_DetailMainSizer->Add( DetailSizer, 1, wxEXPAND|wxALL, 5 );
 
@@ -265,23 +241,27 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
             CloseButton( Config->ReadBool( wxT( "ShowPaneCloseButton" ), true, wxT( "General" ) ) ).
             Dockable( true ).Bottom() );
 
-
     wxString PodcastLayout = Config->ReadStr( wxT( "Podcasts" ), wxEmptyString, wxT( "Positions" ) );
     if( Config->GetIgnoreLayouts() || PodcastLayout.IsEmpty() )
     {
-        m_AuiManager.Update();
         m_VisiblePanels = guPANEL_PODCASTS_VISIBLE_DEFAULT;
+
+        PodcastLayout = wxT( "layout2|name=PodcastsChannels;caption=" ) + wxString( _( "Channels" ) );
+        PodcastLayout += wxT( ";state=2099196;dir=4;layer=0;row=0;pos=0;prop=100000;bestw=50;besth=50;minw=50;minh=50;maxw=-1;maxh=-1;floatx=-1;floaty=-1;floatw=-1;floath=-1|" );
+        PodcastLayout += wxT( "name=PodcastsItems;caption=Podcasts;state=768;dir=5;layer=0;row=0;pos=0;prop=100000;bestw=50;besth=50;minw=50;minh=50;maxw=-1;maxh=-1;floatx=-1;floaty=-1;floatw=-1;floath=-1|" );
+        PodcastLayout += wxT( "name=PodcastsDetails;caption=" ) + wxString( _( "Details" ) );
+        PodcastLayout += wxT( ";state=2099196;dir=3;layer=0;row=0;pos=0;prop=100000;bestw=100;besth=132;minw=100;minh=100;maxw=-1;maxh=-1;floatx=-1;floaty=-1;floatw=-1;floath=-1|dock_size(4,0,0)=181|" );
+        PodcastLayout += wxT( "dock_size(5,0,0)=52|dock_size(3,0,0)=479|" );
+        //m_AuiManager.Update();
     }
-    else
-    {
-        m_AuiManager.LoadPerspective( PodcastLayout, true );
-    }
+
+    m_AuiManager.LoadPerspective( PodcastLayout, true );
 
     Connect( ID_PODCASTS_CHANNEL_ADD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::AddChannel ), NULL, this );
     Connect( ID_PODCASTS_CHANNEL_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::DeleteChannels ), NULL, this );
     Connect( ID_PODCASTS_CHANNEL_PROPERTIES, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelProperties ), NULL, this );
-    Connect( ID_PODCASTS_CHANNEL_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelsCopyTo ), NULL, this );
     Connect( ID_PODCASTS_CHANNEL_UPDATE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::UpdateChannels ), NULL, this );
+    Connect( ID_PODCASTS_CHANNEL_COPYTO, ID_PODCASTS_CHANNEL_COPYTO + 199, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelsCopyTo ), NULL, this );
 
     Connect( ID_PODCASTS_ITEM_UPDATED, guPodcastEvent, wxCommandEventHandler( guPodcastPanel::OnPodcastItemUpdated ), NULL, this );
 
@@ -292,15 +272,13 @@ guPodcastPanel::guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame
     m_PodcastsListBox->Connect( wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, wxListEventHandler( guPodcastPanel::OnPodcastItemActivated ), NULL, this );
 
     Connect( ID_PODCASTS_ITEM_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemPlay ), NULL, this );
-    Connect( ID_PODCASTS_ITEM_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
-    Connect( ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueueAsNext ), NULL, this );
+    Connect( ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALL, ID_PODCASTS_ITEM_ENQUEUE_AFTER_ARTIST, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
     Connect( ID_PODCASTS_ITEM_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDelete ), NULL, this );
     Connect( ID_PODCASTS_ITEM_DOWNLOAD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDownload ), NULL, this );
-    Connect( ID_PODCASTS_ITEM_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
+    Connect( ID_PODCASTS_ITEM_COPYTO, ID_PODCASTS_ITEM_COPYTO + 199, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
 
     Connect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guPodcastPanel::OnConfigUpdated ), NULL, this );
 
-    m_AuiManager.Connect( wxEVT_AUI_PANE_CLOSE, wxAuiManagerEventHandler( guPodcastPanel::OnPaneClose ), NULL, this );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -319,7 +297,7 @@ guPodcastPanel::~guPodcastPanel()
     Disconnect( ID_PODCASTS_CHANNEL_ADD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::AddChannel ), NULL, this );
     Disconnect( ID_PODCASTS_CHANNEL_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::DeleteChannels ), NULL, this );
     Disconnect( ID_PODCASTS_CHANNEL_PROPERTIES, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelProperties ), NULL, this );
-    Disconnect( ID_PODCASTS_CHANNEL_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelsCopyTo ), NULL, this );
+    Disconnect( ID_PODCASTS_CHANNEL_COPYTO, ID_PODCASTS_CHANNEL_COPYTO + 199, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::ChannelsCopyTo ), NULL, this );
     Disconnect( ID_PODCASTS_CHANNEL_UPDATE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::UpdateChannels ), NULL, this );
 
     Disconnect( ID_PODCASTS_ITEM_UPDATED, guPodcastEvent, wxCommandEventHandler( guPodcastPanel::OnPodcastItemUpdated ), NULL, this );
@@ -331,29 +309,42 @@ guPodcastPanel::~guPodcastPanel()
     m_PodcastsListBox->Disconnect( wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, wxListEventHandler( guPodcastPanel::OnPodcastItemActivated ), NULL, this );
 
     Disconnect( ID_PODCASTS_ITEM_PLAY, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemPlay ), NULL, this );
-    Disconnect( ID_PODCASTS_ITEM_ENQUEUE, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
-    Disconnect( ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueueAsNext ), NULL, this );
+    Disconnect( ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALL, ID_PODCASTS_ITEM_ENQUEUE_AFTER_ARTIST, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemEnqueue ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_DEL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDelete ), NULL, this );
     Disconnect( ID_PODCASTS_ITEM_DOWNLOAD, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemDownload ), NULL, this );
-    Disconnect( ID_PODCASTS_ITEM_COPYTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
+    Disconnect( ID_PODCASTS_ITEM_COPYTO, ID_PODCASTS_ITEM_COPYTO + 199, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guPodcastPanel::OnPodcastItemCopyTo ), NULL, this );
 
     Disconnect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guPodcastPanel::OnConfigUpdated ), NULL, this );
 
-    m_AuiManager.Disconnect( wxEVT_AUI_PANE_CLOSE, wxAuiManagerEventHandler( guPodcastPanel::OnPaneClose ), NULL, this );
+}
 
-    m_AuiManager.UnInit();
+// -------------------------------------------------------------------------------- //
+void guPodcastPanel::InitPanelData( void )
+{
+    m_PanelNames.Add( wxT( "PodcastsChannels" ) );
+    m_PanelNames.Add( wxT( "PodcastsDetails" ) );
+
+    m_PanelIds.Add( guPANEL_PODCASTS_CHANNELS );
+    m_PanelIds.Add( guPANEL_PODCASTS_DETAILS );
+
+    m_PanelCmdIds.Add( ID_MENU_VIEW_POD_CHANNELS );
+    m_PanelCmdIds.Add( ID_MENU_VIEW_POD_DETAILS );
 }
 
 // -------------------------------------------------------------------------------- //
 void guPodcastPanel::OnConfigUpdated( wxCommandEvent &event )
 {
-    guConfig * Config = ( guConfig * ) guConfig::Get();
-
-    // Check that the directory to store podcasts are created
-    m_PodcastsPath = Config->ReadStr( wxT( "Path" ), wxGetHomeDir() + wxT( "/.guayadeque/Podcasts" ), wxT( "Podcasts" ) );
-    if( !wxDirExists( m_PodcastsPath ) )
+    int Flags = event.GetInt();
+    if( Flags & guPREFERENCE_PAGE_FLAG_PODCASTS )
     {
-        wxMkdir( m_PodcastsPath, 0770 );
+        guConfig * Config = ( guConfig * ) guConfig::Get();
+
+        // Check that the directory to store podcasts are created
+        m_PodcastsPath = Config->ReadStr( wxT( "Path" ), wxGetHomeDir() + wxT( "/.guayadeque/Podcasts" ), wxT( "Podcasts" ) );
+        if( !wxDirExists( m_PodcastsPath ) )
+        {
+            wxMkdir( m_PodcastsPath, 0770 );
+        }
     }
 }
 
@@ -420,6 +411,8 @@ void guPodcastPanel::AddChannel( wxCommandEvent &event )
                 //guLogMessage( wxT( "The Channel have DownloadType : %u" ), PodcastChannel.m_DownloadType );
                 m_Db->SavePodcastChannel( &PodcastChannel );
 
+                PodcastChannel.CheckDeleteItems( m_Db );
+
                 PodcastChannel.CheckDownloadItems( m_Db, m_MainFrame );
 
                 m_ChannelsListBox->ReloadItems();
@@ -486,6 +479,8 @@ void guPodcastPanel::ChannelProperties( wxCommandEvent &event )
 
         m_Db->SavePodcastChannel( &PodcastChannel );
 
+        PodcastChannel.CheckDeleteItems( m_Db );
+
         PodcastChannel.CheckDownloadItems( m_Db, m_MainFrame );
 
         m_ChannelsListBox->ReloadItems();
@@ -528,7 +523,17 @@ void guPodcastPanel::ChannelsCopyTo( wxCommandEvent &event )
         }
     }
 
-    event.SetId( ID_MAINFRAME_COPYTO );
+    Index = event.GetId() - ID_PODCASTS_CHANNEL_COPYTO;
+    if( Index > 99 )
+    {
+        Index -= 100;
+        event.SetId( ID_MAINFRAME_COPYTODEVICE_TRACKS );
+    }
+    else
+    {
+        event.SetId( ID_MAINFRAME_COPYTO );
+    }
+    event.SetInt( Index );
     event.SetClientData( ( void * ) Tracks );
     wxPostEvent( m_MainFrame, event );
 }
@@ -713,7 +718,7 @@ void guPodcastPanel::OnPodcastItemUpdated( wxCommandEvent &event )
 }
 
 // -------------------------------------------------------------------------------- //
-void guPodcastPanel::OnSelectPodcasts( bool enqueue, const bool asnext )
+void guPodcastPanel::OnSelectPodcasts( bool enqueue, const int aftercurrent )
 {
     int Index;
     int Count;
@@ -776,7 +781,7 @@ void guPodcastPanel::OnSelectPodcasts( bool enqueue, const bool asnext )
         {
             if( enqueue )
             {
-                m_PlayerPanel->AddToPlayList( Tracks, true, asnext );
+                m_PlayerPanel->AddToPlayList( Tracks, true, aftercurrent );
             }
             else
             {
@@ -802,13 +807,7 @@ void guPodcastPanel::OnPodcastItemPlay( wxCommandEvent &event )
 // -------------------------------------------------------------------------------- //
 void guPodcastPanel::OnPodcastItemEnqueue( wxCommandEvent &event )
 {
-    OnSelectPodcasts( true );
-}
-
-// -------------------------------------------------------------------------------- //
-void guPodcastPanel::OnPodcastItemEnqueueAsNext( wxCommandEvent &event )
-{
-    OnSelectPodcasts( true, true );
+    OnSelectPodcasts( true, event.GetId() - ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALL );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -859,82 +858,22 @@ void guPodcastPanel::OnPodcastItemCopyTo( wxCommandEvent &event )
     guTrackArray * Tracks = new guTrackArray();
     m_PodcastsListBox->GetSelectedSongs( Tracks );
 
-    event.SetId( ID_MAINFRAME_COPYTO );
+
+    int Index = event.GetId() - ID_PODCASTS_ITEM_COPYTO;
+    if( Index > 99 )
+    {
+        Index -= 100;
+        event.SetId( ID_MAINFRAME_COPYTODEVICE_TRACKS );
+    }
+    else
+    {
+        event.SetId( ID_MAINFRAME_COPYTO );
+    }
+    event.SetInt( Index );
     event.SetClientData( ( void * ) Tracks );
     wxPostEvent( wxTheApp->GetTopWindow(), event );
 }
 
-// -------------------------------------------------------------------------------- //
-bool guPodcastPanel::IsPanelShown( const int panelid ) const
-{
-    return ( m_VisiblePanels & panelid );
-}
-
-// -------------------------------------------------------------------------------- //
-void guPodcastPanel::ShowPanel( const int panelid, bool show )
-{
-    wxString PaneName;
-
-    switch( panelid )
-    {
-        case guPANEL_PODCASTS_CHANNELS :
-            PaneName = wxT( "PodcastsChannels" );
-            break;
-
-        case guPANEL_PODCASTS_DETAILS :
-            PaneName = wxT( "PodcastsDetails" );
-            break;
-
-        default :
-            return;
-
-    }
-
-    wxAuiPaneInfo &PaneInfo = m_AuiManager.GetPane( PaneName );
-    if( PaneInfo.IsOk() )
-    {
-        if( show )
-            PaneInfo.Show();
-        else
-            PaneInfo.Hide();
-
-        m_AuiManager.Update();
-    }
-
-    if( show )
-        m_VisiblePanels |= panelid;
-    else
-        m_VisiblePanels ^= panelid;
-
-    guLogMessage( wxT( "Id: %i Pane: %s Show:%i  Flags:%08X" ), panelid, PaneName.c_str(), show, m_VisiblePanels );
-}
-
-// -------------------------------------------------------------------------------- //
-void guPodcastPanel::OnPaneClose( wxAuiManagerEvent &event )
-{
-    wxAuiPaneInfo * PaneInfo = event.GetPane();
-    wxString PaneName = PaneInfo->name;
-    int CmdId = 0;
-
-    if( PaneName == wxT( "PodcastsChannels" ) )
-    {
-        CmdId = ID_MENU_VIEW_POD_CHANNELS;
-    }
-    else if( PaneName == wxT( "PodcastsDetails" ) )
-    {
-        CmdId = ID_MENU_VIEW_POD_DETAILS;
-    }
-
-    guLogMessage( wxT( "OnPaneClose: %s  %i" ), PaneName.c_str(), CmdId );
-    if( CmdId )
-    {
-        wxCommandEvent evt( wxEVT_COMMAND_MENU_SELECTED, CmdId );
-        AddPendingEvent( evt );
-    }
-
-    event.Veto();
-}
-
 // -------------------------------------------------------------------------------- //
 void guPodcastPanel::SelectPodcast( const int podcastid )
 {
@@ -949,6 +888,8 @@ void guPodcastPanel::SelectChannel( const int channelid )
     m_ChannelsListBox->SetSelection( m_ChannelsListBox->FindItem( channelid ) );
 }
 
+
+
 // -------------------------------------------------------------------------------- //
 // guChannelsListBox
 // -------------------------------------------------------------------------------- //
@@ -995,35 +936,32 @@ void guChannelsListBox::CreateContextMenu( wxMenu * Menu ) const
     int SelCount = GetSelectedCount();
 
     MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_ADD, _( "New Channel" ), _( "Add a new podcast channel" ) );
-    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
     Menu->Append( MenuItem );
 
     if( SelCount )
     {
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_DEL, _( "Delete" ), _( "delete this podcast channels and all its items" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_del ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
         Menu->Append( MenuItem );
 
-//        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_UNDELETE, _( "Undelete" ), _( "Show all deleted podcasts of the selected channels" ) );
-//        //MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
-//        Menu->Append( MenuItem );
-
         Menu->AppendSeparator();
+
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_UPDATE, _( "Update" ), _( "Update the podcast items of the selected channels" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_doc_save ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_doc_save ) );
         Menu->Append( MenuItem );
 
         if( SelCount == 1 )
         {
             MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_PROPERTIES, _( "Properties" ), _( "Edit the podcast channel" ) );
-            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
             Menu->Append( MenuItem );
         }
 
         Menu->AppendSeparator();
-        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_CHANNEL_COPYTO, _( "Copy to..." ), _( "Copy the current selected podcasts to a directory or device" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
-        Menu->Append( MenuItem );
+
+        guMainFrame * MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
+        MainFrame->CreateCopyToMenu( Menu, ID_PODCASTS_CHANNEL_COPYTO );
     }
 }
 
@@ -1051,6 +989,7 @@ guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
     m_Db = db;
 
     guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->RegisterObject( this );
 
     m_ColumnNames.Add( _( "Status" ) );
     m_ColumnNames.Add( _( "Title" ) );
@@ -1095,6 +1034,10 @@ guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
         InsertColumn( Column );
     }
 
+    Connect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guPodcastListBox::OnConfigUpdated ), NULL, this );
+
+    CreateAcceleratorTable();
+
     ReloadItems();
 }
 
@@ -1103,6 +1046,8 @@ guPodcastListBox::guPodcastListBox( wxWindow * parent, guDbLibrary * db ) :
 guPodcastListBox::~guPodcastListBox()
 {
     guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->UnRegisterObject( this );
+
     int index;
     int count = m_ColumnNames.Count();
     for( index = 0; index < count; index++ )
@@ -1123,6 +1068,43 @@ guPodcastListBox::~guPodcastListBox()
     {
         delete m_Images[ index ];
     }
+
+    Disconnect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guPodcastListBox::OnConfigUpdated ), NULL, this );
+}
+
+// -------------------------------------------------------------------------------- //
+void guPodcastListBox::OnConfigUpdated( wxCommandEvent &event )
+{
+    int Flags = event.GetInt();
+    if( Flags & guPREFERENCE_PAGE_FLAG_ACCELERATORS )
+    {
+        CreateAcceleratorTable();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guPodcastListBox::CreateAcceleratorTable( void )
+{
+    wxAcceleratorTable AccelTable;
+    wxArrayInt AliasAccelCmds;
+    wxArrayInt RealAccelCmds;
+
+    AliasAccelCmds.Add( ID_SONG_PLAY );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+
+    RealAccelCmds.Add( ID_PODCASTS_ITEM_PLAY );
+    RealAccelCmds.Add( ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALL );
+    RealAccelCmds.Add( ID_PODCASTS_ITEM_ENQUEUE_AFTER_TRACK );
+    RealAccelCmds.Add( ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALBUM );
+    RealAccelCmds.Add( ID_PODCASTS_ITEM_ENQUEUE_AFTER_ARTIST );
+
+    if( guAccelDoAcceleratorTable( AliasAccelCmds, RealAccelCmds, AccelTable ) )
+    {
+        SetAcceleratorTable( AccelTable );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -1210,8 +1192,6 @@ void guPodcastListBox::GetItemsList( void )
 // -------------------------------------------------------------------------------- //
 void guPodcastListBox::ReloadItems( bool reset )
 {
-    wxASSERT( m_Db );
-
     //
     wxArrayInt Selection;
     int FirstVisible = 0;
@@ -1255,26 +1235,48 @@ void guPodcastListBox::OnKeyDown( wxKeyEvent &event )
 // -------------------------------------------------------------------------------- //
 void guPodcastListBox::CreateContextMenu( wxMenu * Menu ) const
 {
-    wxArrayInt Selection = GetSelectedItems();
-    if( Selection.Count() )
+    int SelCount = GetSelectedCount();
+    if( SelCount )
     {
         wxMenuItem * MenuItem;
-        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_PLAY, _( "Play" ), _( "Play current selected songs" ) );
+        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_PLAY,
+                                wxString( _( "Play" ) ) + guAccelGetCommandKeyCodeString( ID_SONG_PLAY ),
+                                _( "Play current selected songs" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
         Menu->Append( MenuItem );
 
-        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_ENQUEUE, _( "Enqueue" ), _( "Add current selected songs to playlist" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
+        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALL,
+                                wxString( _( "Enqueue" ) ) + guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALL ),
+                                _( "Add current selected songs to playlist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
         Menu->Append( MenuItem );
 
-        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected songs to playlist as Next Tracks" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_add ) );
-        Menu->Append( MenuItem );
+        wxMenu * EnqueueMenu = new wxMenu();
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_PODCASTS_ITEM_ENQUEUE_AFTER_TRACK,
+                                wxString( _( "Current Track" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_TRACK ),
+                                _( "Add current selected tracks to playlist after the current track" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_PODCASTS_ITEM_ENQUEUE_AFTER_ALBUM,
+                                wxString( _( "Current Album" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALBUM ),
+                                _( "Add current selected tracks to playlist after the current album" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        MenuItem = new wxMenuItem( EnqueueMenu, ID_PODCASTS_ITEM_ENQUEUE_AFTER_ARTIST,
+                                wxString( _( "Current Artist" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ARTIST ),
+                                _( "Add current selected tracks to playlist after the current artist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+        EnqueueMenu->Append( MenuItem );
+
+        Menu->Append( wxID_ANY, _( "Enqueue after" ), EnqueueMenu );
 
         Menu->AppendSeparator();
 
         MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_DEL, _( "Delete" ), _( "Delete the current selected podcasts" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_del ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
         Menu->Append( MenuItem );
 
         Menu->AppendSeparator();
@@ -1284,9 +1286,9 @@ void guPodcastListBox::CreateContextMenu( wxMenu * Menu ) const
         Menu->Append( MenuItem );
 
         Menu->AppendSeparator();
-        MenuItem = new wxMenuItem( Menu, ID_PODCASTS_ITEM_COPYTO, _( "Copy to..." ), _( "Copy the current selected podcasts to a directory or device" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
-        Menu->Append( MenuItem );
+
+        guMainFrame * MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
+        MainFrame->CreateCopyToMenu( Menu, ID_PODCASTS_ITEM_COPYTO );
     }
     else
     {
