@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 #ifndef PODCASTSPANEL_H
 #define PODCASTSPANEL_H
 
+#include "AuiManagedPanel.h"
 #include "DbLibrary.h"
 #include "PlayerPanel.h"
 
@@ -48,6 +49,8 @@
 #define guPODCASTS_COLUMN_LASTPLAY      8
 #define guPODCASTS_COLUMN_ADDEDDATE     9
 
+#define guPODCASTS_COLUMN_COUNT         10
+
 // -------------------------------------------------------------------------------- //
 class guChannelsListBox : public guListBox
 {
@@ -77,13 +80,13 @@ class guPodcastPanel;
 class guPodcastListBox : public guListView
 {
   protected :
-    guDbLibrary *       m_Db;
-    guPodcastItemArray  m_PodItems;
-    wxArrayInt          m_PodChFilters;
-    int                 m_Order;
-    bool                m_OrderDesc;
-    wxImage *           m_Images[ guPODCAST_STATUS_ERROR + 1 ];
-    wxArrayString       m_ColumnNames;
+    guDbLibrary *               m_Db;
+    guPodcastItemArray          m_PodItems;
+    wxArrayInt                  m_PodChFilters;
+    int                         m_Order;
+    bool                        m_OrderDesc;
+    wxImage *                   m_Images[ guPODCAST_STATUS_ERROR + 1 ];
+    wxArrayString               m_ColumnNames;
 
     virtual void                DrawItem( wxDC &dc, const wxRect &rect, const int row, const int col ) const;
     virtual void                CreateContextMenu( wxMenu * Menu ) const;
@@ -94,6 +97,9 @@ class guPodcastListBox : public guListView
     virtual int                 GetDragFiles( wxFileDataObject * files );
     virtual void                OnKeyDown( wxKeyEvent &event );
 
+    void                        OnConfigUpdated( wxCommandEvent &event );
+    void                        CreateAcceleratorTable();
+
   public :
     guPodcastListBox( wxWindow * parent, guDbLibrary * NewDb );
     ~guPodcastListBox();
@@ -117,7 +123,7 @@ class guPodcastListBox : public guListView
 class guMainFrame;
 
 // -------------------------------------------------------------------------------- //
-class guPodcastPanel : public wxPanel
+class guPodcastPanel : public guAuiManagedPanel
 {
   private:
     void                MainSplitterOnIdle( wxIdleEvent& );
@@ -132,16 +138,12 @@ class guPodcastPanel : public wxPanel
 
     void                OnPodcastItemPlay( wxCommandEvent &event );
     void                OnPodcastItemEnqueue( wxCommandEvent &event );
-    void                OnPodcastItemEnqueueAsNext( wxCommandEvent &event );
     void                OnPodcastItemDelete( wxCommandEvent &event );
     void                OnPodcastItemDownload( wxCommandEvent &event );
     void                OnPodcastItemCopyTo( wxCommandEvent &event );
 
 
   protected:
-    wxAuiManager                m_AuiManager;
-    unsigned int                m_VisiblePanels;
-
     guDbLibrary *               m_Db;
     guMainFrame *               m_MainFrame;
     guPlayerPanel *             m_PlayerPanel;
@@ -149,8 +151,6 @@ class guPodcastPanel : public wxPanel
     int                         m_LastChannelInfoId;
     int                         m_LastPodcastInfoId;
 
-//    wxSplitterWindow *          m_MainSplitter;
-//    wxSplitterWindow *          m_TopSplitter;
 	guChannelsListBox *         m_ChannelsListBox;
     guPodcastListBox *          m_PodcastsListBox;
 	wxBoxSizer *                m_DetailMainSizer;
@@ -177,27 +177,28 @@ class guPodcastPanel : public wxPanel
     void UpdateChannels( wxCommandEvent &event );
 
     void ClearDownloadThread( void );
-    void OnSelectPodcasts( bool enqueue = false, const bool asnext = false );
+    void OnSelectPodcasts( bool enqueue = false, const int aftercurrent = 0 );
 
     void OnConfigUpdated( wxCommandEvent &event );
 
-    void OnPaneClose( wxAuiManagerEvent &event );
-
 public:
     guPodcastPanel( wxWindow * parent, guDbLibrary * db, guMainFrame * mainframe, guPlayerPanel * playerpanel );
     ~guPodcastPanel();
 
-    bool IsPanelShown( const int panelid ) const;
-    void ShowPanel( const int panelid, bool show );
-
-    void SelectPodcast( const int podcastid );
-    void SelectChannel( const int channelid );
+    virtual void                InitPanelData( void );
+    void                        SelectPodcast( const int podcastid );
+    void                        SelectChannel( const int channelid );
 
     void GetCounters( wxLongLong * count, wxLongLong * len, wxLongLong * size )
     {
         m_Db->GetPodcastCounters( m_PodcastsListBox->GetFilters(), count, len, size );
     }
 
+    virtual int                 GetListViewColumnCount( void ) { return guPODCASTS_COLUMN_COUNT; }
+    virtual bool                GetListViewColumnData( const int id, int * index, int * width, bool * enabled ) { return m_PodcastsListBox->GetColumnData( id, index, width, enabled ); }
+    virtual bool                SetListViewColumnData( const int id, const int index, const int width, const bool enabled, const bool refresh = false ) { return m_PodcastsListBox->SetColumnData( id, index, width, enabled, refresh ); }
+
+
     friend class guPodcastDownloadQueueThread;
 };
 
