@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -87,36 +87,58 @@ guRadioGenreEditor::guRadioGenreEditor( wxWindow * parent, guDbRadios * db ) :
 }
 
 // -------------------------------------------------------------------------------- //
-wxArrayString guRadioGenreEditor::GetGenres( void )
+int guListItemsFind( guListItems &items, const wxString &name )
+{
+    int Index;
+    int Count = items.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( items[ Index ].m_Name == name )
+            return Index;
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
+int guListItemsFind( guListItems &items, const int &id )
+{
+    int Index;
+    int Count = items.Count();
+    for( Index = 0; Index < Count; Index++ )
+    {
+        if( items[ Index ].m_Id == id )
+            return Index;
+    }
+    return wxNOT_FOUND;
+}
+
+// -------------------------------------------------------------------------------- //
+void guRadioGenreEditor::GetGenres( wxArrayString &addedgenres, wxArrayInt &deletedgenres )
 {
-    wxArrayString RetVal;
     int index;
     int count = m_CheckListBox->GetCount();
     for( index = 0; index < count; index++ )
     {
         if( m_CheckListBox->IsChecked( index ) )
         {
-            RetVal.Add( m_RadioGenres[ index ] );
+            if( guListItemsFind( m_AddedGenres, m_RadioGenres[ index ] ) == wxNOT_FOUND )
+                addedgenres.Add( m_RadioGenres[ index ] );
+        }
+        else
+        {
+            int Pos;
+            if( ( Pos = guListItemsFind( m_AddedGenres, m_RadioGenres[ index ] ) ) != wxNOT_FOUND )
+            {
+                deletedgenres.Add( m_AddedGenres[ Pos ].m_Id );
+            }
         }
-    }
-    if( !m_InputTextCtrl->IsEmpty() )
-    {
-        RetVal.Add( m_InputTextCtrl->GetValue() );
     }
 
-    // we add all and then remove the existans as guListBox dont have the Index function
-    // so we check the other direction
-    count = m_AddedGenres.Count();
-    int item;
-    for( index = 0; index < count; index++ )
+    if( !m_InputTextCtrl->IsEmpty() )
     {
-        item = RetVal.Index( m_AddedGenres[ index ].m_Name );
-        if( item != wxNOT_FOUND )
-        {
-            RetVal.RemoveAt( item );
-        }
+        if( guListItemsFind( m_AddedGenres, m_InputTextCtrl->GetValue() ) == wxNOT_FOUND )
+            addedgenres.Add( m_InputTextCtrl->GetValue() );
     }
-    return RetVal;
 }
 
 // -------------------------------------------------------------------------------- //
