@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -21,12 +21,13 @@
 #include "SelCoverFile.h"
 
 #include "Config.h"
+#include "Commands.h"
 
 #include <wx/filedlg.h>
 
 // -------------------------------------------------------------------------------- //
 guSelCoverFile::guSelCoverFile( wxWindow * parent, guDbLibrary * db, const int albumid ) :
-    wxDialog( parent, wxID_ANY, _( "Select Cover File" ), wxDefaultPosition, wxSize( 400, 125 ), wxDEFAULT_DIALOG_STYLE )
+    wxDialog( parent, wxID_ANY, _( "Select Cover File" ), wxDefaultPosition, wxSize( 400, 132 ), wxDEFAULT_DIALOG_STYLE )
 {
     m_Db = db;
     if( albumid != wxNOT_FOUND )
@@ -40,6 +41,8 @@ guSelCoverFile::guSelCoverFile( wxWindow * parent, guDbLibrary * db, const int a
         }
     }
 
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+
     // GUI
 	SetSizeHints( wxDefaultSize, wxDefaultSize );
 
@@ -61,9 +64,14 @@ guSelCoverFile::guSelCoverFile( wxWindow * parent, guDbLibrary * db, const int a
 
 	MainSizer->Add( ControlsSizer, 1, wxEXPAND, 5 );
 
+	m_EmbedToFilesChkBox = new wxCheckBox( this, wxID_ANY, _( "Embed into tracks" ), wxDefaultPosition, wxDefaultSize, 0 );
+    m_EmbedToFilesChkBox->SetValue( Config->ReadBool( wxT( "EmbedToFiles" ), false, wxT( "General" ) ) );
+	MainSizer->Add( m_EmbedToFilesChkBox, 0, wxRIGHT|wxLEFT, 5 );
+
 	wxStdDialogButtonSizer * StdBtnSizer = new wxStdDialogButtonSizer();
-	wxButton * StdBtnSizerOK = new wxButton( this, wxID_OK );
-	StdBtnSizer->AddButton( StdBtnSizerOK );
+	m_StdBtnOk = new wxButton( this, wxID_OK );
+	m_StdBtnOk->Enable( false );
+	StdBtnSizer->AddButton( m_StdBtnOk );
 	wxButton * StdBtnSizerCancel = new wxButton( this, wxID_CANCEL );
 	StdBtnSizer->AddButton( StdBtnSizerCancel );
 	StdBtnSizer->Realize();
@@ -73,12 +81,21 @@ guSelCoverFile::guSelCoverFile( wxWindow * parent, guDbLibrary * db, const int a
 	this->Layout();
 
 	m_SelFileBtn->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guSelCoverFile::OnSelFileClicked ), NULL, this );
+	m_FileLink->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( guSelCoverFile::OnPathChanged ), NULL, this );
+
+	Connect( ID_SELCOVERDIALOG_FINISH, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSelCoverFile::OnCoverFinish ), NULL, this );
 }
 
 // -------------------------------------------------------------------------------- //
 guSelCoverFile::~guSelCoverFile()
 {
+    guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->WriteBool( wxT( "EmbedToFiles" ), m_EmbedToFilesChkBox->GetValue(), wxT( "General" ) );
+
 	m_SelFileBtn->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( guSelCoverFile::OnSelFileClicked ), NULL, this );
+	m_FileLink->Disconnect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( guSelCoverFile::OnPathChanged ), NULL, this );
+
+	Disconnect( ID_SELCOVERDIALOG_FINISH, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSelCoverFile::OnCoverFinish ), NULL, this );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -97,11 +114,20 @@ void guSelCoverFile::OnSelFileClicked( wxCommandEvent& event )
         if( FileDialog->ShowModal() == wxID_OK )
         {
             m_FileLink->SetValue( FileDialog->GetPath() );
+
+            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_SELCOVERDIALOG_FINISH );
+            AddPendingEvent( event );
         }
         FileDialog->Destroy();
     }
 }
 
+// -------------------------------------------------------------------------------- //
+void guSelCoverFile::OnCoverFinish( wxCommandEvent& event )
+{
+    EndModal( wxID_OK );
+}
+
 // -------------------------------------------------------------------------------- //
 wxString guSelCoverFile::GetSelFile( void )
 {
@@ -109,3 +135,9 @@ wxString guSelCoverFile::GetSelFile( void )
 }
 
 // -------------------------------------------------------------------------------- //
+void guSelCoverFile::OnPathChanged( wxCommandEvent &event )
+{
+	m_StdBtnOk->Enable( !m_FileLink->IsEmpty() );
+}
+
+// -------------------------------------------------------------------------------- //
