@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -83,9 +83,18 @@
 #include <wx/fileconf.h>
 
 
-#define SHOUTCAST_GET_GENRE_URL         wxT( "http://yp.shoutcast.com/sbin/newxml.phtml" )
-#define SHOUTCAST_GET_STATIONS_URL      wxT( "http://yp.shoutcast.com/sbin/newxml.phtml?genre=%s" )
-#define SHOUTCAST_SEARCH_STATIONS_URL   wxT( "http://yp.shoutcast.com/sbin/newxml.phtml?search=%s" )
+//#define SHOUTCAST_GET_GENRE_URL         wxT( "http://yp.shoutcast.com/sbin/newxml.phtml" )
+//s o 1 N 1 5 v h C B 7 8 Z 6 k 4 __ s h 1 i q r P H n h j F m X i T __ f a 1 _ B m i c Y Q P D b b K m
+//a m 1 S G I 9 p 6 W l 8 Y b b W __ f a 1 6 6 9 M u i R P o r U B w __ s h 1 7 t O U i _ e z L 6 1 _ A
+#define SHOUTCAST_DEV_KEY0   "sh1iqrPH"
+#define SHOUTCAST_DEV_KEY1   "nhjFmXiT"
+#define SHOUTCAST_DEV_KEY    SHOUTCAST_DEV_KEY0 SHOUTCAST_DEV_KEY1
+#define SHOUTCAST_GET_GENRE_URL         wxT( "http://api.shoutcast.com/legacy/genrelist?k=" SHOUTCAST_DEV_KEY )
+//#define SHOUTCAST_GET_STATIONS_URL      wxT( "http://yp.shoutcast.com/sbin/newxml.phtml?genre=%s" )
+#define SHOUTCAST_GET_STATIONS_URL      wxT( "http://api.shoutcast.com/legacy/genresearch?k=" SHOUTCAST_DEV_KEY "&genre=%s" )
+//#define SHOUTCAST_SEARCH_STATIONS_URL   wxT( "http://yp.shoutcast.com/sbin/newxml.phtml?search=%s" )
+#define SHOUTCAST_SEARCH_STATIONS_URL   wxT( "http://api.shoutcast.com/legacy/stationsearch?k=" SHOUTCAST_DEV_KEY "&search=%s" )
+//#define SHOUTCAST_GET_STATION_PLAYLIST  wxT( "http://yp.shoutcast.com/sbin/tunein-station.pls?id=%u" )
 #define SHOUTCAST_GET_STATION_PLAYLIST  wxT( "http://yp.shoutcast.com/sbin/tunein-station.pls?id=%u" )
 
 
@@ -138,7 +147,8 @@ void guShoutCast::GetStations( const int source, const int flags, const wxString
     //            SHOUTCAST_GET_STATIONS_URL : SHOUTCAST_SEARCH_STATIONS_URL, guURLEncode( GenreName ).c_str() ).c_str() );
 
     Content = GetUrlContent( wxString::Format( source == guRADIO_SOURCE_GENRE ?
-                SHOUTCAST_GET_STATIONS_URL : SHOUTCAST_SEARCH_STATIONS_URL, guURLEncode( GenreName ).c_str() ) );
+                SHOUTCAST_GET_STATIONS_URL : SHOUTCAST_SEARCH_STATIONS_URL, guURLEncode( GenreName ).c_str() ),
+                            wxEmptyString, true );
     //
     if( Content.Length() )
     {
@@ -159,6 +169,7 @@ void guShoutCast::GetStations( const int source, const int flags, const wxString
                     {
                         XmlNode->GetPropVal( wxT( "name" ), &StationName );
                         StationName.Replace( wxT( " - [SHOUTcast.com]" ), wxT( "" ) );
+                        StationName.Replace( wxT( " - a SHOUTcast.com member station" ), wxT( "" ) );
                         XmlNode->GetPropVal( wxT( "mt" ), &StationType );
                         XmlNode->GetPropVal( wxT( "genre" ), &StationGenre );
                         XmlNode->GetPropVal( wxT( "ct" ), &StationCurrent );
@@ -209,59 +220,7 @@ wxString guShoutCast::GetStationUrl( const int id ) const
 // -------------------------------------------------------------------------------- //
 guStationPlayList guShoutCast::GetStationPlayList( const int StationId ) const
 {
-    wxString                Content;
-    guStationPlayList       RetVal;
-    guStationPlayListItem * NewStation;
-    wxFileConfig *          Config;
-    Content = GetUrlContent( wxString::Format( SHOUTCAST_GET_STATION_PLAYLIST, StationId ) );
-    if( Content.Length() )
-    {
-        //guLogMessage( Content );
-        wxStringInputStream ConfigData( Content );
-        Config = new wxFileConfig( ConfigData );
-        if( Config )
-        {
-            if( Config->HasGroup( wxT( "playlist" ) ) )
-            {
-                Config->SetPath( wxT( "playlist" ) );
-                int count;
-                if( Config->Read( wxT( "numberofentries" ), &count ) )
-                {
-                    if( !count )
-                    {
-                        guLogMessage( wxT( "This station playlist is empty" ) );
-                    }
-                    else
-                    {
-                        for( int index = 1; index <= count; index++ )
-                        {
-                            NewStation = new guStationPlayListItem();
-
-                            wxASSERT( NewStation );
-
-                            Config->Read( wxString::Format( wxT( "File%u" ), index ), &NewStation->m_Location );
-                            Config->Read( wxString::Format( wxT( "Title%u" ), index ), &NewStation->m_Name );
-                            if( NewStation->m_Name.IsEmpty() )
-                                NewStation->m_Name = NewStation->m_Location;
-                            RetVal.Add( NewStation );
-                        }
-                    }
-                }
-            }
-            else
-            {
-                guLogError( wxT( "ee: Station Playlist without 'playlist' group" ) );
-            }
-            delete Config;
-        }
-        else
-          guLogError( wxT( "ee: Station Playlist empty" ) );
-    }
-    else
-    {
-        guLogError( wxT( "Retrieving station playlist empty response" ) );
-    }
-    return RetVal;
+    return GetStationPlayList( wxString::Format( SHOUTCAST_GET_STATION_PLAYLIST, StationId ) );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -295,8 +254,6 @@ guStationPlayList guShoutCast::GetStationPlayList( const wxString &stationurl )
                         {
                             NewStation = new guStationPlayListItem();
 
-                            wxASSERT( NewStation );
-
                             Config->Read( wxString::Format( wxT( "File%u" ), index ), &NewStation->m_Location );
                             Config->Read( wxString::Format( wxT( "Title%u" ), index ), &NewStation->m_Name );
                             if( NewStation->m_Name.IsEmpty() )
