@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@
 // -------------------------------------------------------------------------------- //
 #include "SoListBox.h"
 
+#include "Accelerators.h"
 #include "Config.h" // Configuration
 #include "Commands.h"
 #include "Images.h"
@@ -33,12 +34,14 @@
 #include <wx/imaglist.h>
 
 // -------------------------------------------------------------------------------- //
-guSoListBox::guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confname, long style ) :
+guSoListBox::guSoListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * NewDb, wxString confname, long style ) :
     guListView( parent, style|wxLB_MULTIPLE|guLISTVIEW_ALLOWDRAG|guLISTVIEW_COLUMN_CLICK_EVENTS,
-        wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHSCROLL|wxVSCROLL|wxSUNKEN_BORDER )
+        wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHSCROLL|wxVSCROLL )
 {
     guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->RegisterObject( this );
 
+    m_LibPanel = libpanel;
     m_Db = NewDb;
     m_ConfName = confname;
     m_ItemsFirst = wxNOT_FOUND;
@@ -80,9 +83,9 @@ guSoListBox::guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confn
     Connect( guEVT_LISTBOX_ITEM_COL_CLICKED, wxListEventHandler( guSoListBox::OnItemColumnClicked ), NULL, this );
     Connect( guEVT_LISTBOX_ITEM_COL_RCLICKED, wxListEventHandler( guSoListBox::OnItemColumnRClicked ), NULL, this );
 
-//    Connect( ID_SONG_EDIT_COLUMN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumEdit ), NULL, this );
-//    Connect( ID_SONG_SET_COLUMN, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumSet ), NULL, this );
-//    Connect( ID_SONG_SET_RATING_0, ID_SONG_SET_RATING_5, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnColumnSetRating ), NULL, this );
+    Connect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guSoListBox::OnConfigUpdated ), NULL, this );
+
+    CreateAcceleratorTable();
 
     ReloadItems();
 }
@@ -92,6 +95,8 @@ guSoListBox::guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confn
 guSoListBox::~guSoListBox()
 {
     guConfig * Config = ( guConfig * ) guConfig::Get();
+    Config->UnRegisterObject( this );
+
     //int ColId;
     int index;
     int count = guSONGS_COLUMN_COUNT;
@@ -117,6 +122,49 @@ guSoListBox::~guSoListBox()
     Disconnect( ID_SONGS_COMMANDS, ID_SONGS_COMMANDS + 99, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( guSoListBox::OnCommandClicked ) );
     Disconnect( guEVT_LISTBOX_ITEM_COL_CLICKED, wxListEventHandler( guSoListBox::OnItemColumnClicked ), NULL, this );
     Disconnect( guEVT_LISTBOX_ITEM_COL_RCLICKED, wxListEventHandler( guSoListBox::OnItemColumnRClicked ), NULL, this );
+
+    Disconnect( ID_CONFIG_UPDATED, guConfigUpdatedEvent, wxCommandEventHandler( guSoListBox::OnConfigUpdated ), NULL, this );
+}
+
+// -------------------------------------------------------------------------------- //
+void guSoListBox::OnConfigUpdated( wxCommandEvent &event )
+{
+    int Flags = event.GetInt();
+    if( Flags & guPREFERENCE_PAGE_FLAG_ACCELERATORS )
+    {
+        CreateAcceleratorTable();
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guSoListBox::CreateAcceleratorTable( void )
+{
+    wxAcceleratorTable AccelTable;
+    wxArrayInt AliasAccelCmds;
+    wxArrayInt RealAccelCmds;
+
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SAVE );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITLABELS );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_EDITTRACKS );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    AliasAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    AliasAccelCmds.Add( ID_PLAYER_PLAYLIST_SEARCH );
+
+    RealAccelCmds.Add( ID_SONG_SAVETOPLAYLIST );
+    RealAccelCmds.Add( ID_SONG_EDITLABELS );
+    RealAccelCmds.Add( ID_SONG_EDITTRACKS );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALL );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_TRACK );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ALBUM );
+    RealAccelCmds.Add( ID_SONG_ENQUEUE_AFTER_ARTIST );
+    RealAccelCmds.Add( ID_LIBRARY_SEARCH );
+
+    if( guAccelDoAcceleratorTable( AliasAccelCmds, RealAccelCmds, AccelTable ) )
+    {
+        SetAcceleratorTable( AccelTable );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -201,6 +249,11 @@ wxString guSoListBox::OnGetItemText( const int row, const int col ) const
         {
             return Song->m_Format;
         }
+
+        case guSONGS_COLUMN_FILEPATH :
+        {
+            return Song->m_FileName; // contains both path and filename
+        }
     }
     return wxEmptyString;
 }
@@ -254,6 +307,7 @@ void guSoListBox::GetItemsList( void )
 // -------------------------------------------------------------------------------- //
 void guSoListBox::ReloadItems( bool reset )
 {
+    //guLogMessage( wxT( "guSoListBox::ReloadItems( %i )"), reset );
     //
     wxArrayInt Selection;
     int FirstVisible = GetFirstVisibleLine();
@@ -303,6 +357,8 @@ int guSoListBox::GetSelectedSongs( guTrackArray * tracks )
         item = GetNextSelected( cookie );
     }
     m_ItemsMutex.Unlock();
+    if( m_LibPanel )
+        m_LibPanel->NormalizeTracks( tracks );
     return tracks->Count();
 }
 
@@ -311,7 +367,7 @@ void guSoListBox::GetAllSongs( guTrackArray * tracks )
 {
     int index;
     m_ItemsMutex.Lock();
-    int count = m_Items.Count();
+    int count = GetItemCount();
     for( index = 0; index < count; index++ )
     {
         guTrack * Track = new guTrack();
@@ -333,6 +389,8 @@ int guSoListBox::GetDragFiles( wxFileDataObject * files )
     guTrackArray Songs;
     int index;
     int count = GetSelectedSongs( &Songs );
+    if( m_LibPanel )
+        m_LibPanel->NormalizeTracks( &Songs, true );
     for( index = 0; index < count; index++ )
     {
        wxString FileName = guFileDnDEncode( Songs[ index ].m_FileName );
@@ -364,6 +422,7 @@ wxArrayString guSoListBox::GetColumnNames( void ) const
     ColumnNames.Add( _( "Last Play" ) );
     ColumnNames.Add( _( "Added Date" ) );
     ColumnNames.Add( _( "Format" ) );
+    ColumnNames.Add( _( "Filepath" ) );
     return ColumnNames;
 }
 
@@ -377,7 +436,6 @@ void AddSongsCommands( wxMenu * Menu, int SelCount )
     if( Menu )
     {
         SubMenu = new wxMenu();
-        wxASSERT( SubMenu );
 
         guConfig * Config = ( guConfig * ) guConfig::Get();
         wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
@@ -410,16 +468,22 @@ void guSoListBox::AppendFastEditMenu( wxMenu * menu, const int selcount ) const
 {
     wxMenuItem * MenuItem;
     //guLogMessage( wxT( "guSoLisBox::AppendFastEditMenu %i - %i" ), m_LastColumnRightClicked, m_LastRowRightClicked );
+    if( m_LastColumnRightClicked == wxNOT_FOUND || ( m_LastColumnRightClicked >= ( int ) m_Columns->Count() ) )
+    {
+        return;
+    }
 
     int ColumnId = GetColumnId( m_LastColumnRightClicked );
 
     // If its a column not editable
-    if( ColumnId == guSONGS_COLUMN_LENGTH ||
+    if( ColumnId == guSONGS_COLUMN_RATING ||
+        ColumnId == guSONGS_COLUMN_LENGTH ||
         ColumnId == guSONGS_COLUMN_BITRATE ||
         ColumnId == guSONGS_COLUMN_PLAYCOUNT ||
         ColumnId == guSONGS_COLUMN_LASTPLAY ||
         ColumnId == guSONGS_COLUMN_ADDEDDATE ||
-        ColumnId == guSONGS_COLUMN_FORMAT )
+        ColumnId == guSONGS_COLUMN_FORMAT ||
+        ColumnId == guSONGS_COLUMN_FILEPATH )
         return;
 
     wxArrayString ColumnNames = GetColumnNames();
@@ -433,121 +497,148 @@ void guSoListBox::AppendFastEditMenu( wxMenu * menu, const int selcount ) const
 
     if( selcount > 1 )
     {
-        if( ColumnId == guSONGS_COLUMN_RATING )
-        {
-            wxMenu * RatingMenu = new wxMenu();
+        MenuText = _( "Set" );
+        MenuText += wxT( " " ) + ColumnNames[ ColumnId ] + wxT( " " );
+        MenuText += _( "to" );
+        wxString ItemText = OnGetItemText( m_LastRowRightClicked, m_LastColumnRightClicked );
+        ItemText.Replace( wxT( "&" ), wxT( "&&" ) );
+        MenuText += wxT( " '" ) + ItemText + wxT( "'" );
 
-            int Rating = m_Items[ m_LastRowRightClicked - m_ItemsFirst ].m_Rating;
-
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_0, wxT( "☆☆☆☆☆" ), _( "Set the rating to 0" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating <= 0 );
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_1, wxT( "★☆☆☆☆" ), _( "Set the rating to 1" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating == 1 );
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_2, wxT( "★★☆☆☆" ), _( "Set the rating to 2" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating == 2 );
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_3, wxT( "★★★☆☆" ), _( "Set the rating to 3" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating == 3 );
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_4, wxT( "★★★★☆" ), _( "Set the rating to 4" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating == 4 );
-            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_5, wxT( "★★★★★" ), _( "Set the rating to 5" ), wxITEM_CHECK );
-            RatingMenu->Append( MenuItem );
-            MenuItem->Check( Rating == 5 );
-
-            menu->AppendSubMenu( RatingMenu, _( "Set Rating" ), _( "Set the current track rating" ) );
-        }
-        else
-        {
-            MenuText = _( "Set" );
-            MenuText += wxT( " " ) + ColumnNames[ ColumnId ] + wxT( " " );
-            MenuText += _( "to" );
-            MenuText += wxT( " '" ) + OnGetItemText( m_LastRowRightClicked, m_LastColumnRightClicked ) + wxT( "'" );
-
-            MenuItem = new wxMenuItem( menu, ID_SONG_SET_COLUMN,  MenuText, _( "Set the clicked column for the selected tracks" ) );
-            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
-            menu->Append( MenuItem );
-        }
+        MenuItem = new wxMenuItem( menu, ID_SONG_SET_COLUMN,  MenuText, _( "Set the clicked column for the selected tracks" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
+        menu->Append( MenuItem );
     }
-
-    menu->AppendSeparator();
 }
 
+
 // -------------------------------------------------------------------------------- //
 void guSoListBox::CreateContextMenu( wxMenu * Menu ) const
 {
     wxMenuItem * MenuItem;
     int SelCount = GetSelectedCount();
+    int ContextMenuFlags = m_LibPanel ? m_LibPanel->GetContextMenuFlags() : guLIBRARY_CONTEXTMENU_DEFAULT;
 
-    if( SelCount )
-    {
-        MenuItem = new wxMenuItem( Menu, ID_SONG_PLAY, _( "Play" ), _( "Play current selected songs" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
-        Menu->Append( MenuItem );
-    }
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_PLAYALL, _( "Play All" ), _( "Play all songs" ) );
+    MenuItem = new wxMenuItem( Menu, ID_SONG_PLAY,
+                            wxString( _( "Play" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_PLAY ),
+                            _( "Play current selected songs" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_player_tiny_light_play ) );
     Menu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
 
-    if( SelCount )
-    {
-        MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE, _( "Enqueue" ), _( "Add current selected songs to the playlist" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
-        Menu->Append( MenuItem );
-
-        MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE_ASNEXT, _( "Enqueue Next" ), _( "Add current selected songs to the playlist as Next Tracks" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
-        Menu->Append( MenuItem );
-    }
-
-    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUEALL, _( "Enqueue All" ), _( "Add all songs to the playlist" ) );
+    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUE_AFTER_ALL,
+                            wxString( _( "Enqueue" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALL ),
+                            _( "Add current selected songs to the playlist" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
     Menu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
 
-    MenuItem = new wxMenuItem( Menu, ID_SONG_ENQUEUEALL_ASNEXT, _( "Enqueue All Next" ), _( "Add all songs to the playlist as Next Tracks" ) );
+    wxMenu * EnqueueMenu = new wxMenu();
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_SONG_ENQUEUE_AFTER_TRACK,
+                            wxString( _( "Current Track" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_TRACK ),
+                            _( "Add current selected tracks to playlist after the current track" ) );
     MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
-    Menu->Append( MenuItem );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_SONG_ENQUEUE_AFTER_ALBUM,
+                            wxString( _( "Current Album" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ALBUM ),
+                            _( "Add current selected tracks to playlist after the current album" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    MenuItem = new wxMenuItem( EnqueueMenu, ID_SONG_ENQUEUE_AFTER_ARTIST,
+                            wxString( _( "Current Artist" ) ) +  guAccelGetCommandKeyCodeString( ID_SONG_ENQUEUE_AFTER_ARTIST ),
+                            _( "Add current selected tracks to playlist after the current artist" ) );
+    MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_add ) );
+    EnqueueMenu->Append( MenuItem );
+    MenuItem->Enable( SelCount );
+
+    Menu->Append( wxID_ANY, _( "Enqueue after" ), EnqueueMenu );
 
     if( SelCount )
     {
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE_LIBRARY, _( "Remove from Library" ), _( "Remove the current selected tracks from library" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
+        MenuItem = new wxMenuItem( Menu, ID_SONG_EDITLABELS,
+                                wxString( _( "Edit Labels" ) ) +  guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITLABELS ),
+                                _( "Edit the labels assigned to the selected songs" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tags ) );
         Menu->Append( MenuItem );
 
+        if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_EDIT_TRACKS )
+        {
+            MenuItem = new wxMenuItem( Menu, ID_SONG_EDITTRACKS,
+                                wxString( _( "Edit Songs" ) ) +  guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_EDITTRACKS ),
+                                _( "Edit the songs selected" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit ) );
+            Menu->Append( MenuItem );
+
+            Menu->AppendSeparator();
+
+            AppendFastEditMenu( Menu, SelCount );
+
+
+
+            wxMenu * RatingMenu = new wxMenu();
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_0,
+                                    wxT( "☆☆☆☆☆" ),
+                                    _( "Set the rating to 0" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_1,
+                                    wxT( "★☆☆☆☆" ),
+                                    _( "Set the rating to 1" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_2,
+                                    wxT( "★★☆☆☆" ),
+                                    _( "Set the rating to 2" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_3,
+                                    wxT( "★★★☆☆" ),
+                                    _( "Set the rating to 3" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_4,
+                                    wxT( "★★★★☆" ),
+                                    _( "Set the rating to 4" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            MenuItem = new wxMenuItem( RatingMenu, ID_SONG_SET_RATING_5,
+                                    wxT( "★★★★★" ),
+                                    _( "Set the rating to 5" ), wxITEM_NORMAL );
+            RatingMenu->Append( MenuItem );
+
+            Menu->AppendSubMenu( RatingMenu, _( "Set Rating" ), _( "Set the current track rating" ) );
+        }
+
+        Menu->AppendSeparator();
+
+        MenuItem = new wxMenuItem( Menu, ID_SONG_SAVETOPLAYLIST,
+                                wxString( _( "Save to Playlist" ) ) +  guAccelGetCommandKeyCodeString( ID_PLAYER_PLAYLIST_SAVE ),
+                                _( "Save all selected tracks as a playlist" ) );
+        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_doc_save ) );
+        Menu->Append( MenuItem );
+
+        Menu->AppendSeparator();
+
+        if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_DELETEFROMLIBRARY )
+        {
+            MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE_LIBRARY, _( "Remove from Library" ), _( "Remove the current selected tracks from library" ) );
+            MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
+            Menu->Append( MenuItem );
+        }
+
         MenuItem = new wxMenuItem( Menu, ID_SONG_DELETE_DRIVE, _( "Delete from Drive" ), _( "Remove the current selected tracks from drive" ) );
         MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tiny_edit_clear ) );
         Menu->Append( MenuItem );
 
         Menu->AppendSeparator();
 
-        MenuItem = new wxMenuItem( Menu, ID_SONG_SAVEPLAYLIST, _( "Save to Playlist" ), _( "Save all selected tracks as a playlist" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_doc_save ) );
-        Menu->Append( MenuItem );
-
-        MenuItem = new wxMenuItem( Menu, ID_SONG_COPYTO, _( "Copy to..." ), _( "Copy the current selected songs to a directory or device" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit_copy ) );
-        Menu->Append( MenuItem );
-
-        Menu->AppendSeparator();
-
-        MenuItem = new wxMenuItem( Menu, ID_SONG_EDITLABELS, _( "Edit Labels" ), _( "Edit the labels assigned to the selected songs" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_tags ) );
-        Menu->Append( MenuItem );
-
-        MenuItem = new wxMenuItem( Menu, ID_SONG_EDITTRACKS, _( "Edit Songs" ), _( "Edit the songs selected" ) );
-        MenuItem->SetBitmap( guImage( guIMAGE_INDEX_edit ) );
-        Menu->Append( MenuItem );
-
-        Menu->AppendSeparator();
-
-        AppendFastEditMenu( Menu, SelCount );
-
         wxMenu *     SubMenu;
         SubMenu = new wxMenu();
 
@@ -560,20 +651,47 @@ void guSoListBox::CreateContextMenu( wxMenu * Menu ) const
         MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ALBUMARTIST, _( "Album Artist" ), _( "Selects the album artist of the current song" ) );
         SubMenu->Append( MenuItem );
 
+        MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_COMPOSER, _( "Composer" ), _( "Selects the composer of the current song" ) );
+        SubMenu->Append( MenuItem );
+
         MenuItem = new wxMenuItem( Menu, ID_SONG_BROWSE_ALBUM, _( "Album" ), _( "Select the album of the current song" ) );
         SubMenu->Append( MenuItem );
 
         Menu->AppendSubMenu( SubMenu, _( "Select" ), _( "Search in the library" ) );
 
-        Menu->AppendSeparator();
-
-        if( SelCount == 1 )
+        if( ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COPY_TO ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) ||
+            ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS ) )
         {
-            AddOnlineLinksMenu( Menu );
-        }
+            Menu->AppendSeparator();
 
-        AddSongsCommands( Menu, SelCount );
+            if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COPY_TO )
+            {
+                if( m_LibPanel )
+                {
+                    m_LibPanel->CreateCopyToMenu( Menu, ID_SONG_COPYTO );
+                }
+                else
+                {
+                    guMainFrame * MainFrame = ( guMainFrame * ) wxTheApp->GetTopWindow();
+                    MainFrame->CreateCopyToMenu( Menu, ID_SONG_COPYTO );
+                }
+            }
+
+            if( SelCount == 1 && ( ContextMenuFlags & guLIBRARY_CONTEXTMENU_LINKS ) )
+            {
+                AddOnlineLinksMenu( Menu );
+            }
+
+            if( ContextMenuFlags & guLIBRARY_CONTEXTMENU_COMMANDS )
+            {
+                AddSongsCommands( Menu, SelCount );
+            }
+        }
     }
+
+    if( m_LibPanel )
+        m_LibPanel->CreateContextMenu( Menu, guLIBRARY_ELEMENT_TRACKS );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -589,7 +707,6 @@ void guSoListBox::OnSearchLinkClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Links = Config->ReadAStr( wxT( "Link" ), wxEmptyString, wxT( "SearchLinks" ) );
-            wxASSERT( Links.Count() > 0 );
 
             index -= ID_LASTFM_SEARCH_LINK;
             wxString SearchLink = Links[ index ];
@@ -620,7 +737,6 @@ void guSoListBox::OnCommandClicked( wxCommandEvent &event )
         if( Config )
         {
             wxArrayString Commands = Config->ReadAStr( wxT( "Cmd" ), wxEmptyString, wxT( "Commands" ) );
-            wxASSERT( Commands.Count() > 0 );
 
             index -= ID_SONGS_COMMANDS;
             wxString CurCmd = Commands[ index ];
@@ -737,12 +853,6 @@ void guSoListBox::OnItemColumnClicked( wxListEvent &event )
         if( m_Items[ Row - m_ItemsFirst ].m_Rating == Rating )
             Rating = 0;
 
-//        m_Items[ Row - m_ItemsFirst ].m_Rating = Rating;
-//        m_Db->SetTrackRating( m_Items[ Row - m_ItemsFirst ].m_SongId, Rating );
-//        //RefreshLine( Row );
-//
-//        // Update the track in database, playlist, etc
-//        ( ( guMainFrame * ) wxTheApp->GetTopWindow() )->UpdatedTrack( guUPDATED_TRACKS_NONE, &m_Items[ Row - m_ItemsFirst ] );
         wxCommandEvent RatingEvent( wxEVT_COMMAND_MENU_SELECTED, ID_SONG_SET_RATING_0 + Rating );
         AddPendingEvent( RatingEvent );
     }
@@ -800,6 +910,7 @@ void guSoListBox::UpdatedTracks( const guTrackArray * tracks )
 {
     int ItemIndex;
     int ItemCount = m_Items.Count();
+    bool ClearSelection = false;
 
     if( !ItemCount )
         return;
@@ -812,13 +923,19 @@ void guSoListBox::UpdatedTracks( const guTrackArray * tracks )
 
         for( ItemIndex = 0; ItemIndex < ItemCount; ItemIndex++ )
         {
-            if( CurTrack.m_FileName == m_Items[ ItemIndex ].m_FileName )
+            if( ( ( CurTrack.m_SongId == m_Items[ ItemIndex ].m_SongId ) &&
+                    ( ( CurTrack.m_Type == guTRACK_TYPE_JAMENDO ) || ( CurTrack.m_Type == guTRACK_TYPE_MAGNATUNE ) ) ) ||
+                ( CurTrack.m_FileName == m_Items[ ItemIndex ].m_FileName ) )
             {
                 m_Items[ ItemIndex ] = CurTrack;
                 RefreshLine( ItemIndex + m_ItemsFirst );
+                if( !ClearSelection )
+                    ClearSelection = true;
             }
         }
     }
+    if( ClearSelection )
+        ClearSelectedItems();
 }
 
 // -------------------------------------------------------------------------------- //
