@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -41,24 +41,29 @@
 #define guSONGS_COLUMN_LASTPLAY     13
 #define guSONGS_COLUMN_ADDEDDATE    14
 #define guSONGS_COLUMN_FORMAT       15
+#define guSONGS_COLUMN_FILEPATH     16
 
-#define guSONGS_COLUMN_COUNT        16
+#define guSONGS_COLUMN_COUNT        17
+
+class guLibPanel;
 
 // -------------------------------------------------------------------------------- //
 class guSoListBox : public guListView
 {
   protected :
-    guDbLibrary *       m_Db;
-    guTrackArray        m_Items;
-    wxMutex             m_ItemsMutex;
-    int                 m_ItemsFirst;
-    int                 m_ItemsLast;
-    wxString            m_ConfName;
-    int                 m_LastColumnRightClicked;
-    int                 m_LastRowRightClicked;
+    guLibPanel *                m_LibPanel;
 
-    wxBitmap *          m_NormalStar;
-    wxBitmap *          m_SelectStar;
+    guDbLibrary *               m_Db;
+    guTrackArray                m_Items;
+    wxMutex                     m_ItemsMutex;
+    int                         m_ItemsFirst;
+    int                         m_ItemsLast;
+    wxString                    m_ConfName;
+    int                         m_LastColumnRightClicked;
+    int                         m_LastRowRightClicked;
+
+    wxBitmap *                  m_NormalStar;
+    wxBitmap *                  m_SelectStar;
 
     virtual void                DrawItem( wxDC &dc, const wxRect &rect, const int row, const int col ) const;
     virtual void                CreateContextMenu( wxMenu * Menu ) const;
@@ -80,8 +85,11 @@ class guSoListBox : public guListView
 
     virtual void                AppendFastEditMenu( wxMenu * menu, const int selcount ) const;
 
+    virtual void                OnConfigUpdated( wxCommandEvent &event );
+    virtual void                CreateAcceleratorTable();
+
   public :
-    guSoListBox( wxWindow * parent, guDbLibrary * NewDb, wxString confname, long style = 0 );
+    guSoListBox( wxWindow * parent, guLibPanel * libpanel, guDbLibrary * NewDb, wxString confname, long style = 0 );
     ~guSoListBox();
 
     virtual void                ReloadItems( bool reset = true );
