@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -44,7 +44,7 @@ class guGauge : public wxControl
 
 
   public :
-    guGauge() : wxControl() {};
+    guGauge() : wxControl() { m_LastValue = wxNOT_FOUND; m_PaintWidth = 0; };
     guGauge( wxWindow * parent, const wxString &label = wxEmptyString, bool showporcent = true, wxWindowID id = wxID_ANY, unsigned int max = 100,
                const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, long style = wxGA_HORIZONTAL );
     ~guGauge( void );
@@ -63,18 +63,33 @@ class guStatusBar : public wxStatusBar
   private:
     guGaugeArray        m_Gauges;
     wxStaticBitmap *    m_ASBitmap;
+    wxStaticBitmap *    m_PlayMode;
     wxStaticText *      m_SelInfo;
+    bool                m_ForceGapless;
+
+    int                 m_LastClickAction;
+    wxTimer             m_ClickTimer;
 
     void                OnSize( wxSizeEvent &event );
     void                SetSizes( int fieldcnt );
     void                UpdateGauges( void );
-    void                OnAudioScrobbleClicked( wxMouseEvent &event );
+
+    void                OnAudioScrobbleClicked( void );
+    void                OnAudioScrobbleDClicked( void );
+    void                OnPlayModeClicked( void );
+    void                OnPlayModeDClicked( void );
+
+    void                OnButtonClick( wxMouseEvent &event );
+    void                OnButtonDClick( wxMouseEvent &event );
+    void                OnTimerEvent( wxTimerEvent &event );
+
+    void                OnConfigUpdated( wxCommandEvent &event );
 
   public:
                         guStatusBar( wxWindow * parent );
     virtual             ~guStatusBar();
 
-    void                SetAudioScrobbleService( bool Enabled = false );
+    void                UpdateAudioScrobbleIcon( bool Enabled = false );
     int                 AddGauge( const wxString &text = wxEmptyString, bool showporcent = true );
     int                 RemoveGauge( int gaugeid );
     void                Pulse( int id ) { /*m_Gauges[ id ]->Pulse(); */ };
@@ -83,6 +98,9 @@ class guStatusBar : public wxStatusBar
 
     void                SetSelInfo( const wxString &label );
 
+    void                SetPlayMode( const bool forcegapless );
+    void                SetAudioScrobble( const bool audioscrobble );
+
 };
 
 
