@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -31,18 +31,22 @@
 #include <id3v2framefactory.h>
 #include <textidentificationframe.h>
 #include <unsynchronizedlyricsframe.h>
+#include <asffile.h>
 #include <mpegfile.h>
 #include <flacfile.h>
 #include <mp4file.h>
 #include <mpcfile.h>
 #include <oggfile.h>
 #include <vorbisfile.h>
+#include <wavpackfile.h>
+#include <trueaudiofile.h>
 
 #include <xiphcomment.h>
 
 #include <mp4tag.h>
 #include <apetag.h>
 #include <id3v2tag.h>
+#include <asftag.h>
 
 // FLAC Dev files
 #include <FLAC/metadata.h>
@@ -58,6 +62,13 @@ using namespace TagLib;
 #define wxStringToTString(s) TagLib::String(s.ToUTF8(), TagLib::String::UTF8)
 #define TStringTowxString(s) wxString::FromUTF8( s.toCString(true) )
 
+#define     guTRACK_CHANGED_DATA_NONE           0
+#define     guTRACK_CHANGED_DATA_TAGS           ( 1 << 0 )
+#define     guTRACK_CHANGED_DATA_IMAGES         ( 1 << 1 )
+#define     guTRACK_CHANGED_DATA_LYRICS         ( 1 << 2 )
+#define     guTRACK_CHANGED_DATA_LABELS         ( 1 << 3 )
+#define     guTRACK_CHANGED_DATA_RATING         ( 1 << 4 )
+
 // -------------------------------------------------------------------------------- //
 class guTagInfo
 {
@@ -94,7 +105,7 @@ class guTagInfo
 
     void                SetFileName( const wxString &filename );
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
 
     virtual bool        CanHandleImages( void );
     virtual wxImage *   GetImage( void );
@@ -120,7 +131,7 @@ class guMp3TagInfo : public guTagInfo
     ~guMp3TagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
     virtual bool        CanHandleImages( void );
     virtual wxImage *   GetImage( void );
     virtual bool        SetImage( const wxImage * image );
@@ -142,7 +153,7 @@ class guFlacTagInfo : public guTagInfo
     ~guFlacTagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
 
     virtual bool        CanHandleImages( void );
     virtual wxImage *   GetImage();
@@ -164,7 +175,7 @@ class guOggTagInfo : public guTagInfo
     ~guOggTagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
 
     virtual bool        CanHandleImages( void );
     virtual wxImage *   GetImage( void );
@@ -186,7 +197,7 @@ class guMp4TagInfo : public guTagInfo
     ~guMp4TagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
 
 #ifdef TAGLIB_WITH_MP4_COVERS
     virtual bool        CanHandleImages( void );
@@ -210,7 +221,7 @@ class guApeTagInfo : public guTagInfo
     ~guApeTagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
     virtual bool        CanHandleLyrics( void );
     virtual wxString    GetLyrics( void );
     virtual bool        SetLyrics( const wxString &lyrics );
@@ -227,23 +238,89 @@ class guMpcTagInfo : public guTagInfo
     ~guMpcTagInfo();
 
     virtual bool        Read( void );
-    virtual bool        Write( void );
+    virtual bool        Write( const int changedflag );
 
     virtual bool        CanHandleImages( void );
     virtual wxImage *   GetImage( void );
     virtual bool        SetImage( const wxImage * image );
 };
 
+// -------------------------------------------------------------------------------- //
+class guWavPackTagInfo : public guTagInfo
+{
+  protected :
+    TagLib::APE::Tag * m_ApeTag;
+
+  public :
+    guWavPackTagInfo( const wxString &filename = wxEmptyString );
+    ~guWavPackTagInfo();
+
+    virtual bool        Read( void );
+    virtual bool        Write( const int changedflag );
+
+    virtual bool        CanHandleImages( void );
+    virtual wxImage *   GetImage( void );
+    virtual bool        SetImage( const wxImage * image );
+    virtual bool        CanHandleLyrics( void );
+    virtual wxString    GetLyrics( void );
+    virtual bool        SetLyrics( const wxString &lyrics );
+};
+
+// -------------------------------------------------------------------------------- //
+class guTrueAudioTagInfo : public guTagInfo
+{
+  protected :
+    ID3v2::Tag *        m_TagId3v2;
+
+  public :
+    guTrueAudioTagInfo( const wxString &filename = wxEmptyString );
+    ~guTrueAudioTagInfo();
+
+    virtual bool        Read( void );
+    virtual bool        Write( const int changedflag );
+
+    virtual bool        CanHandleImages( void );
+    virtual wxImage *   GetImage( void );
+    virtual bool        SetImage( const wxImage * image );
+    virtual bool        CanHandleLyrics( void );
+    virtual wxString    GetLyrics( void );
+    virtual bool        SetLyrics( const wxString &lyrics );
+};
+
+// -------------------------------------------------------------------------------- //
+class guASFTagInfo : public guTagInfo
+{
+  protected :
+    ASF::Tag *        m_ASFTag;
+
+  public :
+    guASFTagInfo( const wxString &filename = wxEmptyString );
+    ~guASFTagInfo();
+
+    virtual bool        Read( void );
+    virtual bool        Write( const int changedflag );
+
+    virtual bool        CanHandleImages( void );
+    virtual wxImage *   GetImage( void );
+    virtual bool        SetImage( const wxImage * image );
+    virtual bool        CanHandleLyrics( void );
+    virtual wxString    GetLyrics( void );
+    virtual bool        SetLyrics( const wxString &lyrics );
+};
+
 
 class guImagePtrArray;
 
 // -------------------------------------------------------------------------------- //
 wxImage *   guTagGetPicture( const wxString &filename );
 bool        guTagSetPicture( const wxString &filename, wxImage * picture );
+bool        guTagSetPicture( const wxString &filename, const wxString &imagefile );
 wxString    guTagGetLyrics( const wxString &filename );
-bool        guTagSetPicture( const wxString &filename, wxString &lyrics );
-void        UpdateImages( const guTrackArray &songs, const guImagePtrArray &images );
-void        UpdateLyrics( const guTrackArray &songs, const wxArrayString &lyrics );
-
+bool        guTagSetLyrics( const wxString &filename, wxString &lyrics );
+void        guUpdateTracks( const guTrackArray &tracks, const guImagePtrArray &images,
+                    const wxArrayString &lyrics, const wxArrayInt &changedflags );
+void        guUpdateImages( const guTrackArray &songs, const guImagePtrArray &images, const wxArrayInt &changedflags );
+void        guUpdateLyrics( const guTrackArray &songs, const wxArrayString &lyrics, const wxArrayInt &changedflags );
+bool        guStrDiskToDiskNum( const wxString &diskstr, int &disknum, int &disktotal );
 #endif
 // -------------------------------------------------------------------------------- //
