@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -19,7 +19,10 @@
 //
 // -------------------------------------------------------------------------------- //
 #include "Utils.h"
+
 #include "Config.h"
+#include "DbLibrary.h"
+#include "FileRenamer.h"    // NormalizeField
 
 #include <wx/curl/http.h>
 #include <wx/process.h>
@@ -166,26 +169,19 @@ wxImage * guGetRemoteImage( const wxString &url, int &imgtype )
 
     wxString FileName = Uri.GetPath().Lower();
 
-    if( FileName.EndsWith( wxT( ".jpg" ) ) ||
-        FileName.EndsWith( wxT( ".jpeg" ) ) )
-      imgtype = wxBITMAP_TYPE_JPEG;
-    else if( FileName.EndsWith( wxT( ".png" ) ) )
-      imgtype = wxBITMAP_TYPE_PNG;
-    else if( FileName.EndsWith( wxT( ".gif" ) ) )
-      imgtype = wxBITMAP_TYPE_GIF;
-    else if( FileName.EndsWith( wxT( ".bmp" ) ) )
-      imgtype = wxBITMAP_TYPE_BMP;
-    else
-      imgtype = wxBITMAP_TYPE_INVALID;
+    //guLogMessage( wxT( "Downloading '%s' from '%s'" ), FileName.c_str(), url.c_str() );
 
-    if( imgtype != wxBITMAP_TYPE_INVALID )
-    {
-        wxMemoryOutputStream Buffer;
-        wxCurlHTTP http;
-        http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
-        if( http.Get( Buffer, url ) )
+    wxMemoryOutputStream Buffer;
+    wxCurlHTTP http;
+    http.AddHeader( wxT( "User-Agent: " ) guDEFAULT_BROWSER_USER_AGENT );
+    //http.AddHeader( wxT( "Accept: */*" ) );
+    //http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
+    try {
+        http.Get( Buffer, url );
+        if( Buffer.IsOk() )
         {
             long ResCode = http.GetResponseCode();
+            //guLogMessage( wxT( "ResCode: %lu" ), ResCode );
             if( ( ResCode < 200 ) || ( ResCode > 299 ) )
             {
                 //guLogMessage( wxT( "Code   : %u\n%s" ), ResCode, http.GetResponseHeader().c_str() );
@@ -202,37 +198,57 @@ wxImage * guGetRemoteImage( const wxString &url, int &imgtype )
                 }
             }
 
-            if( ResCode != 200 )
-            {
-                guLogMessage( wxT( "Error %u getting remote image '%s'\n%s" ),
-                    http.GetResponseCode(),
-                    url.c_str(),
-                    http.GetResponseHeader().c_str() );
-            }
+    //        if( ResCode != 200 )
+    //        {
+    //            guLogMessage( wxT( "Error %u getting remote image '%s'\n%s" ),
+    //                http.GetResponseCode(),
+    //                url.c_str(),
+    //                http.GetResponseHeader().c_str() );
+    //        }
 
             if( Buffer.IsOk() )
             {
                 wxMemoryInputStream Ins( Buffer );
                 if( Ins.IsOk() )
                 {
-                    Image = new wxImage( Ins, imgtype );
-                    if( Image )
+                    if( FileName.EndsWith( wxT( ".jpg" ) ) ||
+                        FileName.EndsWith( wxT( ".jpeg" ) ) )
+                      imgtype = wxBITMAP_TYPE_JPEG;
+                    else if( FileName.EndsWith( wxT( ".png" ) ) )
+                      imgtype = wxBITMAP_TYPE_PNG;
+                    else if( FileName.EndsWith( wxT( ".gif" ) ) )
+                      imgtype = wxBITMAP_TYPE_GIF;
+                    else if( FileName.EndsWith( wxT( ".bmp" ) ) )
+                      imgtype = wxBITMAP_TYPE_BMP;
+                    else
+                      imgtype = wxBITMAP_TYPE_INVALID;
+
+                    if( imgtype != wxBITMAP_TYPE_INVALID )
                     {
-                        if( Image->IsOk() )
+                        Image = new wxImage( Ins, imgtype );
+                        if( Image )
                         {
-                            return Image;
+                            if( Image->IsOk() )
+                            {
+                                return Image;
+                            }
+                            delete Image;
                         }
-                        delete Image;
                     }
                 }
             }
         }
     }
+    catch( ... )
+    {
+        guLogError( wxT( "Exception downloading image '%s'" ), url.c_str() );
+    }
+
     return NULL;
 }
 
 // -------------------------------------------------------------------------------- //
-bool DownloadImage( const wxString &source, const wxString &target, int maxwidth, int maxheight )
+bool DownloadImage( const wxString &source, const wxString &target, const int imagetype, int maxwidth, int maxheight )
 {
     bool        RetVal = false;
     int         ImageType;
@@ -247,13 +263,19 @@ bool DownloadImage( const wxString &source, const wxString &target, int maxwidth
             else
                 Image->Rescale( maxwidth, maxwidth, wxIMAGE_QUALITY_HIGH );
         }
-        RetVal = Image->SaveFile( target, wxBITMAP_TYPE_JPEG );
+        RetVal = Image->SaveFile( target, imagetype );
 
         delete Image;
     }
     return RetVal;
 }
 
+// -------------------------------------------------------------------------------- //
+bool DownloadImage( const wxString &source, const wxString &target, int maxwidth, int maxheight )
+{
+    return DownloadImage( source, target, wxBITMAP_TYPE_JPEG, maxwidth, maxheight );
+}
+
 // -------------------------------------------------------------------------------- //
 int DownloadFile( const wxString &Source, const wxString &Target )
 {
@@ -399,10 +421,10 @@ wxString GetUrlContent( const wxString &url, const wxString &referer, bool gzipp
     wxString RetVal = wxEmptyString;
 
     http.AddHeader( wxT( "User-Agent: " ) guDEFAULT_BROWSER_USER_AGENT );
-    http.AddHeader( wxT( "Accept: */*" ) );
+    http.AddHeader( wxT( "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8" ) );
     if( gzipped )
     {
-        http.AddHeader( wxT( "Accept Encoding: gzip" ) );
+        http.AddHeader( wxT( "Accept Encoding: gzip,deflate" ) );
     }
     http.AddHeader( wxT( "Accept-Charset: utf-8" ) );
     if( !referer.IsEmpty() )
@@ -415,17 +437,30 @@ wxString GetUrlContent( const wxString &url, const wxString &referer, bool gzipp
     wxMemoryOutputStream Buffer;
     http.SetOpt( CURLOPT_FOLLOWLOCATION, 1 );
     http.Get( Buffer, url );
+
     if( Buffer.IsOk() )
     {
         int ResponseCode = http.GetResponseCode();
+        //guLogMessage( wxT( "ResponseCode: %i" ), ResponseCode );
         if( ResponseCode >= 300  && ResponseCode < 400 )
         {
+            //guLogMessage( wxT( "Response %u:\n%s\n%s" ), http.GetResponseCode(), http.GetResponseHeader().c_str(), http.GetResponseBody().c_str() );
             wxString Location = http.GetResponseHeader();
             int Pos = Location.Lower().Find( wxT( "location: " ) );
             if( Pos != wxNOT_FOUND )
             {
                 Location = Location.Mid( Pos + 10 );
                 Location.Truncate( Location.Find( wxT( "\r\n" ) ) );
+                if( Location.StartsWith( wxT( "/" ) ) )
+                {
+                    wxURI Uri( url );
+                    wxString NewURL;
+                    if( Uri.HasScheme() )
+                        NewURL = Uri.GetScheme() + wxT( "://" );
+                    NewURL += Uri.GetServer();
+                    NewURL += Location;
+                    Location = NewURL;
+                }
                 return GetUrlContent( Location, referer, gzipped );
             }
         }
@@ -433,8 +468,7 @@ wxString GetUrlContent( const wxString &url, const wxString &referer, bool gzipp
             return wxEmptyString;
 
         wxString ResponseHeaders = http.GetResponseHeader();
-//        guLogMessage( wxT( "Response %u:\n%s\n%s" ),
-//            http.GetResponseCode(), http.GetResponseHeader().c_str(), http.GetResponseBody().c_str() );
+        //guLogMessage( wxT( "Response %u:\n%s\n%s" ), http.GetResponseCode(), http.GetResponseHeader().c_str(), http.GetResponseBody().c_str() );
 
         if( ResponseHeaders.Lower().Find( wxT( "content-encoding: gzip" ) ) != wxNOT_FOUND )
         {
@@ -452,14 +486,16 @@ wxString GetUrlContent( const wxString &url, const wxString &referer, bool gzipp
 //            Ins.Read( Outs );
             if( Buffer.GetLength() )
             {
-                RetVal = wxString::FromUTF8( ( const char * ) Buffer.GetOutputStreamBuffer()->GetBufferStart(), Buffer.GetLength() );
+                size_t Count = Buffer.GetLength();
+                const char * pData = ( const char * ) Buffer.GetOutputStreamBuffer()->GetBufferStart();
+                RetVal = wxString( pData, wxConvUTF8, Count );
                 if( RetVal.IsEmpty() )
                 {
-                    RetVal = wxString::From8BitData( ( const char * ) Buffer.GetOutputStreamBuffer()->GetBufferStart(), Buffer.GetLength() );
-                }
-                if( RetVal.IsEmpty() )
-                {
-                    RetVal = wxString( ( const char * ) Buffer.GetOutputStreamBuffer()->GetBufferStart(), wxConvISO8859_1 );
+                    RetVal = wxString( pData, wxConvISO8859_1, Count );
+                    if( RetVal.IsEmpty() )
+                    {
+                        RetVal = wxString( pData, wxConvLibc, Count );
+                    }
                 }
             }
         }
@@ -469,7 +505,7 @@ wxString GetUrlContent( const wxString &url, const wxString &referer, bool gzipp
     {
         guLogError( wxT( "Could not get '%s'" ), url.c_str() );
     }
-    //guLogMessage( wxT( "Response:\n%s" ), RetVal.c_str() );
+    //guLogMessage( wxT( "Response:\n%s\n###############" ), RetVal.c_str() );
     return RetVal;
 }
 
@@ -553,3 +589,174 @@ bool guRenameFile( const wxString &oldname, const wxString &newname, bool overwr
 }
 
 // -------------------------------------------------------------------------------- //
+wxString guGetNextXMLChunk( wxFile &xmlfile, wxFileOffset &CurPos, const char * startstr, const char * endstr )
+{
+    #define XMLREAD_BUFFER_SIZE         10240
+    wxString RetVal;
+    //wxFileOffset CurPos = xmlfile.Tell();
+    CurPos = xmlfile.Tell();
+    wxFileOffset StartPos = wxNOT_FOUND;
+    wxFileOffset EndPos = wxNOT_FOUND;
+    int endstrlen = strlen( endstr );
+    char * Buffer = ( char * ) malloc( XMLREAD_BUFFER_SIZE + 1 );
+    if( Buffer )
+    {
+        while( StartPos == wxNOT_FOUND )
+        {
+            int ReadCount = xmlfile.Read( Buffer, XMLREAD_BUFFER_SIZE );
+            if( ReadCount != wxInvalidOffset && ReadCount > 0 )
+            {
+                Buffer[ ReadCount ] = 0;
+            }
+            else
+            {
+                break;
+            }
+            char * StartString = strstr( Buffer, startstr );
+            if( StartString )
+            {
+                StartPos = CurPos + ( StartString - Buffer );
+                break;
+            }
+            else
+            {
+                CurPos += ReadCount;
+            }
+        }
+
+        if( StartPos != wxNOT_FOUND )
+        {
+            xmlfile.Seek( StartPos );
+            CurPos = StartPos;
+            while( EndPos == wxNOT_FOUND )
+            {
+                int ReadCount = xmlfile.Read( Buffer, XMLREAD_BUFFER_SIZE );
+                if( ReadCount != wxInvalidOffset && ReadCount > 0 )
+                {
+                    Buffer[ ReadCount ] = 0;
+                }
+                else
+                {
+                    break;
+                }
+                char * EndString = strstr( Buffer, endstr );
+                if( EndString )
+                {
+                    EndPos = CurPos + ( EndString - Buffer ) + endstrlen;
+                    break;
+                }
+                else
+                {
+                    // Prevent that </artist was partially included
+                    CurPos += ReadCount - endstrlen;
+                    xmlfile.Seek( -endstrlen, wxFromCurrent );
+                }
+            }
+
+            if( EndPos != wxNOT_FOUND )
+            {
+                //guLogMessage( wxT( "Found From %lli => %lli  (%lli)" ), StartPos, EndPos, EndPos - StartPos );
+                int BufferSize = EndPos - StartPos;
+                if( BufferSize )
+                {
+                    char * BufferString = ( char * ) malloc( BufferSize + 1 );
+                    if( BufferString )
+                    {
+                        xmlfile.Seek( StartPos );
+                        int ReadCount = xmlfile.Read( BufferString, BufferSize );
+                        if( ReadCount != wxInvalidOffset )
+                        {
+                            BufferString[ ReadCount ] = 0;
+                            RetVal = wxString( BufferString, wxConvUTF8 );
+                            //guLogMessage( wxT( "%s" ), RetVal.c_str() );
+                        }
+
+                        free( BufferString );
+                    }
+                }
+            }
+        }
+
+        free( Buffer );
+    }
+
+
+    return RetVal;
+}
+
+// -------------------------------------------------------------------------------- //
+wxString guExpandTrackMacros( const wxString &pattern, guTrack * track, const int indexpos )
+{
+    wxString RetVal = pattern;
+
+    if( RetVal.Find( wxT( "{a}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{a}" ), NormalizeField( track->m_ArtistName ) );
+    if( RetVal.Find( wxT( "{al}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{al}" ), NormalizeField( track->m_ArtistName.Lower() ) );
+    if( RetVal.Find( wxT( "{au}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{au}" ), NormalizeField( track->m_ArtistName.Upper() ) );
+    if( RetVal.Find( wxT( "{a1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{a1}" ), NormalizeField( track->m_ArtistName.Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{aa}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{aa}" ), NormalizeField( track->m_AlbumArtist ) );
+    if( RetVal.Find( wxT( "{aal}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{aal}" ), NormalizeField( track->m_AlbumArtist.Lower() ) );
+    if( RetVal.Find( wxT( "{aau}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{aau}" ), NormalizeField( track->m_AlbumArtist.Upper() ) );
+    if( RetVal.Find( wxT( "{aa1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{aa1}" ), NormalizeField( track->m_AlbumArtist.Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{A}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{A}" ), NormalizeField( track->m_AlbumArtist.IsEmpty() ? track->m_ArtistName : track->m_AlbumArtist ) );
+    if( RetVal.Find( wxT( "{Al}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{Al}" ), NormalizeField( ( track->m_AlbumArtist.IsEmpty() ? track->m_ArtistName : track->m_AlbumArtist ).Lower() ) );
+    if( RetVal.Find( wxT( "{Au}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{Au}" ), NormalizeField( ( track->m_AlbumArtist.IsEmpty() ? track->m_ArtistName : track->m_AlbumArtist ).Upper() ) );
+    if( RetVal.Find( wxT( "{A1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{A1}" ), NormalizeField( ( track->m_AlbumArtist.IsEmpty() ? track->m_ArtistName : track->m_AlbumArtist ).Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{b}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{b}" ), NormalizeField( track->m_AlbumName ) );
+    if( RetVal.Find( wxT( "{bl}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{bl}" ), NormalizeField( track->m_AlbumName.Lower() ) );
+    if( RetVal.Find( wxT( "{bu}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{bu}" ), NormalizeField( track->m_AlbumName.Upper() ) );
+    if( RetVal.Find( wxT( "{b1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{b1}" ), NormalizeField( track->m_AlbumName.Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{bp}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{bp}" ), NormalizeField( track->m_FileName.BeforeLast( wxT( '/' ) ) ) );
+    if( RetVal.Find( wxT( "{c}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{c}" ), NormalizeField( track->m_Composer ) );
+    if( RetVal.Find( wxT( "{cl}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{cl}" ), NormalizeField( track->m_Composer.Lower() ) );
+    if( RetVal.Find( wxT( "{cu}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{cu}" ), NormalizeField( track->m_Composer.Upper() ) );
+    if( RetVal.Find( wxT( "{c1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{c1}" ), NormalizeField( track->m_Composer.Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{f}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{f}" ), wxFileNameFromPath( track->m_FileName.BeforeLast( wxT( '.' ) ) ) );
+    if( RetVal.Find( wxT( "{g}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{g}" ), NormalizeField( track->m_GenreName ) );
+    if( RetVal.Find( wxT( "{gl}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{gl}" ), NormalizeField( track->m_GenreName.Lower() ) );
+    if( RetVal.Find( wxT( "{gu}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{gu}" ), NormalizeField( track->m_GenreName.Upper() ) );
+    if( RetVal.Find( wxT( "{g1}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{g1}" ), NormalizeField( track->m_GenreName.Trim( false )[ 0 ] ) );
+    if( RetVal.Find( wxT( "{n}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{n}" ), wxString::Format( wxT( "%02u" ), track->m_Number ) );
+    if( RetVal.Find( wxT( "{t}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{t}" ), NormalizeField( track->m_SongName ) );
+    if( RetVal.Find( wxT( "{tl}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{tl}" ), NormalizeField( track->m_SongName.Lower() ) );
+    if( RetVal.Find( wxT( "{tu}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{tu}" ), NormalizeField( track->m_SongName.Upper() ) );
+    if( RetVal.Find( wxT( "{y}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{y}" ), wxString::Format( wxT( "%u" ), track->m_Year ) );
+    if( RetVal.Find( wxT( "{d}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{d}" ), NormalizeField( track->m_Disk ) );
+    if( RetVal.Find( wxT( "{i}" ) ) != wxNOT_FOUND )
+        RetVal.Replace( wxT( "{i}" ), wxString::Format( wxT( "%04u" ), indexpos ) );
+
+    return RetVal;
+}
+
+// -------------------------------------------------------------------------------- //
