@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -20,7 +20,12 @@
 // -------------------------------------------------------------------------------- //
 #ifndef _UTILS_H
 #define _UTILS_H
+
 #include <wx/wx.h>
+#include <wx/file.h>
+#include <wx/wfstream.h>
+#include <wx/mstream.h>
+#include <wx/xml/xml.h>
 
 #if 0
 #define guLogMessage    //wxLogMessage
@@ -43,10 +48,10 @@ template <typename T>
 inline const T &guBound( const T &min, const T &val, const T &max ) { return guMax( min, guMin( max, val ) ); }
 
 
-#define guDEFAULT_BROWSER_USER_AGENT    wxT( "Mozilla/5.0 (X11; U; Linux i686; es-ES; rv:1.9.1.8) Gecko/20100214 Ubuntu/9.10 (karmic) Firefox/3.5.8" )
+#define guDEFAULT_BROWSER_USER_AGENT    wxT( "Mozilla/5.0 (X11; U; Linux x86_64; es-AR; rv:1.9.2.13) Gecko/20101206 Ubuntu/10.10 (maverick) Firefox/3.6.13" )
 
 // -------------------------------------------------------------------------------- //
-void inline guImageResize( wxImage * image, int maxsize )
+void inline guImageResize( wxImage * image, int maxsize, bool forceresize = false )
 {
     int w = image->GetWidth();
     int h = image->GetHeight();
@@ -54,17 +59,20 @@ void inline guImageResize( wxImage * image, int maxsize )
     double ratio = wxMin( static_cast<double>( maxsize ) / h,
                           static_cast<double>( maxsize ) / w );
 
-    if( ratio < 1 )
+    if( forceresize || ( ratio < 1 ) )
     {
         image->Rescale( ( w * ratio ) + .5, ( h * ratio ) + .5, wxIMAGE_QUALITY_HIGH );
     }
 }
 
+class guTrack;
+
 // -------------------------------------------------------------------------------- //
-wxString LenToString( unsigned int len );
+wxString LenToString( const unsigned int len );
 wxString SizeToString( wxFileOffset size );
 wxArrayString guSplitWords( const wxString &InputStr );
 wxImage * guGetRemoteImage( const wxString &url, int &imgtype );
+bool DownloadImage( const wxString &source, const wxString &target, const int imagetype, int maxwidth, int maxheight );
 bool DownloadImage( const wxString &source, const wxString &taget, int maxwidth = -1, int maxheight = -1 );
 int DownloadFile( const wxString &Source, const wxString &Target );
 wxString RemoveSearchFilters( const wxString &Album );
@@ -80,6 +88,8 @@ bool CheckFileLibPath( const wxArrayString &LibPaths, const wxString &filename )
 int guGetFileMode( const wxString &filepath );
 bool guSetFileMode( const wxString &filepath, int mode, bool adding = false );
 bool guRenameFile( const wxString &oldname, const wxString &newname, bool overwrite = true );
+wxString guGetNextXMLChunk( wxFile &xmlfile, wxFileOffset &CurPos, const char * startstr, const char * endstr );
+wxString guExpandTrackMacros( const wxString &pattern, guTrack * track, const int indexpos = 0 );
 
 // -------------------------------------------------------------------------------- //
 #endif
