@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -27,6 +27,8 @@ BEGIN_EVENT_TABLE( guVumeter, wxControl )
 	EVT_PAINT( guVumeter::OnPaint )
 END_EVENT_TABLE()
 
+// -------------------------------------------------------------------------------- //
+// guVumter
 // -------------------------------------------------------------------------------- //
 guVumeter::guVumeter( wxWindow * parent, wxWindowID id, const int style ) : wxControl( parent, id )
 {
@@ -39,6 +41,31 @@ guVumeter::guVumeter( wxWindow * parent, wxWindowID id, const int style ) : wxCo
 	m_OrangeOn      = wxColour( 255, 128,   0 );
 	m_RedOff        = wxColour( 128,   0,   0 );
 	m_RedOn         = wxColour( 255,   0,   0 );
+	m_OffBitmap     = NULL;
+	m_OnBitmap      = NULL;
+	m_LastHeight    = wxNOT_FOUND;
+	m_LastWidth     = wxNOT_FOUND;
+
+    Connect( wxEVT_SIZE, wxSizeEventHandler( guVumeter::OnChangedSize ), NULL, this );
+}
+
+// -------------------------------------------------------------------------------- //
+guVumeter::~guVumeter()
+{
+    wxMutexLocker Locker( m_BitmapMutex );
+    if( m_OffBitmap )
+    {
+        delete m_OffBitmap;
+        m_OffBitmap = NULL;
+    }
+
+    if( m_OnBitmap )
+    {
+        delete m_OnBitmap;
+        m_OnBitmap = NULL;
+    }
+
+    Disconnect( wxEVT_SIZE, wxSizeEventHandler( guVumeter::OnChangedSize ), NULL, this );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -100,138 +127,45 @@ static inline float IEC_Scale( float db )
 void guVumeter::PaintHoriz( void )
 {
 	wxPaintDC dc( this );
-    wxRect Rect;
 	wxCoord Width;
 	wxCoord Height;
 	GetClientSize( &Width, &Height );
-    //guLogMessage( wxT( "%0.2f -> %0.2f" ), m_PeakLevel, IEC_Scale( m_PeakLevel ) );
     //m_PeakLevel = TestValues[ wxGetUTCTime() % 19 ];
+    //guLogMessage( wxT( "%0.2f -> %0.2f" ), m_PeakLevel, IEC_Scale( m_PeakLevel ) );
     int PeakLevel = IEC_Scale( m_PeakLevel ) * 100;
     int DecayLevel = IEC_Scale( m_DecayLevel ) * 100;
     //guLogMessage( wxT( "Peak: %i" ), PeakLevel );
 
-	int SizeG = ( Width * 10 ) / 100;
-
     dc.SetPen( * wxTRANSPARENT_PEN );
 
-    Rect.x = SizeG * 7;
-    Rect.y = 0;
-    Rect.width = SizeG;
-    Rect.height = Height;
+    dc.DrawBitmap( * m_OffBitmap, 0, 0, false );
 
-    if( !PeakLevel )
-    {
-        dc.SetBrush( m_GreenOff );
-        dc.DrawRectangle( 0, 0, Rect.x, Height );
-    }
-    else if( PeakLevel > 70 )
-    {
-        dc.SetBrush( m_GreenOn );
-        dc.DrawRectangle( 0, 0, Rect.x, Height );
-    }
-    else
-    {
-        dc.SetBrush( m_GreenOn );
-        int LevelWidth = PeakLevel * Rect.x / 70;
-        dc.DrawRectangle( 0, 0, LevelWidth, Height );
-        dc.SetBrush( m_GreenOff );
-        dc.DrawRectangle( LevelWidth, 0, Rect.x - LevelWidth, Height );
-    }
+    wxRect ClipRect;
+    ClipRect.y = 0;
+    ClipRect.height = Height;
+    ClipRect.x = 0;
+    ClipRect.width = ( PeakLevel * Width ) / 100;
+    dc.SetClippingRegion( ClipRect );
+    dc.DrawBitmap( * m_OnBitmap, 0, 0, false );
+    dc.DestroyClippingRegion();
 
-    if( PeakLevel < 70 )
+    if( DecayLevel && ( DecayLevel > PeakLevel ) && ( DecayLevel < 100 ) )
     {
-        dc.GradientFillLinear( Rect, m_GreenOff, m_OrangeOff );
-    }
-    else if( PeakLevel > 80 )
-    {
-        dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn );
-    }
-    else
-    {
-        dc.GradientFillLinear( Rect, m_GreenOff, m_OrangeOff );
-        wxRect ClipRect = Rect;
-        ClipRect.width = ( PeakLevel - 70 ) * Rect.width / 10;
-        dc.SetClippingRegion( ClipRect );
-        dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn );
-        dc.DestroyClippingRegion();
-    }
-
-    Rect.x += Rect.width;
-    if( PeakLevel < 80 )
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOff, m_RedOff );
-    }
-    else if( PeakLevel > 90 )
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn );
-    }
-    else
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOff, m_RedOff );
-        wxRect ClipRect = Rect;
-        ClipRect.width = ( PeakLevel - 80 ) * Rect.width / 10;
-        dc.SetClippingRegion( ClipRect );
-        dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn );
-        dc.DestroyClippingRegion();
-    }
-
-    Rect.x += Rect.width;
-    if( PeakLevel < 90 )
-    {
-        dc.SetBrush( m_RedOff );
-        dc.DrawRectangle( Rect.x, 0, Rect.width, Height );
-    }
-    else
-    {
-        dc.SetBrush( m_RedOn );
-        int LevelWidth = ( PeakLevel - 90 ) * Rect.width / 10;
-        dc.DrawRectangle( Rect.x, 0, LevelWidth, Height );
-        dc.SetBrush( m_RedOff );
-        dc.DrawRectangle( Rect.x + LevelWidth, 0, Rect.width - LevelWidth, Height );
-    }
-
-    //
-    // Draw the decay level
-    if( DecayLevel && ( DecayLevel > PeakLevel ) && ( DecayLevel < 99 ) )
-    {
-        wxRect ClipRect = Rect;
+        ClipRect.x = ( DecayLevel * Width ) / 100;;
         ClipRect.width = 2;
-        ClipRect.x = ( DecayLevel * Width ) / 100;
         dc.SetClippingRegion( ClipRect );
-
-        if( DecayLevel < 70 )       // at Solid green
-        {
-            dc.SetBrush( m_GreenOn );
-            dc.DrawRectangle( 0, 0, SizeG * 7, Height );
-        }
-        else if( DecayLevel < 80 )  // at green to orange
-        {
-            Rect.x = SizeG * 7;
-            dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn );
-        }
-        else if( DecayLevel < 90 )  // at orange to red
-        {
-            Rect.x = SizeG * 8;
-            dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn );
-        }
-        else                        // at solid red
-        {
-            dc.SetBrush( m_RedOn );
-            dc.DrawRectangle( Width - Rect.width, 0, Rect.width, Height );
-        }
+        dc.DrawBitmap( * m_OnBitmap, 0, 0, false );
         dc.DestroyClippingRegion();
     }
 
     dc.SetPen( * wxBLACK_PEN );
     dc.DrawText( wxString::Format( wxT( "%02.1f" ), m_PeakLevel ), 2,  ( Height / 2 ) - 8 );
-    //guLogMessage( wxT( "%f" ), m_PeakLevel );
 }
 
 // -------------------------------------------------------------------------------- //
 void guVumeter::PaintVert( void )
 {
 	wxPaintDC dc( this );
-    wxRect Rect;
 	wxCoord Width;
 	wxCoord Height;
 	GetClientSize( &Width, &Height );
@@ -240,151 +174,166 @@ void guVumeter::PaintVert( void )
     //m_PeakLevel = TestValues[ wxGetUTCTime() % 20 ];
     int PeakLevel = IEC_Scale( m_PeakLevel ) * 100;
     int DecayLevel = IEC_Scale( m_DecayLevel ) * 100;
-
-	int SizeG = ( Height * 10 ) / 100;
+    //guLogMessage( wxT( "Peak: %i" ), PeakLevel );
 
     dc.SetPen( * wxTRANSPARENT_PEN );
 
-    Rect.x = 0;
-    Rect.y = Height - ( SizeG * 7 );
-    Rect.width = Width;
-    Rect.height = Height - Rect.y;
+    dc.DrawBitmap( * m_OffBitmap, 0, 0, false );
 
-    //guLogMessage( wxT( "Current Level: %i" ), PeakLevel );
-    if( !PeakLevel )
-    {
-        dc.SetBrush( m_GreenOff );
-        dc.DrawRectangle( 0, Rect.y, Width, Rect.height );
-    }
-    else if( PeakLevel > 70 )
-    {
-        dc.SetBrush( m_GreenOn );
-        dc.DrawRectangle( 0, Rect.y, Width, Rect.height );
-    }
-    else
-    {
-        int LevelHeight = ( Height * PeakLevel ) / 100;
-        dc.SetBrush( m_GreenOff );
-        dc.DrawRectangle( 0, Rect.y, Width, Rect.height - LevelHeight );
-        dc.SetBrush( m_GreenOn );
-        dc.DrawRectangle( 0, Height - LevelHeight, Width, LevelHeight );
-    }
+    wxRect ClipRect;
+    ClipRect.x = 0;
+    ClipRect.width = Width;
+    ClipRect.height = ( PeakLevel * Height ) / 100;
+    ClipRect.y = Height - ClipRect.height;
+    dc.SetClippingRegion( ClipRect );
+    dc.DrawBitmap( * m_OnBitmap, 0, 0, false );
+    dc.DestroyClippingRegion();
 
-    Rect.height = SizeG;
-    Rect.y -= SizeG;
-
-    if( PeakLevel < 70 )
+    if( DecayLevel && ( DecayLevel > PeakLevel ) && ( DecayLevel < 100 ) )
     {
-        dc.GradientFillLinear( Rect, m_GreenOff, m_OrangeOff, wxNORTH );
-    }
-    else if( PeakLevel > 80 )
-    {
-        dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn, wxNORTH );
-    }
-    else
-    {
-        dc.GradientFillLinear( Rect, m_GreenOff, m_OrangeOff, wxNORTH );
-        wxRect ClipRect = Rect;
-        ClipRect.height = ( PeakLevel - 70 ) * SizeG / 10;
-        ClipRect.y += ( SizeG - ClipRect.height );
-        dc.SetClippingRegion( ClipRect );
-        dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn, wxNORTH );
-        dc.DestroyClippingRegion();
-    }
-
-    Rect.y -= SizeG;
-    if( PeakLevel < 80 )
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOff, m_RedOff, wxNORTH );
-    }
-    else if( PeakLevel > 90 )
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn, wxNORTH );
-    }
-    else
-    {
-        dc.GradientFillLinear( Rect, m_OrangeOff, m_RedOff, wxNORTH );
-        wxRect ClipRect = Rect;
-        ClipRect.height = ( PeakLevel - 80 ) * SizeG / 10;
-        ClipRect.y += ( SizeG - ClipRect.height );
-        dc.SetClippingRegion( ClipRect );
-        dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn, wxNORTH );
-        dc.DestroyClippingRegion();
-    }
-
-    Rect.height = Rect.y;
-    //Rect.y = 0;
-    if( PeakLevel < 90 )
-    {
-        dc.SetBrush( m_RedOff );
-        dc.DrawRectangle( 0, 0, Width, Rect.height );
-    }
-    else
-    {
-        dc.SetBrush( m_RedOff );
-        int LevelHeight = ( PeakLevel - 90 ) * Rect.height / 10;
-        dc.DrawRectangle( 0, 0, Width, Rect.height - LevelHeight );
-        dc.SetBrush( m_RedOn );
-        dc.DrawRectangle( 0, Rect.height - LevelHeight, Width, LevelHeight );
-    }
-
-
-    // Draw the decay level
-    Rect.height = SizeG * 7;
-    Rect.y = Height - Rect.height;
-    if( DecayLevel && ( DecayLevel > PeakLevel ) && ( DecayLevel < 99 ) )
-    {
-        wxRect ClipRect = Rect;
         ClipRect.height = 2;
-        ClipRect.y = Height - ( ( Height * DecayLevel ) / 100 );
+        ClipRect.y = Height - ( DecayLevel * Height ) / 100;
         dc.SetClippingRegion( ClipRect );
-
-        if( DecayLevel < 70 )       // at Solid green
-        {
-            dc.SetBrush( m_GreenOn );
-            dc.DrawRectangle( 0, Rect.y, Width, Rect.height );
-        }
-        else if( DecayLevel < 80 )  // at green to orange
-        {
-            Rect.y -= SizeG;
-            Rect.height = SizeG;
-            dc.GradientFillLinear( Rect, m_GreenOn, m_OrangeOn, wxNORTH );
-        }
-        else if( DecayLevel < 90 )  // at orange to red
-        {
-            Rect.y -= SizeG * 2;
-            Rect.height = SizeG;
-            //Rect.y = SizeG + Rect.width;
-            dc.GradientFillLinear( Rect, m_OrangeOn, m_RedOn, wxNORTH );
-        }
-        else                        // at solid red
-        {
-            dc.SetBrush( m_RedOn );
-            dc.DrawRectangle( 0, 0, Width, SizeG );
-        }
+        dc.DrawBitmap( * m_OnBitmap, 0, 0, false );
         dc.DestroyClippingRegion();
     }
 
     dc.SetPen( * wxBLACK_PEN );
     dc.DrawRotatedText( wxString::Format( wxT( "%02.1f" ), m_PeakLevel ), ( Width / 2 ) - 7,  Height - 2, 90 );
-    //guLogMessage( wxT( "%f" ), m_PeakLevel );
 }
 
 
+// -------------------------------------------------------------------------------- //
+void guVumeter::DrawHVumeter( wxBitmap * bitmap, int width, int height, wxColour &green, wxColour &orange, wxColour &red )
+{
+    if( bitmap && bitmap->IsOk() )
+    {
+        wxRect Rect;
+        wxMemoryDC MemDC;
+        MemDC.SelectObject( * bitmap );
+
+        MemDC.SetPen( * wxTRANSPARENT_PEN );
+
+        Rect.x = 0;
+        Rect.y = 0;
+        Rect.width = ( 84 * width ) / 100;
+        Rect.height = height;
+
+        MemDC.SetBrush( green );
+        MemDC.DrawRectangle( 0, 0, Rect.width, height );
+
+        Rect.x = Rect.width;
+        Rect.width = ( 8 * width ) / 100;
+
+        MemDC.GradientFillLinear( Rect, green, orange );
+
+        Rect.x += Rect.width;
+        Rect.width = width - Rect.x;
+
+        MemDC.GradientFillLinear( Rect, orange, red );
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guVumeter::DrawVVumeter( wxBitmap * bitmap, int width, int height, wxColour &green, wxColour &orange, wxColour &red )
+{
+    if( bitmap && bitmap->IsOk() )
+    {
+        wxRect Rect;
+        wxMemoryDC MemDC;
+        MemDC.SelectObject( * bitmap );
+
+        MemDC.SetPen( * wxTRANSPARENT_PEN );
+
+        Rect.x = 0;
+        Rect.width = width;
+        Rect.y = ( height * 16 ) / 100;
+        Rect.height = height - Rect.y;
+
+        MemDC.SetBrush( green );
+        MemDC.DrawRectangle( 0, Rect.y, width, Rect.height );
+
+        Rect.height = ( 8 * height ) / 100;
+        Rect.y -= Rect.height;
+
+        MemDC.GradientFillLinear( Rect, green, orange, wxNORTH );
+
+        Rect.height = Rect.y;
+        Rect.y = 0;
+
+        MemDC.GradientFillLinear( Rect, orange, red, wxNORTH );
+    }
+}
+
+// -------------------------------------------------------------------------------- //
+void guVumeter::RefreshBitmaps( void )
+{
+    wxMutexLocker Locker( m_BitmapMutex );
+    if( m_OnBitmap )
+        delete m_OnBitmap;
+
+    if( m_OffBitmap )
+        delete m_OffBitmap;
+
+	wxCoord Width;
+	wxCoord Height;
+	GetClientSize( &Width, &Height );
+
+	//guLogMessage( wxT( "RefreshBitmaps %i  %i " ), Width, Height );
+
+    if( m_Style == guVU_HORIZONTAL )
+    {
+        m_OffBitmap = new wxBitmap( Width, Height, -1 );
+        DrawHVumeter( m_OffBitmap, Width, Height, m_GreenOff, m_OrangeOff, m_RedOff );
+        m_OnBitmap = new wxBitmap( Width, Height, -1 );
+        DrawHVumeter( m_OnBitmap, Width, Height, m_GreenOn, m_OrangeOn, m_RedOn );
+    }
+    else
+    {
+        m_OffBitmap = new wxBitmap( Width, Height, -1 );
+        DrawVVumeter( m_OffBitmap, Width, Height, m_GreenOff, m_OrangeOff, m_RedOff );
+        m_OnBitmap = new wxBitmap( Width, Height, -1 );
+        DrawVVumeter( m_OnBitmap, Width, Height, m_GreenOn, m_OrangeOn, m_RedOn );
+    }
+}
+
 // -------------------------------------------------------------------------------- //
 void guVumeter::OnPaint( wxPaintEvent &WXUNUSED(event) )
 {
+    wxMutexLocker Locker( m_BitmapMutex );
     if( m_Style == guVU_HORIZONTAL )
         PaintHoriz();
     else
         PaintVert();
 }
 
+// -------------------------------------------------------------------------------- //
+void guVumeter::OnChangedSize( wxSizeEvent &event )
+{
+    wxSize Size = event.GetSize();
 
+    if( ( Size.GetWidth() != m_LastWidth ) || ( Size.GetHeight() != m_LastHeight ) )
+    {
+        m_LastWidth = Size.GetWidth();
+        m_LastHeight = Size.GetHeight();
+
+        RefreshBitmaps();
+    }
+    event.Skip();
+}
+
+
+
+
+
+// -------------------------------------------------------------------------------- //
+// guPlayerVumeters
 // -------------------------------------------------------------------------------- //
 guPlayerVumeters::guPlayerVumeters( wxWindow * parent ) :
     wxPanel( parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL )
 {
+    m_LastWidth = wxNOT_FOUND;
+    m_LastHeight = wxNOT_FOUND;
 	wxFont CurrentFont = wxSystemSettings::GetFont( wxSYS_SYSTEM_FONT );
 
 	this->SetMinSize( wxSize( -1, 40 ) );
@@ -453,7 +402,7 @@ guPlayerVumeters::guPlayerVumeters( wxWindow * parent ) :
 	wxStaticText * HRedLabel = new wxStaticText( this, wxID_ANY, wxT("-3"), wxDefaultPosition, wxDefaultSize, 0 );
 	HRedLabel->Wrap( -1 );
 	HRedLabel->SetFont( CurrentFont );
-	HLabelsSizer->Add( HRedLabel, 10, wxALIGN_BOTTOM, 5 );
+	HLabelsSizer->Add( HRedLabel, 7, wxALIGN_BOTTOM, 5 );
 
 	wxStaticText * HClipLabel = new wxStaticText( this, wxID_ANY, wxT("0"), wxDefaultPosition, wxDefaultSize, 0 );
 	HClipLabel->Wrap( -1 );
@@ -503,7 +452,7 @@ guPlayerVumeters::guPlayerVumeters( wxWindow * parent ) :
 	wxStaticText * VClipLabel = new wxStaticText( this, wxID_ANY, wxT("0"), wxDefaultPosition, wxDefaultSize, 0 );
 	VClipLabel->Wrap( -1 );
 	VClipLabel->SetFont( CurrentFont );
-	VLabelsSizer->Add( VClipLabel, 10, wxALIGN_BOTTOM|wxALIGN_RIGHT|wxTOP, 5 );
+	VLabelsSizer->Add( VClipLabel, 8, wxALIGN_BOTTOM|wxALIGN_RIGHT|wxTOP, 5 );
 
 	wxStaticText * VRedLabel = new wxStaticText( this, wxID_ANY, wxT("-3"), wxDefaultPosition, wxDefaultSize, 0 );
 	VRedLabel->Wrap( -1 );
@@ -595,33 +544,40 @@ guPlayerVumeters::~guPlayerVumeters()
 // -------------------------------------------------------------------------------- //
 void guPlayerVumeters::OnChangedSize( wxSizeEvent &event )
 {
-    bool Changed = false;
     wxSize Size = event.GetSize();
-    //guLogMessage( wxT( "%i, %i" ), Size.GetWidth(), Size.GetHeight() );
-    if( Size.GetWidth() >= Size.GetHeight() )
-    {
-        if( m_VumMainSizer->IsShown( m_VVumFlexSizer ) )
-        {
-            m_VumMainSizer->Hide( m_VVumFlexSizer );
-            m_VumMainSizer->Show( m_HVumFlexSizer );
-            m_VumLeft = m_HVumLeft;
-            m_VumRight = m_HVumRight;
-            Changed = true;
-        }
-    }
-    else if( m_VumMainSizer->IsShown( m_HVumFlexSizer ) )
-    {
-        m_VumMainSizer->Hide( m_HVumFlexSizer );
-        m_VumMainSizer->Show( m_VVumFlexSizer );
-        m_VumLeft = m_VVumLeft;
-        m_VumRight = m_VVumRight;
-        Changed = true;
-    }
 
-    if( Changed )
+    if( ( Size.GetWidth() != m_LastWidth ) || ( Size.GetHeight() != m_LastHeight ) )
     {
-        m_VumMainSizer->Layout();
-        //m_VumMainSizer->FitInside( this );
+        bool ChangedOrientation = false;
+
+        m_LastWidth = Size.GetWidth();
+        m_LastHeight = Size.GetHeight();
+
+        //guLogMessage( wxT( "%i, %i" ), Size.GetWidth(), Size.GetHeight() );
+        if( m_LastWidth >= m_LastHeight )
+        {
+            if( m_VumMainSizer->IsShown( m_VVumFlexSizer ) )
+            {
+                m_VumMainSizer->Hide( m_VVumFlexSizer );
+                m_VumMainSizer->Show( m_HVumFlexSizer );
+                m_VumLeft = m_HVumLeft;
+                m_VumRight = m_HVumRight;
+                ChangedOrientation = true;
+            }
+        }
+        else if( m_VumMainSizer->IsShown( m_HVumFlexSizer ) )
+        {
+            m_VumMainSizer->Hide( m_HVumFlexSizer );
+            m_VumMainSizer->Show( m_VVumFlexSizer );
+            m_VumLeft = m_VVumLeft;
+            m_VumRight = m_VVumRight;
+            ChangedOrientation = true;
+        }
+
+        if( ChangedOrientation )
+        {
+            m_VumMainSizer->Layout();
+        }
     }
     event.Skip();
 }
