@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -25,6 +25,7 @@
 DBusHandlerResult Handle_Messages( DBusConnection * conn, DBusMessage * msg, void * udata )
 {
 	guDBusServer * DBusObj = ( guDBusServer * ) udata;
+	//guDBusMessage * Msg = new guDBusMessage( msg );
 	guDBusMessage * Msg = new guDBusMessage( msg );
 	guDBusMethodReturn * Reply = NULL;
 	if( Msg->NeedReply() )
@@ -33,6 +34,12 @@ DBusHandlerResult Handle_Messages( DBusConnection * conn, DBusMessage * msg, voi
 	}
 	DBusHandlerResult RetVal = DBusObj->HandleMessages( Msg, Reply );
 	//printf( "*** End of handle_messages ***\n" );
+
+	delete Msg;
+
+	if( Reply )
+        delete Reply;
+
  	return RetVal;
 }
 
@@ -47,8 +54,6 @@ void Handle_Response( DBusPendingCall * PCall, void * udata )
     DBusObj->HandleMessages( Msg, NULL );
 
     delete Msg;
-
-    //delete Msg;
 	dbus_message_unref( reply );
 	dbus_pending_call_unref( PCall );
 }
@@ -113,6 +118,12 @@ DBusConnection * guDBusServer::GetConnection()
     return m_DBusConn;
 }
 
+// -------------------------------------------------------------------------------- //
+bool guDBusServer::HasOwner( const char * name )
+{
+    return dbus_bus_name_has_owner( m_DBusConn, name, NULL );
+}
+
 // -------------------------------------------------------------------------------- //
 bool guDBusServer::RequestName( const char * name )
 {
@@ -144,16 +155,17 @@ DBusHandlerResult guDBusServer::HandleMessages( guDBusMessage * msg, guDBusMessa
         if( m_Clients[ index ]->HandleMessages( msg, reply ) == DBUS_HANDLER_RESULT_HANDLED )
         {
             RetVal = DBUS_HANDLER_RESULT_HANDLED;
-            m_ClientsMutex.Unlock();
-            break;
         }
         m_ClientsMutex.Unlock();
+
+        if( RetVal == DBUS_HANDLER_RESULT_HANDLED )
+            break;
     }
 
-    delete msg;
-
-    if( reply )
-        delete reply;
+//    delete msg;
+//
+//    if( reply )
+//        delete reply;
 
     return RetVal;
 }
@@ -216,8 +228,8 @@ guDBusMessage * guDBusServer::SendWithReplyAndBlock( guDBusMessage * msg, int ti
 
     DBusMessage * Result = dbus_connection_send_with_reply_and_block( m_DBusConn, msg->GetMessage(), timeout, &error );
     Reply =  Result ? new guDBusMessage( Result ) : NULL;
-    if( Result )
-        dbus_message_unref( Result );
+//    if( Result )
+//        dbus_message_unref( Result );
     return Reply;
 }
 
@@ -272,6 +284,12 @@ guDBusClient::~guDBusClient()
     UnRegisterClient();
 }
 
+// -------------------------------------------------------------------------------- //
+bool guDBusClient::HasOwner( const char * name )
+{
+    return m_DBusServer->HasOwner( name );
+}
+
 // -------------------------------------------------------------------------------- //
 bool guDBusClient::RegisterClient( void )
 {
@@ -361,23 +379,26 @@ guDBusMessage::guDBusMessage( int type )
 // -------------------------------------------------------------------------------- //
 guDBusMessage::guDBusMessage( guDBusMessage * msg )
 {
-    wxASSERT( msg );
-    m_DBusMsg = dbus_message_copy( msg->GetMessage() );
+    //m_DBusMsg = dbus_message_copy( msg->GetMessage() );
+    m_DBusMsg = msg->GetMessage();
+    dbus_message_ref( m_DBusMsg );
 }
 
 // -------------------------------------------------------------------------------- //
 guDBusMessage::guDBusMessage( DBusMessage * msg )
 {
-    wxASSERT( msg );
-    m_DBusMsg = dbus_message_copy( msg );
-    //m_DBusMsg = msg;
+    //m_DBusMsg = dbus_message_copy( msg );
+    m_DBusMsg = msg;
+    dbus_message_ref( msg );
 }
 
 // -------------------------------------------------------------------------------- //
 guDBusMessage::~guDBusMessage()
 {
     if( m_DBusMsg )
+    {
         dbus_message_unref( m_DBusMsg );
+    }
 }
 
 // -------------------------------------------------------------------------------- //
@@ -512,9 +533,9 @@ const char * guDBusMessage::GetObjectPath()
 // -------------------------------------------------------------------------------- //
 // guDBusMethodCall
 // -------------------------------------------------------------------------------- //
-guDBusMethodCall::guDBusMethodCall( const char * dest, const char * path, const char *iface, const char * method ) :
-  guDBusMessage( dbus_message_new_method_call( dest, path, iface, method ) )
+guDBusMethodCall::guDBusMethodCall( const char * dest, const char * path, const char *iface, const char * method )
 {
+    m_DBusMsg = dbus_message_new_method_call( dest, path, iface, method );
 }
 
 // -------------------------------------------------------------------------------- //
@@ -537,12 +558,6 @@ guDBusSignal::guDBusSignal( const char * path, const char * iface, const char *
     m_DBusMsg = dbus_message_new_signal( path, iface, name );
 }
 
-// -------------------------------------------------------------------------------- //
-guDBusSignal::~guDBusSignal()
-{
-    //m_DBusMsg = NULL;
-}
-
 // -------------------------------------------------------------------------------- //
 // guDBusThread
 // -------------------------------------------------------------------------------- //
