@@ -1,10 +1,10 @@
 // -------------------------------------------------------------------------------- //
-//	Copyright (C) 2008-2010 J.Rios
+//	Copyright (C) 2008-2011 J.Rios
 //	anonbeat@gmail.com
 //
 //    This Program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2, or (at your option)
+//    the Free Software Foundation; either version 3, or (at your option)
 //    any later version.
 //
 //    This Program is distributed in the hope that it will be useful,
@@ -138,6 +138,11 @@ const char * Introspection_XML_Data_Tracklist =
 "    <method name=\"SetRandom\">\n"
 "      <arg type=\"b\" direction=\"in\" />\n"
 "    </method>\n"
+"    <method name=\"AddTracks\">\n"
+"      <arg type=\"as\" direction=\"in\" />\n"
+"      <arg type=\"b\" direction=\"in\" />\n"
+"      <arg type=\"i\" direction=\"out\" />\n"
+"    </method>\n"
 "    <signal name=\"TrackListChange\">\n"
 "      <arg type=\"i\" />\n"
 "    </signal>\n"
@@ -227,8 +232,8 @@ void FillMetadataArgs( guDBusMessage * reply, const guTrack * CurTrack )
     if( CurTrack->m_Year )
         FillMetadataDetails( &dict, metadata_names[ 9 ], ( const int ) CurTrack->m_Year );
 
-//    if( !CurTrack->m_CoverPath.IsEmpty() )
-//        FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_CoverPath ).mb_str( wxConvUTF8 ) );
+    //if( !CurTrack->m_CoverPath.IsEmpty() )
+    //    FillMetadataDetails( &dict, metadata_names[ 10 ], ( const char * ) ( wxT( "file://" ) + CurTrack->m_CoverPath ).mb_str( wxConvUTF8 ) );
 
     if( CurTrack->m_Rating )
         FillMetadataDetails( &dict, metadata_names[ 11 ], ( const int ) CurTrack->m_Bitrate );
@@ -282,25 +287,26 @@ void FillMetadataArgs( guDBusMessage * reply, const guCurrentTrack * CurTrack )
 // -------------------------------------------------------------------------------- //
 DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage * reply )
 {
-    wxASSERT( msg );
-//    // Show the details of the msg
-//    printf( "Type   : %i\n", msg->GetType() );
-//    printf( "Iface  : %s\n", msg->GetInterface() );
-//    printf( "Path   : %s\n", msg->GetPath() );
-//    printf( "Member : %s\n", msg->GetMember() );
-//    printf( "Sender : %s\n", msg->GetSender() );
-//    printf( "Reply  : %i\n", msg->NeedReply() );
-//    printf( "Serial : %i\n", msg->GetSerial() );
-//    if( reply )
-//      printf( "RSerial: %i\n", reply->GetReplySerial() );
-//    printf( "==MPRIS========================\n" );
-
     //
     DBusHandlerResult RetVal = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     const char *    Interface = msg->GetInterface();
     const char *    Member = msg->GetMember();
+//    const char *    Dest = msg->GetDestination();
     int             Type = msg->GetType();
     const char *    Path = msg->GetPath();
+//    int             Serial = msg->GetSerial();
+//    int             RSerial = msg->GetReplySerial();
+
+//    // Show the details of the msg
+//    guLogMessage( wxT( "==MPRIS========================" ) );
+//    guLogMessage( wxT( "Type   : %i" ), Type );
+//    guLogMessage( wxT( "Iface  : %s" ), wxString::FromAscii( Interface ).c_str() );
+//    guLogMessage( wxT( "Dest   : %s" ), wxString::FromAscii( Dest ).c_str() );
+//    guLogMessage( wxT( "Path   : %s" ), wxString::FromAscii( Path ).c_str() );
+//    guLogMessage( wxT( "OPath  : %s" ), wxString::FromAscii( msg->GetObjectPath() ).c_str() );
+//    guLogMessage( wxT( "Member : %s" ), wxString::FromAscii( Member ).c_str() );
+//    guLogMessage( wxT( "Serial : %i" ), Serial );
+//    guLogMessage( wxT( "RSerial: %i" ), RSerial );
 
     // If its a method call
     if( Type == DBUS_MESSAGE_TYPE_METHOD_CALL )
@@ -313,49 +319,51 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
         if( !strcmp( Interface, "org.freedesktop.DBus.Introspectable" ) &&
             !strcmp( Member, "Introspect" ) )
         {
-            if( !strcmp( Path, "/" ) )
+            const char *    Dest = msg->GetDestination();
+            if( Dest && !strcmp( Dest, GUAYADEQUE_MPRIS_SERVICENAME ) )
             {
-                DBusMessageIter args;
-                dbus_message_iter_init_append( reply->GetMessage(), &args );
-
-                if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Root ) )
+                if( !strcmp( Path, "/" ) )
                 {
-                    guLogError( wxT( "Failed to attach the Introspection info" ) );
-                }
-                Send( reply );
-                Flush();
-                RetVal = DBUS_HANDLER_RESULT_HANDLED;
-            }
-            else if( !strcmp( Path, "/Player" ) )
-            {
-                DBusMessageIter args;
-                dbus_message_iter_init_append( reply->GetMessage(), &args );
+                    DBusMessageIter args;
+                    dbus_message_iter_init_append( reply->GetMessage(), &args );
 
-                if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Player ) )
-                {
-                    guLogError( wxT( "Failed to attach the Introspection info" ) );
+                    if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Root ) )
+                    {
+                        guLogError( wxT( "Failed to attach the Introspection info" ) );
+                    }
+                    Send( reply );
+                    Flush();
+                    RetVal = DBUS_HANDLER_RESULT_HANDLED;
                 }
-                Send( reply );
-                Flush();
-                RetVal = DBUS_HANDLER_RESULT_HANDLED;
-            }
-            else if( !strcmp( Path, "/TrackList" ) )
-            {
-                DBusMessageIter args;
-                dbus_message_iter_init_append( reply->GetMessage(), &args );
+                else if( !strcmp( Path, "/Player" ) )
+                {
+                    DBusMessageIter args;
+                    dbus_message_iter_init_append( reply->GetMessage(), &args );
 
-                if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Tracklist ) )
+                    if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Player ) )
+                    {
+                        guLogError( wxT( "Failed to attach the Introspection info" ) );
+                    }
+                    Send( reply );
+                    Flush();
+                    RetVal = DBUS_HANDLER_RESULT_HANDLED;
+                }
+                else if( !strcmp( Path, "/TrackList" ) )
                 {
-                    guLogError( wxT( "Failed to attach the Introspection info" ) );
+                    DBusMessageIter args;
+                    dbus_message_iter_init_append( reply->GetMessage(), &args );
+
+                    if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_STRING, &Introspection_XML_Data_Tracklist ) )
+                    {
+                        guLogError( wxT( "Failed to attach the Introspection info" ) );
+                    }
+                    Send( reply );
+                    Flush();
+                    RetVal = DBUS_HANDLER_RESULT_HANDLED;
                 }
-                Send( reply );
-                Flush();
-                RetVal = DBUS_HANDLER_RESULT_HANDLED;
             }
         }
-
-
-        if( !strcmp( Interface, "org.freedesktop.MediaPlayer" ) )
+        else if( !strcmp( Interface, "org.freedesktop.MediaPlayer" ) )
         {
             //
             // ROOT
@@ -462,8 +470,9 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                 }
                 else if( !strcmp( Member, "Repeat" ) )
                 {
-                    wxCommandEvent event;
-                    m_PlayerPanel->OnRepeatPlayButtonClick( event );
+                    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_SETREPEAT );
+                    wxPostEvent( m_PlayerPanel, event );
+
                     Send( reply );
                     Flush();
                     RetVal = DBUS_HANDLER_RESULT_HANDLED;
@@ -537,7 +546,9 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     }
                     else
                     {
-                        m_PlayerPanel->SetVolume( Volume );
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_SETVOLUME );
+                        event.SetInt( Volume );
+                        wxPostEvent( m_PlayerPanel, event );
 
                         Send( reply );
                         Flush();
@@ -675,6 +686,8 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                           DBUS_TYPE_BOOLEAN, &PlayTrack,
                           DBUS_TYPE_INVALID );
 
+                    //guLogMessage( wxT( "MPRIS: AddTrack\n%s\n%i" ), wxString( TrackPath, wxConvUTF8 ).c_str(), PlayTrack );
+
                     if( dbus_error_is_set( &error ) )
                     {
                         printf( "Could not read the AddTrack parameters : %s\n", error.message );
@@ -686,28 +699,64 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                         DBusMessageIter args;
                         dbus_message_iter_init_append( reply->GetMessage(), &args );
 
-                        //printf( "AddTrack: %s\n", TrackPath );
+                        wxArrayString * TrackList = new wxArrayString();
+                        TrackList->Add( wxString( TrackPath, wxConvUTF8 ) );
 
-                        if( PlayTrack )
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_ADDTRACKS );
+                        event.SetInt( PlayTrack );
+                        event.SetClientData( TrackList );
+                        wxPostEvent( m_PlayerPanel, event );
+
+                        int TrackAdded = 1;
+                        if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_INT32, &TrackAdded ) )
                         {
-                            m_PlayerPanel->ClearPlayList();
-
-                            if( m_PlayerPanel->GetState() != guMEDIASTATE_STOPPED )
-                            {
-                                wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_STOP );
-                                wxPostEvent( m_PlayerPanel, event );
-                            }
+                            guLogError( wxT( "Failed to attach the AddTrack return code" ) );
                         }
 
-                        m_PlayerPanel->AddToPlayList( wxString( TrackPath, wxConvUTF8 ) );
+                        Send( reply );
+                        Flush();
+                        RetVal = DBUS_HANDLER_RESULT_HANDLED;
+                    }
+                }
+                else if( !strcmp( Member, "AddTracks" ) )
+                {
+                    DBusError error;
+                    dbus_error_init( &error );
 
-                        if( PlayTrack )
+                    char **         Tracks;
+                    dbus_int32_t    TracksCount;
+                    dbus_bool_t     PlayTrack;
+
+                    dbus_message_get_args( msg->GetMessage(), &error,
+                          DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &Tracks, &TracksCount,
+                          DBUS_TYPE_BOOLEAN, &PlayTrack,
+                          DBUS_TYPE_INVALID );
+//
+//                    //guLogMessage( wxT( "MPRIS: AddTrack\n%s\n%i" ), wxString( TrackPath, wxConvUTF8 ).c_str(), PlayTrack );
+//
+                    if( dbus_error_is_set( &error ) )
+                    {
+                        guLogMessage( wxT( "Could not read the AddTracks parameters : '%s'" ), wxString( error.message, wxConvUTF8 ).c_str() );
+                        dbus_error_free( &error );
+                        RetVal =  DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+                    }
+                    else
+                    {
+                        DBusMessageIter args;
+                        dbus_message_iter_init_append( reply->GetMessage(), &args );
+
+                        wxArrayString * TrackList = new wxArrayString();
+                        int Index;
+                        for( Index = 0; Index < TracksCount; Index++ )
                         {
-                            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_PLAY );
-                            m_PlayerPanel->OnNextTrackButtonClick( event );
-                            m_PlayerPanel->OnPlayButtonClick( event );
+                            TrackList->Add( wxString( Tracks[ Index ], wxConvUTF8 ) );
                         }
 
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_ADDTRACKS );
+                        event.SetInt( PlayTrack );
+                        event.SetClientData( TrackList );
+                        wxPostEvent( m_PlayerPanel, event );
+
                         int TrackAdded = 1;
                         if( !dbus_message_iter_append_basic( &args, DBUS_TYPE_INT32, &TrackAdded ) )
                         {
@@ -736,7 +785,10 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     }
                     else
                     {
-                        m_PlayerPanel->RemoveItem( TrackNum );
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_REMOVETRACK );
+                        event.SetInt( TrackNum );
+                        wxPostEvent( m_PlayerPanel, event );
+
                         //Send( reply );
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
@@ -761,7 +813,9 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     }
                     else
                     {
-                        m_PlayerPanel->SetPlayLoop( PlayLoop );
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_SETLOOP );
+                        event.SetInt( PlayLoop );
+                        wxPostEvent( m_PlayerPanel, event );
 
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
@@ -786,8 +840,8 @@ DBusHandlerResult guMPRIS::HandleMessages( guDBusMessage * msg, guDBusMessage *
                     }
                     else
                     {
-                        wxCommandEvent event;
-                        m_PlayerPanel->OnRandomPlayButtonClick( event );
+                        wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_PLAYERPANEL_SETRANDOM );
+                        wxPostEvent( m_PlayerPanel, event );
 
                         Flush();
                         RetVal = DBUS_HANDLER_RESULT_HANDLED;
